<?php
/**
 * Instructor Dashboard Module
 *
 * @since      4.0
 * @package    Instructor_Role
 * @subpackage Instructor_Role/modules/classes
 * @author     WisdmLabs <support@wisdmlabs.com>
 */

namespace InstructorRole\Modules\Classes;

defined( 'ABSPATH' ) || exit;

if ( ! class_exists( 'Instructor_Role_Dashboard' ) ) {
	/**
	 * Class Instructor Role Dashboard Module
	 */
	class Instructor_Role_Dashboard {
		/**
		 * Singleton instance of this class
		 *
		 * @var object  $instance
		 *
		 * @since 4.0
		 */
		protected static $instance = null;

		/**
		 * Plugin Slug
		 *
		 * @var string  $plugin_slug
		 *
		 * @since 4.0
		 */
		protected $plugin_slug = '';

		/**
		 * Constructor
		 */
		public function __construct() {
			$this->plugin_slug = INSTRUCTOR_ROLE_TXT_DOMAIN;
		}

		/**
		 * Get a singleton instance of this class
		 *
		 * @return object
		 * @since   4.0
		 */
		public static function get_instance() {
			if ( null === self::$instance ) {
				self::$instance = new self();
			}
			return self::$instance;
		}

		/**
		 * Enqueue dashboard scripts
		 *
		 * @since 4.0
		 */
		public function enqueue_dashboard_scripts() {
			global $post;

			if ( ! wdm_is_instructor() ) {
				return;
			}
			$layout = ir_get_settings( 'ir_dashboard_layout' );

			if ( 'layout-2' === $layout ) {
				wp_enqueue_style(
					'irb-styles',
					plugins_url( 'css/irb-new-dashboard.css', __DIR__ ),
					array(),
					INSTRUCTOR_ROLE_PLUGIN_VERSION
				);
				wp_enqueue_script(
					'irb-new-dashboard-script',
					plugins_url( 'js/dist/irb-new-dashboard.js', __DIR__ ),
					array( 'ir-lib-apex-charts' )
				);
			} else {
				wp_enqueue_style(
					'irb-styles',
					plugins_url( 'css/irb-admin.css', __DIR__ ),
					array(),
					INSTRUCTOR_ROLE_PLUGIN_VERSION
				);
			}

			$screen = get_current_screen();

			if ( learndash_get_post_type_slug( 'course' ) === $screen->id ) {
				$this->add_course_page_scripts();
			}

		}

		/**
		 * Add course page specific scripts
		 *
		 * @since 4.0
		 */
		public function add_course_page_scripts() {

			wp_enqueue_script(
				'ir-dashboard-course-script',
				plugins_url( 'js/dashboard/ir-dashboard-course-script.js', __DIR__ ),
				array( 'jquery' ),
				filemtime( INSTRUCTOR_ROLE_ABSPATH . 'modules/js/dashboard/ir-dashboard-course-script.js' ),
				1
			);

			$settings = array(
				'course_page',
				'learndash_course_builder',
				'sfwd-courses-settings',
			);

			if ( learndash_get_total_post_count( learndash_get_post_type_slug( 'group' ) ) !== 0 ) {
				$settings[] = 'learndash_course_groups';
			}

			/**
			 * Filter the list of settings for progress bar.
			 *
			 * @since 4.0
			 *
			 * @param array $settings   Settings for progress bar.
			 */
			$settings      = apply_filters( 'ir_filter_progress_bar_settings', $settings );
			$settings_text = sprintf(
			// translators: Total settings count.
				__( '_count_ out of %d settings', 'wdm_instructor_role' ),
				count( $settings )
			);

			wp_localize_script(
				'ir-dashboard-course-script',
				'ir_dashboard_loc',
				array(
					'settings'      => $settings,
					'settings_text' => $settings_text,
					'step_width'    => floatval( 100 / ( count( $settings ) - 1 ) ),
				)
			);

			// wp_enqueue_style(
			// 'ir-dashboard-course-styles',
			// plugins_url( 'css/dashboard/ir-dashboard-course-styles.css', __DIR__ ),
			// array(),
			// filemtime( INSTRUCTOR_ROLE_ABSPATH . '/modules/css/dashboard/ir-dashboard-course-styles.css' )
			// );
		}

		/**
		 * Add save and continue section on course page
		 *
		 * @since 4.0
		 */
		public function add_save_and_continue_section() {
			if ( ! wdm_is_instructor() ) {
				return;
			}

			$layout = ir_get_settings( 'ir_dashboard_layout' );

			// Save and continue is part of layout 1, if not set then lets return.
			if ( 'layout-1' !== $layout ) {
				return;
			}

			$screen = get_current_screen();

			$settings = array(
				'course_page',
				'learndash_course_builder',
				'sfwd-courses-settings',
			);

			if ( learndash_get_total_post_count( learndash_get_post_type_slug( 'group' ) ) !== 0 ) {
				$settings[] = 'learndash_course_groups';
			}

			/**
			 * Filter the list of settings for progress bar.
			 *
			 * @since 4.0
			 *
			 * @param array $settings   Settings for progress bar.
			 */
			$settings = apply_filters( 'ir_filter_progress_bar_settings', $settings );

			$current_setting = 'course_page';
			if ( ! empty( $_GET ) && isset( $_GET['currentTab'] ) ) {
				$current_setting = $_GET['currentTab'];
			}

			/**
			 * Filter the current setting for progress bar.
			 *
			 * @since 4.0
			 *
			 * @param string $current_setting   Currently active setting.
			 */
			$current_setting = apply_filters( 'ir_filter_progress_bar_current_setting', $current_setting );

			$current_setting_count = 0;
			if ( in_array( $current_setting, $settings ) ) {
				$current_setting_count = array_search( $current_setting, $settings );
			}
			$current_setting_count = intval( $current_setting_count ) + 1;

			if ( learndash_get_post_type_slug( 'course' ) === $screen->id ) {
				ir_get_template(
					INSTRUCTOR_ROLE_ABSPATH . '/modules/templates/dashboard/ir-course-footer.template.php',
					array(
						'settings_count'        => count( $settings ),
						'current_setting_count' => $current_setting_count,
						'step_width'            => floatval( 100 / ( count( $settings ) - 1 ) ),
					)
				);
			}
		}

		/**
		 * Add appearance settings tab in Instructor Settings
		 *
		 * @since 4.3.0
		 *
		 * @param array  $tabs          Array of tabs.
		 * @param string $current_tab   Current selected instructor tab.
		 */
		public function add_appearance_settings_tab( $tabs, $current_tab ) {
			// Check if admin.
			if ( ! current_user_can( 'manage_options' ) ) {
				return $tabs;
			}

			// Check if appearance tab already exists.
			if ( ! array_key_exists( 'ir-appearance', $tabs ) ) {
				$tabs['ir-appearance'] = array(
					'title'  => __( 'Appearance', 'wdm_instructor_role' ),
					'access' => array( 'admin' ),
				);
			}
			return $tabs;
		}

		/**
		 * Display Appearance settings for configuring appearance settings.
		 *
		 * @since 4.3.0
		 *
		 * @param string $current_tab   Slug of the selected tab in instructor settings.
		 */
		public function add_appearance_settings_tab_contents( $current_tab ) {
			// Check if admin and appearance tab.
			if ( ! current_user_can( 'manage_options' ) || 'ir-appearance' !== $current_tab ) {
				return;
			}

			$default_fonts    = $this->get_default_fonts();
			$google_fonts     = $this->get_google_fonts();
			$dashboard_layout = ir_get_settings( 'ir_dashboard_layout' );

			if ( empty( $dashboard_layout ) ) {
				$dashboard_layout = 'layout-1';
			}

			ir_get_template(
				INSTRUCTOR_ROLE_ABSPATH . '/modules/templates/dashboard/ir-dashboard-settings-page.template.php',
				array(
					'ir_dashboard_layout'                 => $dashboard_layout,
					'ir_color_preset'                     => ir_get_settings( 'ir_color_preset' ),
					'ir_color_preset_2'                   => ir_get_settings( 'ir_color_preset_2' ),
					'ir_primary_color_1'                  => ir_get_settings( 'ir_primary_color_1' ),
					'ir_primary_color_2'                  => ir_get_settings( 'ir_primary_color_2' ),
					'ir_primary_color_3'                  => ir_get_settings( 'ir_primary_color_3' ),
					'ir_primary_color_4'                  => ir_get_settings( 'ir_primary_color_4' ),
					'ir_primary_color_5'                  => ir_get_settings( 'ir_primary_color_5' ),
					'ir_accent_primary_color'             => ir_get_settings( 'ir_accent_primary_color' ),
					'ir_layout_2_primary_color'           => ir_get_settings( 'ir_layout_2_primary_color' ),
					'ir_layout_2_secondary_color'         => ir_get_settings( 'ir_layout_2_secondary_color' ),
					'ir_layout_2_tertiary_color'          => ir_get_settings( 'ir_layout_2_tertiary_color' ),
					'ir_layout_2_accent_color'            => ir_get_settings( 'ir_layout_2_accent_color' ),
					'ir_layout_2_text_color_1'            => ir_get_settings( 'ir_layout_2_text_color_1' ),
					'ir_layout_2_text_color_2'            => ir_get_settings( 'ir_layout_2_text_color_2' ),
					'ir_layout_2_background_color'        => ir_get_settings( 'ir_layout_2_background_color' ),
					'ir_dashboard_header'                 => ir_get_settings( 'ir_dashboard_header' ),
					'ir_dashboard_logo'                   => ir_get_settings( 'ir_dashboard_logo' ),
					'ir_dashboard_logo_2'                 => ir_get_settings( 'ir_dashboard_logo_2' ),
					'ir_example_logo_2'                   => plugins_url( '/modules/media/ir-example-logo.png', INSTRUCTOR_ROLE_BASE ),
					'ir_logo_alignment_2'                 => ir_get_settings( 'ir_logo_alignment_2' ),
					'ir_dashboard_image_background_color' => ir_get_settings( 'ir_dashboard_image_background_color' ),
					'ir_dashboard_image_background_color_2' => ir_get_settings( 'ir_dashboard_image_background_color_2' ),
					'ir_dashboard_text_sub_title'         => ir_get_settings( 'ir_dashboard_text_sub_title' ),
					'ir_dashboard_text_title'             => ir_get_settings( 'ir_dashboard_text_title' ),
					'ir_dashboard_title_label'            => ir_get_settings( 'ir_dashboard_title_label' ),
					'ir_dashboard_font_family'            => ir_get_settings( 'ir_dashboard_font_family' ),
					'ir_dashboard_title_font_color'       => ir_get_settings( 'ir_dashboard_title_font_color' ),
					'ir_dashboard_sub_title_font_color'   => ir_get_settings( 'ir_dashboard_sub_title_font_color' ),
					'ir_dashboard_text_background_color'  => ir_get_settings( 'ir_dashboard_text_background_color' ),
					'ir_lms_label'                        => ir_get_settings( 'ir_lms_label' ),
					'ir_dashboard_font_size'              => ir_get_settings( 'ir_dashboard_font_size' ),
					'google_fonts'                        => $google_fonts,
					'default_fonts'                       => $default_fonts,
					'recommended_fonts'                   => array(
						'default'  => 'Open Sans',
						'preset_1' => 'Poppins',
						'preset_2' => 'Nunito',
						'preset_3' => 'Solway',
						'preset_4' => 'Open Sans',
					),
				)
			);
		}

		/**
		 * Save appearance settings
		 *
		 * @since 4.3.0
		 */
		public function save_appearance_settings() {
			// If not admin then return.
			if ( ! current_user_can( 'manage_options' ) ) {
				return;
			}

			// Verify nonce.
			if ( ! array_key_exists( 'ir_dashboard_nonce', $_POST ) || ! wp_verify_nonce( filter_input( INPUT_POST, 'ir_dashboard_nonce' ), 'ir_dashboard_settings' ) ) {
				return;
			}

			// Save dashboard settings.
			// Layout 1.
			$ir_color_preset                       = ir_filter_input( 'ir_color_preset' );
			$ir_primary_color_1                    = ir_filter_input( 'ir_primary_color_1' );
			$ir_primary_color_2                    = ir_filter_input( 'ir_primary_color_2' );
			$ir_primary_color_3                    = ir_filter_input( 'ir_primary_color_3' );
			$ir_primary_color_4                    = ir_filter_input( 'ir_primary_color_4' );
			$ir_primary_color_5                    = ir_filter_input( 'ir_primary_color_5' );
			$ir_accent_primary_color               = ir_filter_input( 'ir_accent_primary_color' );
			$ir_dashboard_header                   = ir_filter_input( 'ir_dashboard_header' );
			$ir_dashboard_logo                     = ir_filter_input( 'ir_dashboard_logo' );
			$ir_dashboard_image_background_color   = ir_filter_input( 'ir_dashboard_image_background_color' );
			$ir_dashboard_image_background_color_2 = ir_filter_input( 'ir_dashboard_image_background_color_2' );
			$ir_dashboard_text_title               = ir_filter_input( 'ir_dashboard_text_title' );
			$ir_dashboard_text_sub_title           = ir_filter_input( 'ir_dashboard_text_sub_title' );
			$ir_dashboard_title_font_color         = ir_filter_input( 'ir_dashboard_title_font_color' );
			$ir_dashboard_sub_title_font_color     = ir_filter_input( 'ir_dashboard_sub_title_font_color' );
			$ir_dashboard_text_background_color    = ir_filter_input( 'ir_dashboard_text_background_color' );

			// Layout 2.
			$ir_color_preset_2            = ir_filter_input( 'ir_color_preset_2' );
			$ir_layout_2_primary_color    = ir_filter_input( 'ir_layout_2_primary_color' );
			$ir_layout_2_secondary_color  = ir_filter_input( 'ir_layout_2_secondary_color' );
			$ir_layout_2_tertiary_color   = ir_filter_input( 'ir_layout_2_tertiary_color' );
			$ir_layout_2_accent_color     = ir_filter_input( 'ir_layout_2_accent_color' );
			$ir_layout_2_text_color_1     = ir_filter_input( 'ir_layout_2_text_color_1' );
			$ir_layout_2_text_color_2     = ir_filter_input( 'ir_layout_2_text_color_2' );
			$ir_layout_2_background_color = ir_filter_input( 'ir_layout_2_background_color' );
			$ir_dashboard_logo_2          = ir_filter_input( 'ir_dashboard_logo_2' );
			$ir_logo_alignment_2          = ir_filter_input( 'ir_logo_alignment_2' );
			$ir_dashboard_title_label     = ir_filter_input( 'ir_dashboard_title_label' );
			$ir_dashboard_font_size       = ir_filter_input( 'ir_dashboard_font_size' );

			// Common.
			$ir_dashboard_layout      = ir_filter_input( 'ir_dashboard_layout' );
			$ir_dashboard_font_family = ir_filter_input( 'ir_dashboard_font_family' );
			$ir_lms_label             = ir_filter_input( 'ir_lms_label' );

			ir_set_settings( 'ir_color_preset', $ir_color_preset );
			ir_set_settings( 'ir_color_preset_2', $ir_color_preset_2 );
			ir_set_settings( 'ir_dashboard_layout', $ir_dashboard_layout );
			ir_set_settings( 'ir_primary_color_1', $ir_primary_color_1 );
			ir_set_settings( 'ir_primary_color_2', $ir_primary_color_2 );
			ir_set_settings( 'ir_primary_color_3', $ir_primary_color_3 );
			ir_set_settings( 'ir_primary_color_4', $ir_primary_color_4 );
			ir_set_settings( 'ir_primary_color_5', $ir_primary_color_5 );
			ir_set_settings( 'ir_accent_primary_color', $ir_accent_primary_color );
			ir_set_settings( 'ir_layout_2_primary_color', $ir_layout_2_primary_color );
			ir_set_settings( 'ir_layout_2_secondary_color', $ir_layout_2_secondary_color );
			ir_set_settings( 'ir_layout_2_tertiary_color', $ir_layout_2_tertiary_color );
			ir_set_settings( 'ir_layout_2_accent_color', $ir_layout_2_accent_color );
			ir_set_settings( 'ir_layout_2_text_color_1', $ir_layout_2_text_color_1 );
			ir_set_settings( 'ir_layout_2_text_color_2', $ir_layout_2_text_color_2 );
			ir_set_settings( 'ir_layout_2_background_color', $ir_layout_2_background_color );
			ir_set_settings( 'ir_dashboard_header', $ir_dashboard_header );
			ir_set_settings( 'ir_dashboard_logo', $ir_dashboard_logo );
			ir_set_settings( 'ir_dashboard_logo_2', $ir_dashboard_logo_2 );
			ir_set_settings( 'ir_logo_alignment_2', $ir_logo_alignment_2 );
			ir_set_settings( 'ir_dashboard_image_background_color', $ir_dashboard_image_background_color );
			ir_set_settings( 'ir_dashboard_image_background_color_2', $ir_dashboard_image_background_color_2 );
			ir_set_settings( 'ir_dashboard_text_title', $ir_dashboard_text_title );
			ir_set_settings( 'ir_dashboard_title_label', $ir_dashboard_title_label );
			ir_set_settings( 'ir_dashboard_font_family', $ir_dashboard_font_family );
			ir_set_settings( 'ir_lms_label', $ir_lms_label );
			ir_set_settings( 'ir_dashboard_font_size', $ir_dashboard_font_size );
			ir_set_settings( 'ir_dashboard_text_sub_title', $ir_dashboard_text_sub_title );
			ir_set_settings( 'ir_dashboard_title_font_color', $ir_dashboard_title_font_color );
			ir_set_settings( 'ir_dashboard_sub_title_font_color', $ir_dashboard_sub_title_font_color );
			ir_set_settings( 'ir_dashboard_text_background_color', $ir_dashboard_text_background_color );
		}

		/**
		 * Enqueue scripts for instructor dashboard settings
		 *
		 * @since 4.0
		 */
		public function enqueue_scripts() {
			$screen = get_current_screen();

			// Get current page and tab from URL.
			if ( empty( $_GET ) || ! array_key_exists( 'page', $_GET ) || ! array_key_exists( 'tab', $_GET ) ) {
				return;
			}

			$page = filter_input( INPUT_GET, 'page', FILTER_SANITIZE_FULL_SPECIAL_CHARS );
			$tab  = filter_input( INPUT_GET, 'tab', FILTER_SANITIZE_FULL_SPECIAL_CHARS );

			// Check if instructor settings page and appearance tab.
			if ( 'instuctor' !== $page || 'ir-appearance' !== $tab ) {
				return;
			}

			if ( ! empty( $screen ) && 'learndash-lms_page_instuctor' === $screen->id ) {
				wp_enqueue_style(
					'ir-dashboard-settings-styles',
					plugins_url( 'css/dashboard/ir-dashboard-settings-styles.css', __DIR__ ),
					array(),
					filemtime( INSTRUCTOR_ROLE_ABSPATH . '/modules/css/dashboard/ir-dashboard-settings-styles.css' )
				);

				wp_enqueue_style( 'wp-color-picker' );
				wp_enqueue_script(
					'ir-dashboard-settings-script',
					plugins_url( 'js/dashboard/ir-dashboard-settings-script.js', __DIR__ ),
					array( 'jquery', 'wp-color-picker' ),
					filemtime( INSTRUCTOR_ROLE_ABSPATH . '/modules/js/dashboard/ir-dashboard-settings-script.js' ),
					1
				);
				wp_localize_script(
					'ir-dashboard-settings-script',
					'ir_loc_data',
					array(
						'media_title'  => __( 'Select Image', 'wdm_instructor_role' ),
						'media_button' => __( 'Use this image', 'wdm_instructor_role' ),
					)
				);
			}
		}

		/**
		 * Get google fonts in an array
		 *
		 * @since 4.0
		 */
		public function get_google_fonts() {
			$google_fonts = get_option( 'ir_google_fonts' );
			if ( false === $google_fonts ) {
				$google_fonts = array();
				$font_json    = file_get_contents( INSTRUCTOR_ROLE_ABSPATH . '/modules/media/google-fonts.json' );
				$font_decoded = json_decode( $font_json );
				$fonts        = $font_decoded->items;

				foreach ( $fonts as $key => $value ) {
					array_push( $google_fonts, $value->family );
				}
				update_option( 'ir_google_fonts', $google_fonts );
			}

			return apply_filters( 'ir_filter_google_fonts', $google_fonts );
		}

		/**
		 * Get default web-safe font list
		 *
		 * @since 4.3.0
		 *
		 * @return array    Array of web-safe font list.
		 */
		public function get_default_fonts() {
			$default_fonts = array(
				// 'Arial'               => 'arial, sans-serif',
				'Helvetica'           => 'helvetica, sans-serif',
				'Verdana'             => 'verdana, sans-serif',
				'Tahoma'              => 'Tahoma, sans-serif',
				'Trebuchet'           => 'trebuchet ms, sans-serif',
				'Gill Sans'           => 'gill sans, sans-serif',
				'Times New Roman'     => 'times new roman, serif',
				'Georgia'             => 'georgia, serif',
				'Palatino'            => 'palatino, serif',
				'Baskerville'         => 'baskerville, serif',
				'Courier'             => 'courier, monospace',
				'Monaco'              => 'monaco, monospace',
				'Bradley Hand'        => 'bradley hand, cursive',
				'Luminari'            => 'luminari, fantasy',
				'Comic Sans MS'       => 'comic sans ms, cursive',
				'Optima'              => 'optima, sans-serif',
				'Didot'               => 'didot, serif',
				'American Typewriter' => 'american typewriter, serif',
			);

			return apply_filters( 'ir_filter_get_default_fonts', $default_fonts );
		}

		/**
		 * Add instructor logo on dashboard.
		 *
		 * @since 4.0
		 */
		public function add_instructor_logo() {
			// Check if instructor.
			if ( ! wdm_is_instructor() ) {
				return;
			}
			// Get active layout.
			$layout                     = ir_get_settings( 'ir_dashboard_layout' );
			$dashboard_background_color = '';

			// Get logo settings.
			if ( 'layout-2' === $layout ) {
				$logo_image_url             = $this->get_new_dashboard_logo( ir_get_settings( 'ir_dashboard_logo_2' ) );
				$ir_logo_alignment          = ir_get_settings( 'ir_logo_alignment_2' );
				$dashboard_background_color = ir_get_settings( 'ir_dashboard_image_background_color_2' );

				/**
				 * A url to logo on the instructor dashboard.
				 *
				 * @since 3.5.6
				 *
				 * @param string A URL to the instructor dashboard logo.
				 */
				$logo_url = apply_filters( 'ir_instructor_dashboard_logo_url', '' );

				$args = array(
					'dashboard_logo' => $logo_image_url,
					'logo_url'       => $logo_url,
				);
			} else {
				$header_type = ir_get_settings( 'ir_dashboard_header' );
				if ( 'image' === $header_type ) {
					$ir_dashboard_logo          = ir_get_settings( 'ir_dashboard_logo' );
					$dashboard_background_color = ir_get_settings( 'ir_dashboard_image_background_color' );
					$logo_image_url             = ! empty( $ir_dashboard_logo ) ? esc_attr( wp_get_attachment_image_src( $ir_dashboard_logo )[0] ) : '';
					/**
					 * A url to logo on the instructor dashboard.
					 *
					 * @since 3.5.6
					 *
					 * @param string A URL to the instructor dashboard logo.
					 */
					$logo_url = apply_filters( 'ir_instructor_dashboard_logo_url', '' );

					$args = array(
						'dashboard_logo' => $logo_image_url,
						'logo_url'       => $logo_url,
					);
				} elseif ( 'text' === $header_type ) {
					$ir_dashboard_text_title     = ir_get_settings( 'ir_dashboard_text_title' );
					$ir_dashboard_text_sub_title = ir_get_settings( 'ir_dashboard_text_sub_title' );

					// $ir_dashboard_title_font_family = ir_get_settings( 'ir_dashboard_title_font_family' );
					$ir_dashboard_title_font_color = ir_get_settings( 'ir_dashboard_title_font_color' );

					/**
					 * Customize instructor logo title styles.
					 *
					 * @param string $styles    Styles applied to the logo title.
					 */
					$title_font_styles = apply_filters(
						'ir_filter_logo_title_styles',
						"font-family: 'Poppins', sans-serif;
						font-size: 26px;
						text-align: center;
						padding: 15px;
						color: {$ir_dashboard_title_font_color};"
					);
					// $ir_dashboard_sub_title_font_family = ir_get_settings( 'ir_dashboard_sub_title_font_family' );
					$ir_dashboard_sub_title_font_color = ir_get_settings( 'ir_dashboard_sub_title_font_color' );
					/**
					 * Customize instructor logo subtitle styles.
					 *
					 * @param string $styles    Styles applied to the logo subtitle.
					 */
					$subtitle_font_styles = apply_filters(
						'ir_filter_logo_title_styles',
						"font-family: 'Poppins', sans-serif;
						font-size: 12px;
						text-align: center;
						color: {$ir_dashboard_sub_title_font_color};"
					);

					$dashboard_background_color = ir_get_settings( 'ir_dashboard_text_background_color' );

					$args = array(
						'text_title'           => $ir_dashboard_text_title,
						'text_sub_title'       => $ir_dashboard_text_sub_title,
						'title_font_styles'    => $title_font_styles,
						'subtitle_font_styles' => $subtitle_font_styles,
					);
				}
			}

			if ( 'layout-2' === $layout ) {
				$template_path           = INSTRUCTOR_ROLE_ABSPATH . '/modules/templates/dashboard/ir-dashboard-header-layout-2.template.php';
				$dashboard_title         = ir_get_settings( 'ir_dashboard_title_label' );
				$args['dashboard_title'] = empty( $dashboard_title ) ? __( 'Instructor Dashboard', 'wdm_instructor_role' ) : $dashboard_title;
				$args['logo_alignment']  = empty( $ir_logo_alignment ) ? 'center' : $ir_logo_alignment;
			} else {
				$args['header_type'] = $header_type;
				$template_path       = INSTRUCTOR_ROLE_ABSPATH . '/modules/templates/dashboard/ir-dashboard-header.template.php';
			}

			$dashboard_header = ir_get_template(
				$template_path,
				$args,
				true
			);

			// Apply changes through JS.
			?>
				<script type="text/javascript">
					jQuery(document).ready(function($) {
						$('#adminmenu').prepend(`<?php echo $dashboard_header; ?>`);
						<?php if ( ! empty( $dashboard_background_color ) ) : ?>
							$('.ir-admin-logo').css('background', "<?php echo esc_attr( $dashboard_background_color ); ?>");
						<?php endif; ?>
					});
				</script>
			<?php
		}

		/**
		 * Add accent color styles to instructor dashboard.
		 *
		 * @since 4.0
		 */
		public function add_accent_color_styles() {
			// Check if instructor.
			if ( is_user_logged_in() && wdm_is_instructor() ) {
				$layout               = ir_get_settings( 'ir_dashboard_layout' );
				$enable_color_schemes = ir_get_settings( 'enable_color_schemes' );
				$preset_colors        = array();

				// Check which layout is selected.
				if ( 'layout-2' === $layout ) {
					$ir_color_preset = ir_get_settings( 'ir_color_preset_2' );

					// If default preset, then no need for custom colors.
					if ( 'default' === $ir_color_preset ) {
						return;
					}

					// Get preset colors.
					$preset_colors = self::get_preset_colors( $ir_color_preset, 2 );

					$template_path = INSTRUCTOR_ROLE_ABSPATH . '/modules/templates/dashboard/ir-accent-color-layout-2-styles.template.php';

				} else {
					$ir_color_preset = ir_get_settings( 'ir_color_preset' );

					// If default preset, then no need for custom colors.
					if ( 'default' === $ir_color_preset ) {
						return;
					}

					// Get preset colors.
					$preset_colors = self::get_preset_colors( $ir_color_preset );

					$template_path = INSTRUCTOR_ROLE_ABSPATH . '/modules/templates/dashboard/ir-accent-color-styles.template.php';

				}

				$custom_styling = ir_get_template(
					$template_path,
					$preset_colors,
					1
				);
				wp_add_inline_style( 'irb-styles', $custom_styling );
			}
		}

		/**
		 * Exclude menu items from the list of post types to restrict access to.
		 *
		 * @since 4.0
		 *
		 * @param array $post_types     Array of post types to be excluded.
		 *
		 * @return array                Updated array of excluded post types.
		 */
		public function exclude_menu_items( $post_types ) {
			if ( wdm_is_instructor() ) {
				$post_types[] = 'nav_menu_item';
			}
			return $post_types;
		}

		/**
		 * Remove default admin bar for instructors
		 *
		 * @since 4.0
		 */
		public function remove_admin_bar() {
			if ( is_user_logged_in() && wdm_is_instructor() ) {
				remove_action( 'in_admin_header', 'wp_admin_bar_render', 0 );
				$layout = ir_get_settings( 'ir_dashboard_layout' );
				if ( 'layout-2' !== $layout ) {
					?>
					<script type="text/javascript">
						jQuery(document).ready(function($) {
							$('#adminmenu').prepend(`<?php echo $this->get_toggle_menu_button(); ?>`);
						});
					</script>
					<?php
				}
			}
		}

		/**
		 * Add dashboard menu for instructors
		 *
		 * @since 4.0
		 */
		public function add_instructor_menu() {
			if ( wdm_is_instructor() ) {
				$primary_menu_slug = $this->ir_set_primary_menu_handle();

				if ( has_nav_menu( $primary_menu_slug ) ) {
					echo wp_nav_menu(
						array(
							'theme_location'  => $primary_menu_slug,
							'menu_id'         => 'ir-primary-menu',
							'container_id'    => 'ir-primary-navigation',
							'container_class' => '',
							'echo'            => false,
						)
					);
				} else {
					/**
					 * Allow third party plugins to update default instructor menu.
					 *
					 * @since 3.3.0
					 */
					$template = apply_filters(
						'ir_filter_default_instructor_menu_path',
						INSTRUCTOR_ROLE_ABSPATH . '/modules/templates/settings/ir-default-instructor-menu.template.php'
					);
					include $template;
					// $this->ir_display_fallback_menu();
				}
			}
		}
		/**
		 * Function gets the name of the theme
		 *
		 * @return string $theme_name
		 * @since 4.2.1
		 */
		public function ir_get_theme_name() {
			$theme_data = wp_get_theme();
			$theme_name = $theme_data->Name;
			return $theme_name;
		}

		/**
		 * Function to set primary menu handle
		 * Description : If the user has created a menu and assigned it to the "Instructor Role Menu" location, then use
		 * that menu. Otherwise, use the default menu for the theme
		 *
		 * @return string $primary_menu_handle
		 * @since 4.2.1
		 */
		public function ir_set_primary_menu_handle() {
			$theme_name        = $this->ir_get_theme_name();
			$compatible_themes = array(
				'Kadence'           => 'primary',
				'Astra'             => 'primary',
				'BuddyBoss Theme'   => 'header-menu',
				'eLumine'           => 'primary',
				'Twenty Twenty'     => 'primary',
				'Twenty Twenty-One' => 'primary',
			);

			/**
			 * Allow 3rd party plugins to alter $compatible_themes variable
			 *
			 * @since 4.2.1
			 */
			$compatible_themes = apply_filters( 'ir_filter_compatible_themes', $compatible_themes );

			// get all nav menus
			$nav_menus = get_nav_menu_locations();
			if ( isset( $nav_menus['ir-instructor-menu'] ) && 0 !== $nav_menus['ir-instructor-menu'] && ! empty( $nav_menus['ir-instructor-menu'] ) ) {
				$primary_menu_handle = 'ir-instructor-menu';
			} else {
				// check if element exist in the array
				if ( array_key_exists( $theme_name, $compatible_themes ) ) {
					$primary_menu_handle = $compatible_themes[ $theme_name ];
				} else {
					$primary_menu_handle = '';
				}
			}
			return $primary_menu_handle;
		}

		/**
		 * Render fall back menu if no menu is set
		 *
		 * @since 4.2.1
		 */
		public function ir_display_fallback_menu( array $args = array() ) {
			$latest        = new \WP_Query(
				array(
					'post_type'      => 'page',
					'orderby'        => 'menu_order title',
					'order'          => 'ASC',
					'posts_per_page' => 5,
				)
			);
			$page_ids      = wp_list_pluck( $latest->posts, 'ID' );
			$page_ids      = implode( ',', $page_ids );
			$fallback_args = array(
				'depth'       => -1,
				'include'     => $page_ids,
				'show_home'   => false,
				'before'      => '',
				'after'       => '',
				'menu_id'     => 'ir-primary-menu',
				'menu_class'  => 'ir-primary-menu',
				'container'   => 'ul',
				'fallback_cb' => 'wp_page_menu',
			);
			?>
			<div id="ir-primary-navigation" class="menu-test-container ir-default-menu">
			<?php
				wp_page_menu( $fallback_args );
			?>
			</div>
			<?php

		}
		/**
		 * Render persistant menu items into the dashboard menu.
		 *
		 * @since 4.2.1
		 */
		public function render_persistant_menu_items( $items ) {
			if ( is_admin() ) {

				$home_url = home_url();
				if ( isset( $_COOKIE['wdm_ir_old_user'] ) ) {
					$switch_back = '<li class="switch-back menu-item menu-item-type-custom menu-item-object-custom"><a href=' . $home_url . '/wp-login.php?action=wdm_ir_switchback_user >' . esc_html__( 'Switch to Admin', 'wdm_instructor_role' ) . '</a></li>';
				} else {
					$switch_back = '';
				}

				return $items .= $switch_back . '
				<li class="menu-item menu-item-type-custom menu-item-object-custom"><a href="' . $home_url . '">' . esc_html__( 'Exit Dashboard', 'wdm_instructor_role' ) . '</a></li>';

			} elseif ( wdm_is_instructor() && ir_get_settings( 'wdm_id_ir_dash_pri_menu' ) == 'on' ) {
				// instructor dashboard link
				$admin_url     = admin_url();
				return $items .= '<li class="menu-item menu-item-type-custom menu-item-object-custom"><a href="' . $admin_url . 'admin.php?page=ir_instructor_overview">' . esc_html__( 'Instructor Dashboard', 'wdm_instructor_role' ) . '</a></li>';
			} else {

				return $items;

			}
		}

		/**
		 * Get toggle menu button.
		 *
		 * @since 4.0
		 */
		public function get_toggle_menu_button() {
			$toggle_menu_button = '
			<li id="collapse-menu" class="hide-if-no-js">
				<button type="button" id="collapse-button" aria-label="' . esc_attr__( 'Collapse Main menu' ) . '" aria-expanded="true">
					<span class="irb-icon-side-bar-expand"></span>
				</button>
			</li>';

			/**
			 * Filter the toggle menu button HTML.
			 *
			 * @since 4.0
			 */
			$toggle_menu_button = apply_filters( 'ir_filter_toggle_menu_html', $toggle_menu_button );

			return $toggle_menu_button;
		}

		/**
		 * Instructor Dashboard footer text.
		 *
		 * @since 4.0
		 */
		public function dashboard_footer_text() {
			return __return_empty_string();
		}

		/**
		 * Get preset colors
		 *
		 * @param string $ir_color_preset    The preset key.
		 * @param int    $layout             The layout key. Default set to 1.
		 *
		 * @return array                     List of preset colors.
		 */
		public static function get_preset_colors( $ir_color_preset, $layout = 1 ) {
			$preset_colors = array();

			if ( 2 === $layout ) {
				switch ( $ir_color_preset ) {
					case 'preset_1':
						$preset_colors['primary']      = '#0051F9';
						$preset_colors['secondary']    = '#0F1D3A';
						$preset_colors['tertiary']     = '#03102C';
						$preset_colors['accent']       = '#F26440';
						$preset_colors['text_color_1'] = '#0F2B65';
						$preset_colors['text_color_2'] = '#C4C4C4';
						$preset_colors['background']   = '#E8EBF3';
						break;

					case 'preset_2':
						$preset_colors['primary']      = '#FD9C0F';
						$preset_colors['secondary']    = '#2B1E43';
						$preset_colors['tertiary']     = '#201239';
						$preset_colors['accent']       = '#D757A7';
						$preset_colors['text_color_1'] = '#2C1D7B';
						$preset_colors['text_color_2'] = '#E5E5E5';
						$preset_colors['background']   = '#F7F7F7';
						break;

					case 'preset_3':
						$preset_colors['primary']      = '#E339D8';
						$preset_colors['secondary']    = '#FFDFEE';
						$preset_colors['tertiary']     = '#FFFFFF';
						$preset_colors['accent']       = '#FC9618';
						$preset_colors['text_color_1'] = '#220B6D';
						$preset_colors['text_color_2'] = '#220B6D';
						$preset_colors['background']   = '#F7F7F7';
						break;

					case 'preset_4':
						$preset_colors['primary']      = '#F45E55';
						$preset_colors['secondary']    = '#F4F2EB';
						$preset_colors['tertiary']     = '#FFFFFF';
						$preset_colors['accent']       = '#C89D2A';
						$preset_colors['text_color_1'] = '#22302A';
						$preset_colors['text_color_2'] = '#003722';
						$preset_colors['background']   = '#FFFEF9';
						break;

					case 'custom':
						$preset_colors['primary']      = ir_get_settings( 'ir_layout_2_primary_color' );
						$preset_colors['secondary']    = ir_get_settings( 'ir_layout_2_secondary_color' );
						$preset_colors['tertiary']     = ir_get_settings( 'ir_layout_2_tertiary_color' );
						$preset_colors['accent']       = ir_get_settings( 'ir_layout_2_accent_color' );
						$preset_colors['text_color_1'] = ir_get_settings( 'ir_layout_2_text_color_1' );
						$preset_colors['text_color_2'] = ir_get_settings( 'ir_layout_2_text_color_2' );
						$preset_colors['background']   = ir_get_settings( 'ir_layout_2_background_color' );
						break;

					default:
						$preset_colors['primary']      = '#2067FA';
						$preset_colors['secondary']    = '#FFF1DC';
						$preset_colors['tertiary']     = '#FFFFFF';
						$preset_colors['accent']       = '#F26440';
						$preset_colors['text_color_1'] = '#021768';
						$preset_colors['text_color_2'] = '#021768';
						$preset_colors['background']   = '#F5F3EC';
						break;
				}
			} else {
				switch ( $ir_color_preset ) {
					case 'default':
						break;

					case 'preset_1':
						$preset_colors['primary']      = '#272847';
						$preset_colors['secondary']    = '#1a1b35';
						$preset_colors['tertiary']     = '#363868';
						$preset_colors['accent']       = '#4553e6';
						$preset_colors['text_color_1'] = '#fff';
						$preset_colors['text_color_2'] = '#ddd';
						break;

					case 'preset_2':
						$preset_colors['primary']      = '#1b4332';
						$preset_colors['secondary']    = '#2d6a4f';
						$preset_colors['tertiary']     = '#40916c';
						$preset_colors['accent']       = '#95d5b2';
						$preset_colors['text_color_1'] = '#fff';
						$preset_colors['text_color_2'] = '#74c69d';
						break;

					case 'preset_3':
						$preset_colors['primary']      = '#caf0f8';
						$preset_colors['secondary']    = '#ade8f4';
						$preset_colors['tertiary']     = '#90e0ef';
						$preset_colors['accent']       = '#0096c7';
						$preset_colors['text_color_1'] = '#023e8a';
						$preset_colors['text_color_2'] = '#03045e';
						break;

					case 'custom':
						$preset_colors['primary']      = ir_get_settings( 'ir_primary_color_1' );
						$preset_colors['secondary']    = ir_get_settings( 'ir_primary_color_2' );
						$preset_colors['tertiary']     = ir_get_settings( 'ir_primary_color_3' );
						$preset_colors['accent']       = ir_get_settings( 'ir_accent_primary_color' );
						$preset_colors['text_color_1'] = ir_get_settings( 'ir_primary_color_4' );
						$preset_colors['text_color_2'] = ir_get_settings( 'ir_primary_color_5' );
						break;
				}
			}

			/**
			 * Filter preset color list
			 *
			 * @since 4.1.1
			 *
			 * @param array $preset_colors      List of preset colors.
			 * @param string $ir_color_preset   Configured preset option value.
			 * @param int $layout               The layout being used.
			 */
			return apply_filters( 'ir_filter_preset_colors', $preset_colors, $ir_color_preset, $layout );
		}

		/**
		 * Add font settings for instructor dashboard
		 *
		 * @since 4.3.0
		 */
		public function add_font_settings() {
			// Check if instructor.
			if ( is_user_logged_in() && wdm_is_instructor() ) {
				$layout = ir_get_settings( 'ir_dashboard_layout' );

				// Check if layout 2.
				if ( 'layout-2' !== $layout ) {
					return;
				}

				$font_family = ir_get_settings( 'ir_dashboard_font_family' );
				$font_size   = ir_get_settings( 'ir_dashboard_font_size' );

				$custom_styles = '';

				$default_fonts = array_values( $this->get_default_fonts() );

				// Check and apply font family.
				if ( ! empty( $font_family ) ) {
					$custom_styles .= "
						body ,body select,body .button, button,.ld__builder--app, .ld__builder-sidebar-widget, .ld-global-header h1, optgroup, #ir-primary-menu *, tbody , table th,table td,table tr, .wdm-button, input, textarea, .apexcharts-canvas *, label *, .components-snackbar-list *{
							font-family: {$font_family} !important;
						}
					";
					if ( ! in_array( $font_family, $default_fonts, true ) && -1 !== $font_family ) {
						wp_enqueue_style(
							'ir-google-fonts',
							'https://fonts.googleapis.com/css?family=' . rawurlencode( $font_family ),
							array(),
							strtotime( 'today' )
						);
					}
				}

				// Check and apply Body font size.
				if ( ! empty( $font_size ) && 'large' === $font_size ) {
					$custom_styles .= '
					.wp-menu-name, .wp-menu-image{
						font-size: 18px;
					}
					#adminmenu .wp-submenu a, #adminmenu .wp-submenu li.current a{
						font-size: 16px;
					}
					.irbn-overview-wrap .irbn-overview > h1{
						font-size: 30px;
					}
					.irbn-overview-wrap .irbn-overview .irbn-tiles-wrap .irbn-tile .irbn-tile-right .irbn-tile-value{
						font-size: 42px;
					}
					.irbn-overview-wrap .irbn-overview .irbn-tiles-wrap .irbn-tile .irbn-tile-right .irbn-text{
						font-size: 22px;
					}
					.irbn-overview-wrap .irbn-overview .irbn-charts .irbn-tile .irbn-tile-header, .irbn-overview-wrap .irbn-overview .irbn-sub .irbn-tile .irbn-tile-header{
						font-size: 22px;
					}
					.irbn-overview-wrap select#instructor-courses-select{
						font-size: 16px;
					}
					.irbn-overview-wrap .irbn-overview .irbn-sub .ir-assignment-table-header th, .irbn-overview-wrap .irbn-overview .irbn-sub .ir-assignment-table-body td, .irbn-overview-wrap .irbn-overview .irbn-sub .ir-assignment-table-body td a{
						font-size: 16px;
					}
					.dataTables_wrapper{
						font-size: 16px;
					}
					.no-reports-message{
						font-size: 16px;
					}
					';
				}

				// Add styles.
				if ( ! empty( $custom_styles ) ) {
					wp_add_inline_style( 'irb-styles', $custom_styles );
				}
			}
		}

		/**
		 * Hide LD tab buttons on listing pages.
		 *
		 * @since 4.3.0
		 */
		public function hide_sections_on_dashboard() {
			$screen = get_current_screen();

			if ( ! function_exists( 'learndash_get_post_type_slug' ) ) {
				return;
			}

			$screen_list = array(
				'edit-' . learndash_get_post_type_slug( 'course' ),
				'edit-' . learndash_get_post_type_slug( 'lesson' ),
				'edit-' . learndash_get_post_type_slug( 'topic' ),
				'edit-' . learndash_get_post_type_slug( 'quiz' ),
				'edit-' . learndash_get_post_type_slug( 'question' ),
				'edit-' . learndash_get_post_type_slug( 'certificate' ),
				'edit-' . learndash_get_post_type_slug( 'group' ),
				'groups_page_group_admin_page',
				'edit-' . learndash_get_post_type_slug( 'exam' ),
				'edit-' . learndash_get_post_type_slug( 'assignment' ),
				'edit-' . learndash_get_post_type_slug( 'essay' ),
			);

			if ( ! in_array( $screen->id, $screen_list ) ) {
				return;
			}
			$custom_styles = '
				#sfwd-header div.ld-tab-buttons {
					display: none;
				}
				#sfwd-header {
					margin-bottom: 30px;
				}
			';

			wp_add_inline_style( 'irb-styles', $custom_styles );
		}

		/**
		 * Update LMS Label on instructor dashboard.
		 *
		 * @since 4.3.0
		 */
		public function update_instructor_lms_label() {
			$ir_lms_label = ir_get_settings( 'ir_lms_label' );

			// Check if label set and is instructor.
			if ( empty( $ir_lms_label ) || ! wdm_is_instructor() ) {
				return;
			}
			global $menu;

			foreach ( $menu as $key => $menu_item ) {
				// Compare menu item slug with learndash slug.
				if ( 'learndash-lms' === $menu_item[2] ) {
					// Update label.
					$menu[ $key ][0] = esc_html( $ir_lms_label );
				}
			}
		}

		/**
		 * Add mobile icons for instructor dashboard
		 *
		 * @since 4.3.0
		 *
		 * @param string $nav_menu  The HTML content for the navigation menu.
		 * @param object $args      Nav menu arguments.
		 *
		 * @return array            Updated HTML for navigation menu.
		 */
		public function add_mobile_icons_to_dashboard( $nav_menu, $args ) {
			// Check if admin side.
			if ( ! is_admin() ) {
				return $nav_menu;
			}
			// Check if instructor.
			if ( ! wdm_is_instructor() ) {
				return $nav_menu;
			}

			// Check if instructor dashboard menu.
			if ( ! is_object( $args ) || 'ir-primary-navigation' !== $args->container_id ) {
				return $nav_menu;
			}

			$menu_icons_html = '<div class="wdm-mob-menu wdm-admin-menu-show wdm-hidden">
				<span class="dashicons dashicons-menu-alt"></span>
			</div>
			<div class="ir-mob-dashboard-menu">
				<span class="dashicons dashicons-ellipsis"></span>
			</div>';

			// Check if menu object exists.
			if ( ! property_exists( $args, 'menu' ) || ! $args->menu instanceof \WP_Term ) {
				return $nav_menu;
			}

			/**
			 * Filter the menu icons added to the dashboard navigation HTML.
			 *
			 * @since 4.3.0
			 *
			 * @param string $menu_icons_html   The HTML content for the menu icons to be added.
			 * @param string $nav_menu          The HTML content for the navigation menu.
			 * @param object $args              Nav menu arguments.
			 */
			$menu_icons_html = apply_filters( 'ir_filter_mobile_icons_nav_menu_html', $menu_icons_html, $nav_menu, $args );
			$search_string   = '<div id="ir-primary-navigation" class="menu-' . $args->menu->slug . '-container">';
			$nav_menu        = str_replace( $search_string, $search_string . $menu_icons_html, $nav_menu );

			return $nav_menu;
		}

		/**
		 * Add layout classes to the body tag
		 *
		 * @since 4.3.0
		 *
		 * @param string $classes    Space-separated list of CSS classes.
		 * @return string            Updated space-separated list of CSS classes.
		 */
		public function add_layout_classes_to_body( $classes ) {
			if ( ! wdm_is_instructor() ) {
				return $classes;
			}
			$layout = ir_get_settings( 'ir_dashboard_layout' );
			if ( 'layout-2' === $layout && false === strpos( $classes, 'dashboard-layout-2' ) ) {
				$classes .= ' dashboard-layout-2 ';
			} else {
				$classes .= ' dashboard-layout-1 ';
			}
			return $classes;
		}

		/**
		 * Get dashboard logo for new layouts
		 *
		 * @since 4.3.0
		 *
		 * @param string $new_dashboard_logo    Saved setting for new layout dashboard logo.
		 * @return string
		 */
		public function get_new_dashboard_logo( $new_dashboard_logo ) {
			$dashboard_logo_url = '';

			// Check if logo set.
			if ( empty( $new_dashboard_logo ) ) {
				// Set example logo if not set yet.
				$unset_logo = get_option( 'ir_unset_example_logo' );
				if ( false === $unset_logo ) {
					$dashboard_logo_url = plugins_url( '/modules/media/ir-example-logo.png', INSTRUCTOR_ROLE_BASE );
				}
			} else {
				$dashboard_logo_url = esc_attr( wp_get_attachment_image_src( $new_dashboard_logo, 'full' )[0] );
				update_option( 'ir_unset_example_logo', 1 );
			}
			return $dashboard_logo_url;
		}

		/**
		 * Set the layouts to the new layout for new customers.
		 *
		 * @since 4.3.0
		 */
		public function set_new_layouts() {
			$layout = ir_get_settings( 'ir_dashboard_layout' );

			if ( 'layout-2' !== $layout ) {
				ir_set_settings( 'ir_dashboard_layout', 'layout-2' );
			}
		}

		/**
		 * Add frontend dashboard settings tab in Instructor Settings
		 *
		 * @since 4.4.0
		 *
		 * @param array  $tabs          Array of tabs.
		 * @param string $current_tab   Current selected instructor tab.
		 */
		public function add_frontend_dashboard_settings_tab( $tabs, $current_tab ) {
			// Check if admin.
			if ( ! current_user_can( 'manage_options' ) ) {
				return $tabs;
			}

			// Check if frontend_dashboard tab already exists.
			if ( ! array_key_exists( 'ir-frontend-dashboard', $tabs ) ) {
				$tabs['ir-frontend-dashboard'] = array(
					'title'  => sprintf( /* translators: Course Label */__( 'Frontend %s Creator', 'wdm_instructor_role' ), \LearnDash_Custom_Label::get_label( 'course' ) ),
					'access' => array( 'admin' ),
				);
			}
			return $tabs;
		}

		/**
		 * Display settings for configuring frontend dashboard settings.
		 *
		 * @since 4.3.0
		 *
		 * @param string $current_tab   Slug of the selected tab in instructor settings.
		 */
		public function add_frontend_dashboard_settings_tab_contents( $current_tab ) {
			// Check if admin and frontend dashboard tab.
			if ( ! current_user_can( 'manage_options' ) || 'ir-frontend-dashboard' !== $current_tab ) {
				return;
			}

			ir_get_template(
				INSTRUCTOR_ROLE_ABSPATH . '/modules/templates/frontend-dashboard/ir-frontend-dashboard-settings-page.template.php',
				array(
					'ir_enable_frontend_dashboard' => ir_get_settings( 'ir_enable_frontend_dashboard' ),
					'ir_disable_ld_links'          => ir_get_settings( 'ir_disable_ld_links' ),
					'image_url'                    => plugins_url( '/images/frontend-course-creator-settings.png', __DIR__ ),
					'admin_image_url'              => plugins_url( '/images/frontend-course-creator-admin-settings.png', __DIR__ ),
				)
			);
		}

		/**
		 * Save frontend dashboard settings
		 *
		 * @since 4.4.0
		 */
		public function save_frontend_dashboard_settings() {
			// If not admin then return.
			if ( ! current_user_can( 'manage_options' ) ) {
				return;
			}

			// Verify nonce.
			if ( ! array_key_exists( 'ir_nonce', $_POST ) || ! wp_verify_nonce( filter_input( INPUT_POST, 'ir_nonce' ), 'ir_frontend_dashboard_settings' ) ) {
				return;
			}

			global $wp_rewrite;

			// Save dashboard settings.
			$ir_enable_frontend_dashboard = ir_filter_input( 'ir_enable_frontend_dashboard' );
			$ir_disable_ld_links          = empty( ir_filter_input( 'ir_disable_ld_links' ) ) ? 'off' : 'on';

			ir_set_settings( 'ir_enable_frontend_dashboard', $ir_enable_frontend_dashboard );
			ir_set_settings( 'ir_disable_ld_links', $ir_disable_ld_links );

			// flush_rewrite_rules.
			if ( ir_get_settings( 'ir_enable_frontend_dashboard' ) && get_option( 'permalink_structure' ) === '/%postname%/' ) {
				$wp_rewrite->set_permalink_structure( '/%postname%/' );

				add_rewrite_rule(
					'^course-builder/([\d]+)/?$',
					'index.php?course_builder=true&ir_course=$matches[1]',
					'top'
				);

				add_rewrite_tag( '%course_builder%', '([^&]+)' );

				add_rewrite_rule(
					'^quiz-builder/([\d]+)/?$',
					'index.php?quiz_builder=true&ir_quiz=$matches[1]',
					'top'
				);

				add_rewrite_tag( '%quiz_builder%', '([^&]+)' );

				add_rewrite_rule(
					'^instructor/([^/]+)/?$',
					'index.php?author_name=$matches[1]&{ir_instructor_profile}=true',
					'top'
				);

				flush_rewrite_rules();
			}
		}
	}
}
