/**
 * React hook that is used to mark the block wrapper element.
 * It provides all the necessary props like the class name.
 *
 * @see https://developer.wordpress.org/block-editor/reference-guides/packages/packages-block-editor/#useblockprops
 */
import { useBlockProps } from '@wordpress/block-editor';
import apiFetch from '@wordpress/api-fetch';
import { useState, useEffect } from 'react';
import { Irswitch } from '/images/switch.png';

/**
 * Retrieves the translation of text.
 *
 * @see https://developer.wordpress.org/block-editor/reference-guides/packages/packages-i18n/
 */
import { sprintf, __ } from "@wordpress/i18n";

/**
 * Mantine Components
 */

import {
	AppShell,
	Navbar,
	Header,
	Accordion,
	MantineProvider,
	Text,
	Title,
	Grid,
	Paper,
	Input,
	Checkbox,
	Button,
	Group,
	Space,
	Center,
	CloseButton,
	Skeleton,
	TextInput,
	Modal,
	Tabs,
	FileButton,
	Switch,
	Radio,
	NumberInput,
	MultiSelect,
	Loader,
	Divider,
	Popover,
	Image,
	Tooltip,
	Select,
	Badge,
	HoverCard,
	Textarea,
	Anchor
} from '@mantine/core';
import { TimeInput, DatePicker } from '@mantine/dates';
import { ModalsProvider } from '@mantine/modals';
import { openConfirmModal, closeAllModals } from '@mantine/modals';
import { useForm } from '@mantine/form';
import { showNotification, updateNotification, NotificationsProvider } from '@mantine/notifications';
import { InfoCircle, Tex } from 'tabler-icons-react';
import { Plus } from 'tabler-icons-react';
import { GridDots } from 'tabler-icons-react';
import { LetterL } from 'tabler-icons-react';
import { LetterT } from 'tabler-icons-react';
import { LetterQ } from 'tabler-icons-react';
import { PencilMinus } from 'tabler-icons-react';
import { X } from 'tabler-icons-react';
import { ChevronDown } from 'tabler-icons-react';
import { ChevronUp } from 'tabler-icons-react';
import { Search } from 'tabler-icons-react';
import { ArrowBackUp } from 'tabler-icons-react';
import { Upload } from 'tabler-icons-react';
import { Trash } from 'tabler-icons-react';
import { XboxX } from 'tabler-icons-react';
import { Eye } from 'tabler-icons-react';
import { ChevronLeft } from 'tabler-icons-react';
import { DeviceDesktop, DeviceTablet, DeviceMobile, ArrowsMaximize, ArrowsMinimize, PhotoUp, Replace } from 'tabler-icons-react';
import { CircleCheck, ExclamationMark, ExternalLink, ClockHour4, CalendarEvent } from 'tabler-icons-react';

import EditModule from '/src/components/editModule.js';
import AddSection from '/src/components/addSection.js';
import AddQuiz from '/src/components/addQuiz.js';
import EditSection from '/src/components/editSection.js';
import CoursePage from '/src/components/coursePage.js';
import CourseSettings from '/src/components/courseSettings.js';
import CourseGroups from '/src/components/courseGroups.js';

import { MediaUpload } from '@wordpress/media-utils';

/**
 * React Beautiful DnD Components
 */
// import { DragDropContext, Droppable, Draggable } from 'react-beautiful-dnd';

/**
 * The save function defines the way in which the different attributes should
 * be combined into the final markup, which is then serialized by the block
 * editor into `post_content`.
 *
 * @see https://developer.wordpress.org/block-editor/reference-guides/block-api/block-edit-save/#save
 *
 * @return {WPElement} Element to render.
 */

export default function save() {
	const course_id = getCourseId();
	const all_courses_link = document.getElementsByClassName('ir-course-builder-container')[0].dataset.allCoursesLink;
	const [courseData, setCourseData] = useState(null);
	const [courseList, setCourseList] = useState(null);
	const [lessonList, setLessonList] = useState([]);
	const [topicList, setTopicList] = useState([]);
	const [quizList, setQuizList] = useState([]);
	const [loading, setLoading] = useState(false);
	const [drafting, setDrafting] = useState(false);
	const [lessons_loading, setLessonsLoading] = useState(false);
	const [topics_loading, setTopicsLoading] = useState(false);
	const [quiz_loading, setQuizLoading] = useState(false);
	const [selectedLessons, setSelectedLessons] = useState([]);
	const [selectedTopics, setSelectedTopics] = useState([]);
	const [selectedQuizzes, setSelectedQuizzes] = useState([]);
	const [course_title, setCourseTitle] = useState('draft');
	const [expanded, setExpanded] = useState(false);
	const [lessonEditorRefresh, setLessonEditorRefresh] = useState(0);
	const [topicEditorRefresh, setTopicEditorRefresh] = useState(0);
	const [lessonOpened, setLessonOpened] = useState(false);
	const [checkedLessons, setCheckedLessons] = useState(false);
	const [checkedTopics, setCheckedTopics] = useState(false);
	const [checkedQuizzes, setCheckedQuizzes] = useState(false);
	const [authors, setAuthors] = useState([]);
	const [review_data, setReviewData] = useState([]);

	const [instructorSettings, setInstructorSettings] = useState({
		category_hide_check: 'on',
		category_restrict_check: '',
		enable_buy_pricing: 'on',
		enable_closed_pricing: 'on',
		enable_free_pricing: 'on',
		enable_open_pricing: 'on',
		enable_permalinks: 'on',
		enable_recurring_pricing: 'on'
	});

	const courseform = useForm({
		initialValues: {
			title: '',
			content: '<p></p>',
			featured_image_url: null,
			featured_image: null,
			status: 'draft',
			course_users: null,
			course_groups: null,
			ld_course_category: [],
			prerequisites: [],
			shared_instructors: [],
			author: 0,
			materials: '<p></p>',
			is_published: false,
			visibility: 'public',
			password: ''
		},
		validate: {
			title: (value) => ((value.length > 0) ? null : sprintf(__('Please enter a valid %s title', 'wdm_instructor_role'), ir_fcb_loc.lower_course_label)),
		}
	})


	/**
	 * Get Course ID.
	 * 
	 * @since   4.4.0
	 *
	 * @returns int ID of the Course.
	 */
	function getCourseId() {
		let path = window.location.pathname;
		path = path.replace(/^\/+/g, '');
		path = path.replace(/\/+$/g, '');
		const path_segments = Object.values(path.split('/'));
		return parseInt(path_segments.pop());
	}

	/**
	 * Extract the course builder data from DOM.
	 *
	 * @since   4.4.0
	 *
	 * @param {Object} course_builder The course builder object
	 * @returns Updated course builder object.
	 */
	const extractBuilderData = (course_builder) => {
		// List of lessons.
		jQuery('.ir-builder .ir-builder-lesson').each((ind, lesson) => {
			course_builder["sfwd-lessons"][jQuery(lesson).val()] = {
				"sfwd-topic": {},
				"sfwd-quiz": {},
				"order": ind
			};

			// List of lesson topics.
			jQuery(lesson).parents('.ir-fed-lessons').siblings('.mantine-Accordion-panel').find('.ir-builder-lesson-topic').each((ind, topic) => {
				course_builder["sfwd-lessons"][jQuery(lesson).val()]["sfwd-topic"][jQuery(topic).val()] = {
					"sfwd-quiz": {},
					"order": ind
				};

				// List of topic quizzes.
				jQuery(topic).parents('.ir-topics-acc-top').siblings('.mantine-Accordion-panel').find('.ir-topic-quizzes .ir-builder-quiz').each((ind, topic_quiz) => {
					course_builder["sfwd-lessons"][jQuery(lesson).val()]["sfwd-topic"][jQuery(topic).val()]["sfwd-quiz"][jQuery(topic_quiz).val()] = {
						"order": ind
					};
				});
			});

			// List of lesson quizzes.
			jQuery(lesson).parents('.ir-fed-lessons').siblings('.mantine-Accordion-panel').find('.ir-lesson-quizzes > .ir-fed-quizzes').each((ind, lesson_quiz) => {
				course_builder["sfwd-lessons"][jQuery(lesson).val()]["sfwd-quiz"][jQuery(lesson_quiz).find('.ir-builder-quiz').val()] = {
					"order": ind
				};
			});
		});

		// List of quizzes.
		jQuery('.ir-builder .ir-fed-final-quizzes .ir-builder-quiz').each((ind, course_quiz) => {
			course_builder["sfwd-quiz"][jQuery(course_quiz).val()] = {
				"order": ind
			};
		});

		// List of sections
		jQuery('.ir-builder .ir-builder-top-element').each((ind, section) => {
			if (!jQuery(section).hasClass('ir-section')) {
				return true;
			}
			let steps = [];
			const next_steps = jQuery(section).parents('.ir-fed-sections').nextAll();
			if (0 < next_steps.length) {
				for (let loop = 0; loop < next_steps.length; loop++) {
					const node = next_steps[loop];
					if (jQuery(node).hasClass('ir-fed-sections')) {
						break;
					}
					steps.push(parseInt(jQuery(node).find('.ir-builder-lesson').val()));
				}
			}
			course_builder["section-heading"][jQuery(section).data('id').toString()] = {
				"ID": jQuery(section).data('id'),
				"post_title": jQuery(section).val(),
				"order": ind,
				"type": "section-heading",
				"tree": [],
				"steps": steps
			}
		});

		return course_builder;
	}

	/**
* Extract the course builder data from DOM.
*
* @since   4.4.0
*
* @param {Object} course_builder The course builder object
* @returns Updated course builder object.
*/
	const extractBuilderStructure = (course_builder) => {
		let lessons_list = [];
		// List of lessons.
		jQuery('.ir-builder .ir-builder-lesson').each((ind, lesson) => {
			lessons_list.push(parseInt(jQuery(lesson).val()));
			course_builder["sfwd-lessons"][jQuery(lesson).val()] = {
				"sfwd-topic": {},
				"sfwd-quiz": {},
			};

			// List of lesson topics.
			jQuery(lesson).parents('.ir-fed-lessons').siblings('.mantine-Accordion-panel').find('.ir-builder-lesson-topic').each((ind, topic) => {
				course_builder["sfwd-lessons"][jQuery(lesson).val()]["sfwd-topic"][jQuery(topic).val()] = {
					"sfwd-quiz": {},
				};

				// List of topic quizzes.
				jQuery(topic).parents('.ir-topics-acc-top').siblings('.mantine-Accordion-panel').find('.ir-sortable-quizzes .ir-builder-quiz').each((ind, topic_quiz) => {
					course_builder["sfwd-lessons"][jQuery(lesson).val()]["sfwd-topic"][jQuery(topic).val()]["sfwd-quiz"][jQuery(topic_quiz).val()] = {};
				});
			});

			// List of lesson quizzes.
			jQuery(lesson).parents('.ir-fed-lessons').siblings('.mantine-Accordion-panel').find('.ir-sortable-quizzes > .ir-fed-quizzes').each((ind, lesson_quiz) => {
				course_builder["sfwd-lessons"][jQuery(lesson).val()]["sfwd-quiz"][jQuery(lesson_quiz).find('.ir-builder-quiz').val()] = {};
			});
		});

		// List of quizzes.
		jQuery('.ir-builder .ir-fed-final-quizzes .ir-builder-quiz').each((ind, course_quiz) => {
			course_builder["sfwd-quiz"][jQuery(course_quiz).val()] = {};
		});

		// List of sections
		jQuery('.ir-builder .ir-builder-top-element').each((ind, section) => {
			if (!jQuery(section).hasClass('ir-section')) {
				return true;
			}
			let steps = [];
			const next_steps = jQuery(section).parents('.ir-fed-sections').nextAll();
			if (0 < next_steps.length) {
				for (let loop = 0; loop < next_steps.length; loop++) {
					const node = next_steps[loop];
					if (jQuery(node).hasClass('ir-fed-sections') && steps.length != 0) {
						break;
					}
					if (jQuery(node).find('.ir-builder-lesson').length) {
						steps.push(parseInt(jQuery(node).find('.ir-builder-lesson').val()));
					}
				}
			}
			course_builder["section-heading"][jQuery(section).data('id').toString()] = {
				"ID": jQuery(section).data('id'),
				"post_title": jQuery(section).val(),
				"order": ind,
				"type": "section-heading",
				"tree": [],
				"steps": steps
			}
		});

		course_builder["lessons_list"] = lessons_list;

		return course_builder;
	}

	/**
	 * Update course data via API.
	 * 
	 * @since   4.4.0
	 */
	function updateCourseData(status) {
		if ('draft' === status) {
			setDrafting(true);
		} else {
			setLoading(true);
		}

		// Check if future published date.
		if (new Date().getTime() < getCombinedDate(courseform.values.date, courseform.values.time).getTime()) {
			status = 'future';
		}

		// Check if private status.
		if ('draft' !== status && 'private' === courseform.values.visibility) {
			status = 'private';
		}

		const original_status = status;

		// Check if course review enabled and user is not admin.
		if (ir_fcb_loc.instructor_settings?.review_course) {
			// If admin user and trying to publish post
			if (ir_fcb_loc.is_admin) {
				if (review_data?.length > 0 && 'publish' === original_status) {
					showNotification({
						title: __('Approval Pending', 'wdm_instructor_role'),
						message: sprintf(__('Please review and approve all instructor updates to publish/update the %s', 'wdm_instructor_role'), ir_fcb_loc.lower_course_label),
						icon: <ExclamationMark />,
						color: 'yellow'
					});
					setLoading(false);
					return;
				}
			} else {
				// If not admin user, draft course.
				status = 'draft';
			}
		}

		let course_builder = {
			"sfwd-lessons": {},
			"sfwd-quiz": {},
			"section-heading": {}
		};
		course_builder = extractBuilderData(course_builder);

		let updated_sections = [];
		Object.keys(course_builder['section-heading']).map((item) => {
			updated_sections.push(course_builder['section-heading'][item]);
		});

		setCourseList({
			...courseList,
			sections: updated_sections
		});

		// Update Course Steps
		apiFetch(
			{
				path: "/ir/v1/sfwd-courses/" + course_id + "/steps",
				method: 'POST',
				data: JSON.stringify(course_builder)
			}
		).then((response) => {
			// Update course users.
			if (null !== courseform.values.course_users) {
				apiFetch(
					{
						path: "/ir/v1/sfwd-courses/" + course_id + "/users",
						method: 'POST',
						data: JSON.stringify({
							id: course_id,
							user_ids: courseform.values.course_users
						})
					}
				).then((data) => {
					// console.log(data);
				});
			}

			// Update shared instructors.
			// if (0 !== courseform.values.shared_instructors.length) {
			apiFetch(
				{
					path: "/ir/v1/instructors/" + course_id,
					method: 'POST',
					data: JSON.stringify({
						id: course_id,
						shared_instructors: courseform.values.shared_instructors
					})
				}
			);
			// }

			let course_category = null;
			if (typeof courseform.values.ld_course_category !== 'undefined' && courseform.values.ld_course_category.length > 0) {
				course_category = courseform.values.ld_course_category.filter(function (item) {
					return parseInt(item);
				})
			}

			// Update course settings.
			apiFetch(
				{
					path: "/ldlms/v2/sfwd-courses/" + course_id,
					method: 'POST',
					data: {
						slug: courseform.values.slug,
						status: status,
						title: course_title,
						content: tinymce.get("ir-content-editor").getContent(),
						ld_course_category: course_category,
						ld_course_tag: courseform.values.ld_course_tag,
						materials_enabled: courseform.values.materials_enabled,
						materials: tinymce.get("ir-course-material-editor").getContent(),
						certificate: courseform.values.certificate,
						exam_challenge: courseform.values.exam_challenge,
						disable_content_table: ('enrollees' === courseform.values.disable_content_table) ? true : false,
						lessons_per_page: courseform.values.lessons_per_page,
						lesson_per_page_custom: courseform.values.lesson_per_page_custom,
						topic_per_page_custom: courseform.values.topic_per_page_custom,
						price_type: courseform.values.price_type,
						price_type_paynow_price: courseform.values.price_type_paynow_price,
						price_type_subscribe_price: courseform.values.price_type_subscribe_price,
						trial_price: courseform.values.trial_price,
						price_type_closed_price: courseform.values.price_type_closed_price,
						price_type_closed_custom_button_url: courseform.values.price_type_closed_custom_button_url,
						prerequisite_enabled: courseform.values.prerequisite_enabled,
						prerequisite_compare: courseform.values.prerequisite_compare,
						prerequisites: courseform.values.prerequisites.filter(function (item) { return parseInt(item); }),
						points_enabled: courseform.values.points_enabled,
						points_access: courseform.values.points_access,
						points_amount: courseform.values.points_amount,
						expire_access: courseform.values.expire_access,
						expire_access_days: courseform.values.expire_access_days,
						expire_access_delete_progress: courseform.values.expire_access_delete_progress,
						progression_disabled: ('free' === courseform.values.progression_disabled) ? true : false,
						featured_media: courseform.values.featured_image,
						price_type_paynow_enrollment_url: courseform.values.price_type_paynow_enrollment_url,
						author: courseform.values.author,
						original_status: original_status,
						password: courseform.values.password,
						date: getCombinedDate(courseform.values.date, courseform.values.time)
					}
				}
			).then((resp) => {
				// if ('subscribe' === courseform.values.price_type) {
				// Update remaining course settings.
				apiFetch(
					{
						path: "/ir/v1/sfwd-courses/" + course_id + "/remaining",
						method: 'POST',
						data: JSON.stringify({
							id: course_id,
							type: 'subscribe',
							price_type_subscribe_price_billing_cycle: courseform.values.price_type_subscribe_price_billing_cycle,
							price_type_subscribe_price_billing_interval: courseform.values.price_type_subscribe_price_billing_interval,
							price_type_subscribe_billing_recurring_times: courseform.values.price_type_subscribe_billing_recurring_times,
							trial_duration: courseform.values.trial_duration,
							trial_duration_interval: courseform.values.trial_duration_interval,
							price_type_subscribe_enrollment_url: courseform.values.price_type_subscribe_enrollment_url,
							is_published: ('publish' === resp.status) ? true : false
						})
					}
				);
				// }

				apiFetch(
					{
						path: "/ir/v1/sfwd-courses/" + course_id + "/remaining",
						method: 'POST',
						data: JSON.stringify({
							id: course_id,
							type: 'pagination',
							lessons_per_page: courseform.values.lessons_per_page
						})
					}
				);

				if ('draft' === resp.status) {
					// Check if course review enabled.
					if (ir_fcb_loc.instructor_settings?.review_course && 'draft' !== original_status && !ir_fcb_loc.is_admin) {
						showNotification({
							message: sprintf(__('Your %1$s has been submitted for review to the administrator. Please wait for approval before it can be published. In the meantime, your %1$s is saved as draft.', 'wdm_instructor_role'), ir_fcb_loc.lower_course_label),
							autoClose: 5000
						})
					} else {
						showNotification({
							title: __('Saved as Draft', 'wdm_instructor_role'),
							message: sprintf(__('Your %s has successfully been saved as a draft', 'wdm_instructor_role'), ir_fcb_loc.lower_course_label),
						})
					}

				} else {
					if (!courseform.values.is_published) {
						coursePublished(resp.link);
						courseform.setFieldValue('is_published', true);
					} else {
						if ('draft' === courseform.values.status) {
							showNotification({
								title: sprintf(__('%s Published!', 'wdm_instructor_role'), ir_fcb_loc.course_label),
								message: sprintf(__('Your %s is successfully published and live now', 'wdm_instructor_role'), ir_fcb_loc.lower_course_label),
								autoClose: 5000
							});
						} else {
							showNotification({
								title: sprintf(__('%s Updated', 'wdm_instructor_role'), ir_fcb_loc.course_label),
								message: sprintf(__('Your %s has successfully been updated', 'wdm_instructor_role'), ir_fcb_loc.lower_course_label),
								autoClose: 5000
							});
						}
					}
				}
				courseform.setFieldValue('status', resp.status);
				courseform.setFieldValue('link', resp.link);
				courseform.setFieldValue('slug', resp.slug);

				if (resp.date.length) {
					courseform.setFieldValue('date', new Date(resp.date));
					courseform.setFieldValue('time', new Date(resp.date));
				}

				setVisibility(resp);

				setLoading(false);
				setDrafting(false);
			})

			if (null !== courseform.values.course_groups) {
				let path = "/ir/v1/sfwd-courses/" + course_id + "/groups";

				if (ir_fcb_loc.is_admin) {
					path = "/ldlms/v2/sfwd-courses/" + course_id + "/groups";
				}

				// Update course groups.
				apiFetch(
					{
						path,
						method: 'POST',
						data: {
							id: course_id,
							group_ids: courseform.values.course_groups
						}
					}
				);
			}
		});
	};

	/**
	 * Fetch the course data
	 * 
	 * @since   4.4.0
	 */
	const fetchCourseData = () => {
		apiFetch(
			{
				path: "/ir/v1/sfwd-courses/" + course_id + "/steps_data",
				method: 'GET',
			}
		).then((data) => {
			setCourseData(data)
		});

		apiFetch(
			{
				path: "/ir/v1/sfwd-courses/" + course_id + "/steps",
				method: 'GET',
			}
		)
			.then((data) => setCourseList(data))

		// Fetch list of lessons.
		apiFetch(
			{
				path: "/ir/v1/other-sfwd-lessons/?course=" + course_id,
				method: 'GET',
			}
		).then((data) => setLessonList(data));

		// Fetch list of topics.
		apiFetch(
			{
				path: "/ir/v1/other-sfwd-topics/?course=" + course_id,
				method: 'GET',
			}
		).then((data) => setTopicList(data));

		// Fetch list of quizzes.
		apiFetch(
			{
				path: "/ir/v1/other-sfwd-quizzes/?course=" + course_id,
				method: 'GET',
			}
		).then((data) => setQuizList(data));

		// Get author users.
		apiFetch(
			{
				path: "/ir/v1/users?roles=wdm_instructor,administrator",
				method: "GET"
			}
		).then((data) => {
			let author_list = [];
			data.map((user) => {
				author_list.push({
					label: user.name,
					value: user.id.toString()
				});
			})
			setAuthors(author_list);
		});
	};

	/**
	 * Display a course section.
	 * 
	 * @since   4.4.0
	 *
	 * @param {Object} props Section Properties
	 * @returns Course Section HTML
	 */
	const RenderCourseSection = (props) => {
		const sections = props.sections;
		const lesson = props.lesson;

		// Check if section to be rendered.
		let section = sections.find(function (sec, ind) {
			if (parseInt(sec.steps[0]) == parseInt(lesson)) {
				return true;
			}
		});

		const removeSection = (section) => {
			const updated_sections = courseList.sections.filter(item => section.ID !== item.ID);
			setCourseList({
				...courseList,
				sections: updated_sections
			});
		};

		return (
			<>
				{typeof (section) !== "undefined" && (
					<>
						<Paper className="ir-fed-sections" radius="lg" p="md" withBorder mb={15}>
							<Group align="center" position="apart">
								<Group align="center" spacing="xs">
									<GridDots
										size={20}
										strokeWidth={2}
										color={'#2067FA'}
									/>
									<Text size="lg">{html_entity_decode(section.post_title)}</Text>
									<input type="hidden" id={"ir-section-element-" + section.order} className='ir-section ir-builder-top-element' value={section.post_title} data-id={section.ID} />
								</Group>
								<Group align="center" spacing="sm"  >
									<Group className="ir-br-2" pr={15}>
										<EditSection edit_section={section} courseList={courseList} setCourseList={setCourseList} />
										<X onClick={() => removeSection(section)}
											size={24}
											strokeWidth={2}
											color={'#2067FA'}

										/>
									</Group>
									<Text className="ir-section-name" fz="sm" color="#555552" fw={400}>{__('Section', 'wdm_instructor_role')}</Text>
								</Group>
							</Group>
						</Paper>
						{sections.length && (<RenderCourseSection sections={sections.filter(item => section.ID !== item.ID)} lesson={lesson} />)}
					</>
				)}
			</>
		);
	};

	/**
	 * Display last course sections.
	 * 
	 * @since   4.4.0
	 *
	 * @returns Course Section HTML
	 */
	const RenderLastCourseSections = (props) => {
		const section = props.section;

		// Check whether to render.
		if (0 !== section.steps.length) {
			return '';
		}

		const removeSection = (section) => {
			const updated_sections = courseList.sections.filter(item => section.ID !== item.ID);
			setCourseList({
				...courseList,
				sections: updated_sections
			});
		};

		return (
			<>
				<Paper className="ir-fed-sections" radius="lg" p="md" withBorder mb={15}>
					<Group align="center" position="apart">
						<Group align="center" spacing="xs">
							<GridDots
								size={20}
								strokeWidth={2}
								color={'#2067FA'}
							/>
							<Text size="lg">{html_entity_decode(section.post_title)}</Text>
							<input type="hidden" id={"ir-section-element-" + section.order} className='ir-section ir-builder-top-element' value={section.post_title} data-id={section.ID} />
						</Group>
						<Group align="center" spacing="sm"  >
							<Group className="ir-br-2" pr={15}>
								<EditSection edit_section={section} courseList={courseList} setCourseList={setCourseList} />
								<X onClick={() => removeSection(section)}
									size={24}
									strokeWidth={2}
									color={'#2067FA'}

								/>
							</Group>
							<Text className="ir-section-name" fz="sm" color="#555552" fw={400}>{__('Section', 'wdm_instructor_role')}</Text>
						</Group>
					</Group>
				</Paper>
			</>
		);
	}

	const TopicModal = (props) => {
		const [loading, setLoading] = useState(false);
		const [topic_image, setTopicImage] = useState(null);
		const [checkedList, setCheckedList] = useState([false, false, false]);
		const [topicOpened, setTopicOpened] = useState(false);
		const [showCatFields, setShowCatFields] = useState(false);
		const [expand, setExpand] = useState('');
		const [new_category, setNewCategory] = useState('');
		const [showTagLoader, setShowTagLoader] = useState(false);
		const [parent_category, setParentCategory] = useState('');
		const [category_loading, setCategoryLoader] = useState(false);
		const [topicCategories, setTopicCategories] = useState([]);
		const [topicTags, setTopicTags] = useState([]);

		// const [topic_loading, setTopicLoading] = useState(false);
		const handleChange = (index) => {
			const _checkedList = [...checkedList];
			_checkedList[index] = !_checkedList[index];
			setCheckedList(_checkedList);
		}
		const topicform = useForm({
			initialValues: {
				title: '',
				status: 'publish',
				video_enabled: false,
				video_shown: 'BEFORE',
				assignment_upload_enabled: false,
				forced_timer_enabled: false,
				content: '<p></p>',
				material: '<p></p>',
				featured_image: null,
				tag: [],
				ld_topic_category: [],
				topic_author: 0,
				visibility: 'public',
				password: '',
				date: '',
				time: ''
			},
			validate: {
				title: (value) => (value.length > 0 ? null : __('Please enter a valid title.', 'wdm_instructor_role'))
			}
		});

		const createNewTopic = (values) => {
			setLoading(true);
			const timer_amount = (parseInt(values.timer_hh) * 60 * 60) + (parseInt(values.timer_mm) * 60) + (parseInt(values.timer_ss));
			let topic_category = null;
			if (typeof values.ld_topic_category !== 'undefined' && values.ld_topic_category.length > 0) {
				topic_category = values.ld_topic_category.filter(function (item) {
					return parseInt(item);
				})
			}

			let status = values.status;
			// Check if future published date.
			if (new Date().getTime() < getTopicCombinedDate(values.date, values.time).getTime()) {
				status = 'future';
			}

			// Check if private status.
			if ('draft' !== status && 'private' === values.visibility) {
				status = 'private';
			}

			apiFetch(
				{
					path: "/ldlms/v2/sfwd-topic",
					method: 'POST',
					data: {
						title: values.title,
						content: tinymce.get("ir-topic-content-editor").getContent(),
						status: status,
						course: course_id,
						lesson: props.lesson_id,
						slug: values.slug,
						materials: tinymce.get("ir-topic-material-editor").getContent(),
						materials_enabled: values.materials_enabled,
						video_enabled: values.video_enabled,
						video_url: values.video_url,
						video_shown: values.video_shown,
						video_auto_complete: values.video_auto_complete,
						video_auto_complete_delay: values.video_auto_complete_delay,
						video_show_complete_button: values.video_show_complete_button,
						video_auto_start: values.video_auto_start,
						video_show_controls: values.video_show_controls,
						video_focus_pause: values.video_focus_pause,
						video_resume: values.video_resume,
						assignment_upload_enabled: values.assignment_upload_enabled,
						assignment_points_enabled: values.assignment_points_enabled,
						assignment_points_amount: values.assignment_points_amount,
						assignment_auto_approve: values.assignment_auto_approve,
						assignment_deletion_enabled: values.assignment_deletion_enabled,
						forced_timer_enabled: values.forced_timer_enabled,
						forced_timer_amount: timer_amount,
						assignment_upload_limit_extensions: values.assignment_upload_limit_extensions,
						assignment_upload_limit_size: values.assignment_upload_limit_size,
						assignment_upload_limit_count: values.assignment_upload_limit_count,
						featured_media: values.featured_image,
						visible_type: values.visible_type,
						visible_after: values.visible_after,
						ld_topic_category: topic_category,
						ld_topic_tag: values.tag,
						author: values.topic_author.toString(),
						date: getTopicCombinedDate(values.date, values.time),
						password: values.password
					}
				}
			).then((response) => {
				if (false != response && response.hasOwnProperty('id')) {
					addNewTopic(response, props.lesson_id);

					if ('visible_after_specific_date' === values.visible_type) {
						const date_str = values.enrollment_date_yy + "-" + values.enrollment_date_mm + "-" + values.enrollment_date_dd + " " + values.enrollment_date_hh + ":" + values.enrollment_date_min;
						// dataset.visible_after_specific_date = date_str;
						const enrollment_date = new Date(date_str);
						const visible_after_specific_date = Math.trunc(enrollment_date.getTime() / 1000).toString();

						apiFetch(
							{
								path: "/ir/v1/sfwd-courses/" + course_id + "/remaining",
								method: 'POST',
								data: JSON.stringify({
									id: course_id,
									lesson_id: response.id,
									type: 'release_schedule',
									post_type: 'topic',
									specific_date: visible_after_specific_date
								})
							}
						);
					}
				}
				setLoading(false);
				wp.editor.remove('ir-topic-content-editor');
				wp.editor.remove('ir-topic-material-editor');
			});
		}

		const addNewTopic = (topic, lesson_id) => {
			let topic_tree = { ...courseList.h["sfwd-lessons"][lesson_id]["sfwd-topic"] };
			topic_tree[topic.id] = {
				"sfwd-quiz": []
			}

			// Update course data store if not present.
			if (!courseData.hasOwnProperty(topic.id)) {
				let _courseData = { ...courseData };
				_courseData[topic.id] = {
					post_title: topic.title.rendered,
					post_modified: topic.modified,
					post_status: topic.status
				};
				setCourseData(_courseData);
			}

			// Add to Builder
			setCourseList({
				...courseList,
				h: {
					...courseList.h,
					"sfwd-lessons": {
						...courseList.h["sfwd-lessons"],
						[lesson_id]: {
							...courseList.h["sfwd-lessons"][lesson_id],
							"sfwd-topic": topic_tree
						}
					}
				}
			});

			if ('draft' === topic.status) {
				showNotification({
					title: sprintf(__('Saved as Draft', 'wdm_instructor_role'), ir_fcb_loc.topic_label),
					message: sprintf(__('Your %s has been successfully saved as a draft', 'wdm_instructor_role'), ir_fcb_loc.lower_topic_label),
				});
			} else {
				showNotification({
					title: sprintf(__('%s Published!', 'wdm_instructor_role'), ir_fcb_loc.topic_label),
					message: sprintf(__('Your %s is successfully published and live now', 'wdm_instructor_role'), ir_fcb_loc.lower_topic_label),
				});
			}
		}

		const handleErrors = (errors) => {
			if (errors.title) {
				showNotification({ message: sprintf(__('Please fill %s title field', 'wdm_instructor_role'), ir_fcb_loc.lower_topic_label), color: 'red' });
			}
		}

		/* const openIframeInTab = (device, url) => {
				var win = window.open();
				win.document.write('<iframe style="width:' + device + '; height:100vh; display:block; margin: auto;" src="' + url + '" frameborder="0" allowfullscreen></iframe>')
		} */

		const createNewTopicTag = (query) => {
			setShowTagLoader(true);
			// Create New Topic Tag.
			apiFetch(
				{
					path: "/wp/v2/ld_topic_tag",
					method: 'POST',
					data: {
						name: query
					}
				}
			).then((response) => {
				if (false != response && response.hasOwnProperty('id')) {
					setShowTagLoader(false);
					const item = { value: response.id.toString(), label: response.name };
					setTopicTags((current) => [...current, item]);
					topicform.setFieldValue('tag', [...topicform.values.tag, response.id.toString()])
				}
			});
		}

		const createTopicCategory = () => {
			if (new_category.length === 0) {
				showNotification({
					title: __('Warning', 'wdm_instructor_role'),
					message: __('Category name cannot be empty', 'wdm_instructor_role'),
					icon: <ExclamationMark />,
					color: 'yellow'
				});
				return;
			}

			setCategoryLoader(true);
			let data = {
				name: new_category
			}
			if (null !== parent_category && parent_category.length > 0) {
				data.parent = parent_category;
			}

			// Create New Topic Category.
			apiFetch(
				{
					path: "/wp/v2/ld_topic_category",
					method: 'POST',
					data
				}
			).then((response) => {
				if (false != response && response.hasOwnProperty('id')) {
					const item = { value: response.id.toString(), label: response.name };
					setTopicCategories((current) => [...current, item]);
					topicform.setFieldValue('ld_topic_category', [...topicform.values.ld_topic_category, response.id.toString()])
				}
				setShowCatFields(false);
				setCategoryLoader(false);
				setParentCategory('');
				setNewCategory('');
			});
		}

		const getTopicVisibility = () => {
			let visibility = __('Public', 'wdm_instructor_role');
			if ('password' === topicform.values.visibility) {
				visibility = __('Password Protected', 'wdm_instructor_role');
			} else if ('private' === topicform.values.visibility) {
				visibility = __('Private', 'wdm_instructor_role')
			}
			return visibility;
		}

		const handleTopicVisibilityChange = (value) => {
			if ('private' === value) {
				topicform.setFieldValue('password', '');
				topicform.setFieldValue('visibility', value);
				topicform.setFieldValue('status', value);
			} else if ('password' === value) {
				topicform.setFieldValue('visibility', value);
			} else {
				topicform.setFieldValue('password', '');
				topicform.setFieldValue('visibility', value);
			}
		}

		const getTopicPublishedDate = () => {
			let date_string = '';
			const options = { month: 'long', day: 'numeric', hour12: true, hour: 'numeric', minute: 'numeric' };
			const publish_date = getTopicCombinedDate(topicform.values.date, topicform.values.time);

			return publish_date.toLocaleDateString(undefined, options);
		}

		const getTopicCombinedDate = (date_object, time_object) => {
			if (date_object instanceof Date && time_object instanceof Date) {
				let combined_date = '';
				combined_date = date_object.getFullYear() + '-' + (date_object.getMonth() + 1) + '-' + date_object.getDate() + ' ' + time_object.getHours() + ':' + time_object.getMinutes() + ':' + time_object.getSeconds();

				return new Date(combined_date);
			} else {
				return new Date();
			}
		}

		useEffect(() => {
			setTimeout(function () {
				wp.editor.initialize(
					'ir-topic-content-editor',
					{
						tinymce: {
							wpautop: true,
							height: '300px',
							plugins: 'charmap colorpicker compat3x directionality hr image lists media paste tabfocus textcolor wordpress wpautoresize wpdialogs wpeditimage link',
							toolbar1: 'formatselect bold italic underline strikethrough | bullist numlist | blockquote hr wp_more | alignleft aligncenter alignright | link unlink | media wp_add_media'
						},
						quicktags: false,
						mediaButtons: true,
					}
				);
				wp.editor.initialize(
					'ir-topic-material-editor',
					{
						tinymce: {
							wpautop: true,
							height: '300px',
							plugins: 'charmap colorpicker compat3x directionality hr image lists media paste tabfocus textcolor wordpress wpautoresize wpdialogs wpeditimage link',
							toolbar1: 'formatselect bold italic underline strikethrough | bullist numlist | blockquote hr wp_more | alignleft aligncenter alignright | link unlink | media wp_add_media'
						},
						quicktags: false,
						mediaButtons: true,
					}
				);
			});

			topicform.setValues({
				date: new Date(),
				time: new Date(),
			});
		}, [topicOpened]);

		useEffect(() => {
			if (topicOpened) {
				// Fetch topic categories
				if ("yes" === ir_fcb_loc.ld_cat_tag?.topic) {
					apiFetch(
						{
							path: "/ir/v1/ld-topic-category?per_page=100",
							method: 'GET',
						}
					).then((data) => {
						let categories = [];
						data.map((category, ind) => {
							if (category.hasOwnProperty('children')) {
								categories.push({
									label: category.name,
									value: category.id.toString(),
									children: category.children
								});
							} else {
								categories.push({
									label: category.name,
									value: category.id.toString()
								});
							}
						})
						setTopicCategories(categories);
					});
				}
				if ("yes" === ir_fcb_loc.ld_cat_tag?.topic_tag) {
					// Fetch topic tags
					apiFetch(
						{
							path: "/wp/v2/ld_topic_tag?per_page=100",
							method: 'GET',
						}
					).then((data) => {
						let tags = [];
						data.map((tag, ind) => {
							tags.push({
								label: tag.name,
								value: tag.id.toString()
							})
						}
						)
						setTopicTags(tags)
					});
				}
			}
		}, [topicOpened]);

		return (
			<>
				<Modal className="ir-topic-modal"
					opened={topicOpened}
					onClose={() => {
						setTopicOpened(false);
						wp.editor.remove('ir-topic-content-editor');
						wp.editor.remove('ir-topic-material-editor');
					}}
					fullScreen={expand}
					title={ir_fcb_loc.topic_label}
					size='830px'
				>
					<form onSubmit={topicform.onSubmit(createNewTopic, handleErrors)}>
						{expand && (

							<Button className="ir-expand-modal ir-btn-subtle" onClick={() => setExpand('')}>
								<ArrowsMinimize
									size={25}
									strokeWidth={2}
									color={'#000000'}
								/>
							</Button>
						)}
						{!expand && (
							<Button className="ir-expand-modal ir-btn-subtle" onClick={() => setExpand('fullScreen')}>
								<ArrowsMaximize
									size={25}
									strokeWidth={2}
									color={'#000000'}
								/>
							</Button>
						)}
						<Tabs defaultValue="Content" styles={{
							tab: {
								color: '#A0A3BD',
								fontSize: "16px",
								'&[data-active]': {
									color: '#000000',
								},
							},

						}}>
							<Tabs.List>
								<Tabs.Tab value="Content">{__('Content', 'wdm_instructor_role')}</Tabs.Tab>
								<Tabs.Tab value="Material">{__('Material', 'wdm_instructor_role')}</Tabs.Tab>
								<Tabs.Tab value="settings">{__('Settings', 'wdm_instructor_role')}</Tabs.Tab>
							</Tabs.List>
							<Tabs.Panel className="ir-panel-overflow" value="Content" pt="lg">
								<Group align="center" mt={10}>
									<TextInput className="ir-tt-text ir-parent-settings ir-flex-1"
										label={sprintf(__('%s Title', 'wdm_instructor_role'), ir_fcb_loc.lower_topic_label)}
										placeholder={__('Write title here', 'wdm_instructor_role')}
										size="lg"
										{...topicform.getInputProps('title')}
									/>
								</Group>
								<Group mb={34} mt={20} p={8} className="ir-featured-image-section">
									{null === topic_image && (
										<MediaUpload
											onSelect={(media) => {
												setTopicImage({
													id: media.id,
													url: media.url
												});
												topicform.setFieldValue('featured_image', media.id);
											}}
											allowedTypes={['image']}
											render={({ open }) => (
												<Group>
													<img src={require('/images/featured-placeholder.png')} />
													<Button fw={400} className="ir-btn-subtle" onClick={open} leftIcon={<PhotoUp size={23} strokeWidth={1} />}  >{__('Set Featured Image', 'wdm_instructor_role')}</Button>
												</Group>
											)}
										/>
									)}
									{null !== topic_image && (
										<Group>
											<div style={{ width: 88, marginLeft: 'auto', marginRight: 'auto' }}>
												<Image
													width={88}
													height={61}
													radius="md"
													src={topic_image.url}
												/>
											</div>
											<div>
												<MediaUpload
													onSelect={(media) => {
														setTopicImage({
															id: media.id,
															url: media.url
														});
														topicform.setFieldValue('featured_image', media.id);
													}}
													allowedTypes={['image']}
													value={topic_image.id}
													render={({ open }) => (
														<>
															<Button fw={400} className="ir-btn-subtle" onClick={open} leftIcon={<Replace size={23} strokeWidth={1} />} >{__('Replace', 'wdm_instructor_role')}</Button>
															<Button fw={400} className="ir-btn-subtle ir-red-txt" onClick={() => {
																setTopicImage(null);
																topicform.setFieldValue('featured_image', null);
															}} leftIcon={<Trash size={23} strokeWidth={1} />} >{__('Remove', 'wdm_instructor_role')}</Button>
														</>
													)}
												/>
											</div>
										</Group>
									)}
								</Group>
								{/* <Group spacing="xs" mb={34}> */}
								<Text className="ir-parent-settings" size="lg">{sprintf(__('%s Content', 'wdm_instructor_role'), ir_fcb_loc.lower_topic_label)}</Text>
								<Textarea
									className="ir-tiny-mce-editors"
									id="ir-topic-content-editor"
									minRows={15}
									{...topicform.getInputProps('content')}
								/>
								{/* </Group> */}
								<Group spacing="xs" mb={34} mt={34} className="ir-fc-100">
									<Group position="apart" align="center" mb={10}>
										<Text className="ir-parent-settings" size="lg">{__('ADVANCE CONTENT OPTION', 'wdm_instructor_role')}</Text>
										<Text className="ir-section-name" size="md">{__('You can use only one option at a time', 'wdm_instructor_role')}</Text>
									</Group>
									<Accordion width="100%" variant="contained" radius="md" defaultValue="customization"
										onChange={(option) => (topicform.setValues({
											video_enabled: (option == 'video_enabled') ? true : false,
											assignment_upload_enabled: (option == 'assignment_upload_enabled') ? true : false,
											forced_timer_enabled: (option == 'forced_timer_enabled') ? true : false,
										}))}
									>
										<Accordion.Item value="video_enabled">
											<Group className="ir-acc-left ir-no-chevron" position="apart" align="center">
												<Text size="md" className="ir-d-flex">
													{__('Video Progression', 'wdm_instructor_role')}
													<div className="ir-textinfo">
														<Tooltip label={sprintf(__("Require users to watch the full video as part of the %s progression. Use shortcode [ld_video] to move within the post content.", "wdm_instructor_role"), ir_fcb_loc.lower_course_label)} withArrow>
															<Button><InfoCircle size={20} strokeWidth={2} color={'#444444'} /></Button>
														</Tooltip>
													</div>
												</Text>
												<Accordion.Control><img className="ir-disable" src={require('/images/switch.png')} /><img className="ir-enable" src={require('/images/switchon.png')} /></Accordion.Control>
											</Group>
											<Accordion.Panel className="ir-pl-35">
												<TextInput className="ir-input-settings"
													placeholder={__("Enter your video link/URL here", 'wdm_instructor_role')}
													label={__("Video URL", "wdm_instructor_role")}
													labelPosition="left"
													{...topicform.getInputProps('video_url')}
												/>

												<Radio.Group mt={35}
													name="Display Timing"
													orientation="vertical"
													label={__("Display Timing", "wdm_instructor_role")}
													defaultValue="BEFORE"
													{...topicform.getInputProps('video_shown')}
												>
													<Radio value="BEFORE" label={__("Before completed sub-steps", "wdm_instructor_role")} description={sprintf(__("The video will be shown and must be fully watched before the user can access the %s's associated steps.", "wdm_instructor_role"), ir_fcb_loc.lower_topic_label)}
													/>
													<Radio value="AFTER" label={__("After completing sub-steps", "wdm_instructor_role")} description={sprintf(__("The video will be visible after the user has completed the %s`s associated steps. The full video must be watched in order to complete the %s.", "wdm_instructor_role"), ir_fcb_loc.lower_topic_label, ir_fcb_loc.lower_topic_label)}
													/>
													<div className="ir-conditional-settings pl-0">
														<Group className="ir-acss" spacing={0} align="center" pl={32}>
															<Text className="mantine-InputWrapper-label ir-d-flex">
																{sprintf(__('%s auto-completion', 'wdm_instructor_role'), ir_fcb_loc.topic_label)}
																<div className="ir-textinfo">
																	<Tooltip label={sprintf(__("Automatically mark the %s as completed once the user has watched the full video.", "wdm_instructor_role"), ir_fcb_loc.lower_topic_label)} withArrow>
																		<Button><InfoCircle size={20} strokeWidth={2} color={'#444444'} /></Button>
																	</Tooltip>
																</div>
															</Text>
															<Switch className="ir-vp-settings ir-acss" labelPosition="left" {...topicform.getInputProps('video_auto_complete', { type: 'checkbox' })} />
														</Group>
														<Group className="ir-acss" spacing={0} align="center" pl={32} mt={15}>
															<Text className="mantine-InputWrapper-label ir-d-flex">
																{__('Completion delay', 'wdm_instructor_role')}
																<div className="ir-textinfo">
																	<Tooltip label={sprintf(__("Specify a delay between video completion and %s completion.", "wdm_instructor_role"), ir_fcb_loc.lower_topic_label)} withArrow>
																		<Button><InfoCircle size={20} strokeWidth={2} color={'#444444'} /></Button>
																	</Tooltip>
																</div>
															</Text>
															<TextInput className="ir-input-settings ir-acss"
																placeholder="0"
																labelPosition="left"
																styles={{ input: { width: 85, borderRadius: '6px' } }}
																{...topicform.getInputProps('video_auto_complete_delay')}
															/>
														</Group>
														<Group className="ir-acss" spacing={0} align="center" pl={32} mt={15} mb={15}>
															<Text className="mantine-InputWrapper-label ir-d-flex">
																{__("Mark Complete Button", 'wdm_instructor_role')}
																<div className="ir-textinfo">
																	<Tooltip label={sprintf(__("Display the Mark Complete button on a %s even if not yet clickable.", "wdm_instructor_role"), ir_fcb_loc.lower_topic_label)} withArrow>
																		<Button><InfoCircle size={20} strokeWidth={2} color={'#444444'} /></Button>
																	</Tooltip>
																</div>
															</Text>
															<Switch className="ir-vp-settings ir-acss" labelPosition="left" value="Mark Complete Button" {...topicform.getInputProps('video_show_complete_button', { type: 'checkbox' })} />
														</Group>


													</div>
												</Radio.Group>

												<Switch className="ir-vp-settings" mt={20} labelPosition="left" label={__("Autostart", "wdm_instructor_role")} description={__("The video now starts automatically on page load", "wdm_instructor_role")} {...topicform.getInputProps('video_auto_start', { type: 'checkbox' })} />
												<Switch className="ir-vp-settings" mt={20} labelPosition="left" label={__("Video Controls Display", "wdm_instructor_role")} description={__("Users can pause, move backward and forward within the video", "wdm_instructor_role")} {...topicform.getInputProps('video_show_controls', { type: 'checkbox' })} />
												<Switch className="ir-vp-settings" mt={20} labelPosition="left" label={__("Video Pause on Window Unfocused", "wdm_instructor_role")} {...topicform.getInputProps('video_focus_pause', { type: 'checkbox' })} />
												<Switch className="ir-vp-settings" mt={20} labelPosition="left" label={__("Video resume", "wdm_instructor_role")} {...topicform.getInputProps('video_resume', { type: 'checkbox' })} />
											</Accordion.Panel>
										</Accordion.Item>

										<Accordion.Item value="assignment_upload_enabled">
											<Group className="ir-acc-left ir-no-chevron" position="apart" align="center">
												<Text size="md" className="ir-d-flex">
													{__('Assignment Uploads', 'wdm_instructor_role')}
												</Text>
												<Accordion.Control><img className="ir-disable" src={require('/images/switch.png')} /><img className="ir-enable" src={require('/images/switchon.png')} /></Accordion.Control>
											</Group>
											<Accordion.Panel className="ir-pl-35">
												<TextInput className="ir-input-settings ir-vp-settings ir-au-settings" mb={24}
													placeholder={__("Enter extension here pdf, xls or zip", "wdm_instructor_role")}
													label="File Extensions"
													labelPosition="left"
													{...topicform.getInputProps('assignment_upload_limit_extensions')}
												/>
												<TextInput className="ir-input-settings ir-vp-settings ir-au-settings" mb={24}
													placeholder="512 MB"
													label="File Size Limit"
													labelPosition="left"
													{...topicform.getInputProps('assignment_upload_limit_size')}
												/>
												<Switch className="ir-vp-settings" mt={20} labelPosition="left" label="Points" description={__("Award points for submitting assignments", "wdm_instructor_role")} {...topicform.getInputProps('assignment_points_enabled', { type: 'checkbox' })} />
												<TextInput className="ir-input-settings ir-vcd-text" pl={32}
													placeholder=""
													label={__("available point(s)", "wdm_instructor_role")}
													labelPosition="left"
													type="number"
													styles={{ input: { width: 85, borderRadius: '6px' } }}
													{...topicform.getInputProps('assignment_points_amount')}
												/>
												<Radio.Group mt={35}
													name="Grading Type"
													orientation="vertical"
													label={__("Grading Type", "wdm_instructor_role")}
													defaultValue="auto_approve"
													onChange={(value) => (topicform.setValues({
														assignment_auto_approve: ('auto_approve' === value) ? true : false
													}))}
												>
													<Radio value="auto_approve" label={__("Auto-approve", "wdm_instructor_role")} description={__("No grading or approval needed. The assignment will be automatically approved and full points will be awarded.", "wdm_instructor_role")} />
													<Radio value="manual_grade" label={__("Manually grade", "wdm_instructor_role")} description={sprintf(__("Admin or Group leader approval and grading required. The %s cannot be completed until the assignment is approved", "wdm_instructor_role"), ir_fcb_loc.lower_topic_label)} />
													<div className="ir-conditional-settings pl-0">
														<TextInput className="ir-input-settings ir-acss" pl={32}
															placeholder=""
															label="Limit number of uploaded files"
															labelPosition="left"
															type="number"
															{...topicform.getInputProps('assignment_upload_limit_count')}
															styles={{ input: { width: 85, borderRadius: '6px' } }}
														/>
														<Switch className="ir-vp-settings ir-acss" pl={32} mt={15} mb={15} labelPosition="left" label="Allow file deletion" {...topicform.getInputProps('assignment_deletion_enabled', { type: 'checkbox' })} />
													</div>
												</Radio.Group>
											</Accordion.Panel>
										</Accordion.Item>
										<Accordion.Item value="forced_timer_enabled">
											<Group className="ir-acc-left ir-no-chevron" position="apart" align="center">
												<Text size="md" className="ir-d-flex">
													{sprintf(__('%s Timer', 'wdm_instructor_role'), ir_fcb_loc.topics_label)}
													<div className="ir-textinfo">
														<Tooltip label={__("The module cannot be marked as completed until the set time has elapsed.", "wdm_instructor_role")} withArrow>
															<Button><InfoCircle size={20} strokeWidth={2} color={'#444444'} /></Button>
														</Tooltip>
													</div>
												</Text>
												<Accordion.Control><img className="ir-disable" src={require('/images/switch.png')} /><img className="ir-enable" src={require('/images/switchon.png')} /></Accordion.Control>
											</Group>
											<Accordion.Panel>
												<Group position="center" align="center">
													<NumberInput className="ir-input-settings ir-acss"
														placeholder="HH"
														labelPosition="left"
														type="number"
														min={0}
														styles={{ input: { width: 85, borderRadius: '6px' } }}
														{...topicform.getInputProps('timer_hh')}
													/>
													<NumberInput className="ir-input-settings ir-acss"
														placeholder="MM"
														labelPosition="left"
														type="number"
														min={0}
														styles={{ input: { width: 85, borderRadius: '6px' } }}
														{...topicform.getInputProps('timer_mm')}
													/>
													<NumberInput className="ir-input-settings ir-acss"
														placeholder="SS"
														labelPosition="left"
														type="number"
														min={0}
														styles={{ input: { width: 85, borderRadius: '6px' } }}
														{...topicform.getInputProps('timer_ss')}
													/>
												</Group>
											</Accordion.Panel>
										</Accordion.Item>
									</Accordion>
								</Group>
								<Group spacing="xs" mb={34} className="ir-fc-100">
									<Text className="ir-parent-settings ir-capitalize-letter" size="lg" width="100%" >
										{__('Tags and Categories', 'wdm_instructor_role')}
									</Text>
									<Accordion width="100%" variant="contained" radius="md" defaultValue="customization">
										<Accordion.Item value="ld_topic_tag">
											<Accordion.Control>
												{sprintf(__('%s Tags', 'wdm_instructor_role'), ir_fcb_loc.topic_label)}
											</Accordion.Control>
											<Accordion.Panel>
												{"yes" === ir_fcb_loc.ld_cat_tag?.topic_tag && (
													<MultiSelect className="ir-multiselect-loader"
														icon={showTagLoader && (<Loader size="xs" />)}
														label={sprintf(__("Add %s tags", 'wdm_instructor_role'), ir_fcb_loc.topic_label)}
														placeholder={__("Select items", 'wdm_instructor_role')}
														data={topicTags}
														searchable
														creatable
														getCreateLabel={(query) => `+ Create ${query}`}
														onCreate={createNewTopicTag}
														{...topicform.getInputProps('tag')}
													/>
												)}
												{"yes" !== ir_fcb_loc.ld_cat_tag?.topic_tag && (
													<Text color="dimmed" weight={400}>{sprintf(__("%s tags are disabled from Learndash settings.", 'wdm_instructor_role'), ir_fcb_loc.topic_label)} </Text>
												)}
											</Accordion.Panel>
										</Accordion.Item>

										<Accordion.Item value="ld_topic_category">
											<Accordion.Control>
												{sprintf(__('%s Categories', 'wdm_instructor_role'), ir_fcb_loc.topic_label)}
												<Text size="sm" color="dimmed" weight={400}>
													{__("Child categories cannot be created from here.", "wdm_instructor_role")}
												</Text>
											</Accordion.Control>
											<Accordion.Panel>
												{((ir_fcb_loc.is_admin && ir_fcb_loc.ld_cat_tag.topic) || (false !== ir_fcb_loc.instructor_settings && ('off' !== ir_fcb_loc.instructor_settings?.enable_ld_category) && ir_fcb_loc.ld_cat_tag.topic)) && (
													<>
														<Checkbox.Group
															{...topicform.getInputProps('ld_topic_category')}
															orientation="vertical"
														>
															{null != topicCategories && topicCategories.map((category, index) => (
																<div>
																	<Checkbox value={category.value} label={category.label} />
																	{category.hasOwnProperty('children') && category.children.map((child) => (
																		<Checkbox ml={20} mt={20} size="md" value={child.id.toString()} label={child.name} />
																	))}
																</div>
															))}
														</Checkbox.Group>
														<Group align="center" spacing="xs" mt={20}>
															<Text className="ir-add-cat" size={14} color="#2067FA" fw={600} onClick={() => setShowCatFields(true)}>{sprintf(__('Add new %s category', 'wdm_instructor_role'), ir_fcb_loc.lower_topic_label)}</Text>
															{!showCatFields && (
																<Plus onClick={() => setShowCatFields(true)}
																	size={18}
																	strokeWidth={2}
																	color={'#2067FA'}

																/>
															)}
															{showCatFields && (
																<X onClick={() => setShowCatFields(false)}
																	size={18}
																	strokeWidth={2}
																	color={'#2067FA'}

																/>
															)}
														</Group>
														{showCatFields && (
															<div>
																<TextInput
																	placeholder="Enter new category name"
																	value={new_category}
																	onChange={(event) => setNewCategory(event.currentTarget.value)}
																/>
																{/* <Select mt={6}
																																	placeholder="Select Parent Course Category "
																																	data={topicCategories}
																																	value={parent_category} onChange={setParentCategory}
																															/> */}
																<Button
																	mt={6}
																	className="ir-btn-filled"
																	onClick={createTopicCategory}
																	loading={category_loading}
																>Add</Button>
															</div>
														)}
													</>
												)}

												{((!ir_fcb_loc.is_admin && (false === ir_fcb_loc.instructor_settings || 'off' === ir_fcb_loc.instructor_settings?.enable_ld_category)) || "yes" !== ir_fcb_loc.ld_cat_tag.topic) && (
													<Text color="dimmed" weight={400}>{sprintf(__("%s categories are disabled from Learndash settings.", 'wdm_instructor_role'), ir_fcb_loc.topic_label)} </Text>
												)}
											</Accordion.Panel>
										</Accordion.Item>
									</Accordion>
								</Group>

								{ir_fcb_loc.is_admin && (
									<Group spacing="xs" mb={34} className="ir-fc-100">
										<Text className="ir-parent-settings ir-capitalize-letter" size="lg" width="100%" >{__('Author', 'wdm_instructor_role')}</Text>
										<Accordion width="100%" variant="contained" radius="md">
											<Accordion.Item value="topic-author">
												<Accordion.Control>
													{__('Author', 'wdm_instructor_role')}
												</Accordion.Control>
												<Accordion.Panel>
													<Select mt={6}
														placeholder={sprintf(__("Select %s Author", "wdm_instructor_role"), ir_fcb_loc.topic_label)}
														data={authors}
														{...topicform.getInputProps('topic_author')}
													/>
												</Accordion.Panel>
											</Accordion.Item>
										</Accordion>
									</Group>
								)}

								{(ir_fcb_loc.is_admin || (false !== ir_fcb_loc.instructor_settings && ('off' !== ir_fcb_loc.instructor_settings?.enable_permalinks))) && (
									<Group spacing="xs" mb={34} className="ir-fc-100">
										<Text className="ir-parent-settings" size="lg" width="100%">{sprintf(__('%s URL', 'wdm_instructor_role'), ir_fcb_loc.topic_label)}</Text>
										<Accordion width="100%" variant="contained" radius="md" defaultValue="customization">

											<Accordion.Item value="permalink">
												<Accordion.Control>{sprintf(__('%s URL', 'wdm_instructor_role'), ir_fcb_loc.topic_label)}</Accordion.Control>
												<Accordion.Panel>
													<TextInput className="ir-input-settings ir-url-slug"
														placeholder={__("Enter here", "wdm_instructor_role")}
														label="URL Slug "
														labelPosition="left"
														{...topicform.getInputProps('slug')}
													/>
												</Accordion.Panel>
											</Accordion.Item>
										</Accordion>
									</Group>
								)}
							</Tabs.Panel>
							<Tabs.Panel className="ir-panel-overflow" value="Material" pt="lg">
								<Switch mb={10}
									labelPosition="left"
									label={sprintf(__('%s Material', 'wdm_instructor_role'), ir_fcb_loc.topic_label)}
									size="lg"
									checked={topicform.values.materials_enabled}
									onChange={(event) => topicform.setFieldValue('materials_enabled', event.currentTarget.checked)}
									className={topicform.values.materials_enabled ? 'ir-checked ir-course-access-p-settings ir-switch ir-parent-settings ir-material' : 'ir-unchecked ir-course-access-p-settings ir-switch ir-parent-settings ir-material'}
									description={__('Any content added below is displayed on a separate tab, alongside your main content.', 'wdm_instructor_role')}
								/>
								<div className="ir-conditional-disabled">
									<Textarea
										className="ir-tiny-mce-editors"
										id="ir-topic-material-editor"
										minRows={15}
										{...topicform.getInputProps('materials')}
									/>
								</div>
							</Tabs.Panel>
							<Tabs.Panel className="ir-panel-overflow" value="settings" pt="lg">
								<Group position="apart" align="center" mb={10}>
									<Text size="lg" className="ir-parent-settings">{sprintf(__('%s Access Option', 'wdm_instructor_role'), ir_fcb_loc.topic_label)}</Text>
									<Text className="ir-section-name" size="md">{sprintf(__('Controls the timing and way %s can be accessed', 'wdm_instructor_role'), ir_fcb_loc.topics_label)}</Text>
								</Group>
								<div className="ir-module-settings">
									<Radio.Group className="ir-trs"
										name="TopicRelease"
										orientation="vertical"
										label={sprintf(__('%s Release Schedule', 'wdm_instructor_role'), ir_fcb_loc.topic_label)}
										defaultValue=""
										{...topicform.getInputProps('visible_type')}
									>
										<Radio value="" label={__('Immediately', 'wdm_instructor_role')} description={sprintf(__("The %s is made available on course enrollment", 'wdm_instructor_role'), ir_fcb_loc.topic_label)} />
										<Radio value="visible_after" label={__('Enrollment-based', 'wdm_instructor_role')} description={sprintf(__('The %s will be available X days after course enrollment', 'wdm_instructor_role'), ir_fcb_loc.topic_label)} />
										<TextInput className="ir-input-settings ir-vcd-text" pl={32}
											placeholder=""
											label="days(s)"
											labelPosition="right"
											type="number"
											styles={{ input: { width: 85, borderRadius: '10px' } }}
											{...topicform.getInputProps('visible_after')}
										/>
										<Radio value="visible_after_specific_date" label={__('Specific date', 'wdm_instructor_role')} description={sprintf(__('The %s will be available on a specific date.', 'wdm_instructor_role'), ir_fcb_loc.topic_label)} />
										<Group className="ir-sp-date" pl={32}>
											<Select
												placeholder="MM"
												labelPosition="right"
												styles={{ input: { width: 62, borderRadius: '10px' } }}
												data={[
													{ value: 1, label: 'Jan' },
													{ value: 2, label: 'Feb' },
													{ value: 3, label: 'Mar' },
													{ value: 4, label: 'Apr' },
													{ value: 5, label: 'May' },
													{ value: 6, label: 'Jun' },
													{ value: 7, label: 'Jul' },
													{ value: 8, label: 'Aug' },
													{ value: 9, label: 'Sep' },
													{ value: 10, label: 'Oct' },
													{ value: 11, label: 'Nov' },
													{ value: 12, label: 'Dec' },
												]}
												{...topicform.getInputProps('enrollment_date_mm')}
											/>
											<TextInput
												placeholder="DD"
												label=""
												labelPosition="right"
												type="number"
												styles={{ input: { width: 62, borderRadius: '10px' } }}
												{...topicform.getInputProps('enrollment_date_dd')}
											/>,
											<TextInput
												placeholder="YYYY"
												label=""
												labelPosition="right"
												type="number"
												styles={{ input: { width: 72, borderRadius: '10px' } }}
												{...topicform.getInputProps('enrollment_date_yy')}
											/>@
											<TextInput
												placeholder="HH"
												label=""
												labelPosition="right"
												type="number"
												styles={{ input: { width: 62, borderRadius: '10px' } }}
												{...topicform.getInputProps('enrollment_date_hh')}
											/>:
											<TextInput
												placeholder="MN"
												label=""
												labelPosition="right"
												type="number"
												styles={{ input: { width: 62, borderRadius: '10px' } }}
												{...topicform.getInputProps('enrollment_date_min')}
											/>
										</Group>
									</Radio.Group>
								</div>
							</Tabs.Panel>
						</Tabs>
						<Group spacing="xs" mt={34} className="ir-tm-bottom-actions" position="apart" align="center">
							<Group>
								{/*loading && (<Loader size="md" variant="dots" />)*/}
							</Group>
							<Group spacing="xs">
								{/* <Popover width={165} position="bottom" withArrow shadow="lg" radius={6}>
                                    <Popover.Target>
                                        <Button size="md" fw={400} className="ir-btn-subtle" radius="md" variant="subtle" leftIcon={<Eye size={22} />} >
                                            {__('Preview', 'wdm_instructor_role')}
                                        </Button>
                                    </Popover.Target>
                                    <Popover.Dropdown className="ir-preview-module">
                                        <Button className="ir-btn-subtle" onClick={() => openIframeInTab('100%', ir_fcb_loc.course_preview_link)} size="sm" variant="subtle" leftIcon={<DeviceDesktop size={22} color={'#000000'} />} >
                                            {__('Desktop view', 'wdm_instructor_role')}
                                        </Button>
                                        <Button className="ir-btn-subtle" onClick={() => openIframeInTab('768px', ir_fcb_loc.course_preview_link)} size="sm" variant="subtle" leftIcon={<DeviceTablet size={22} color={'#000000'} />} >
                                            {__('Tablet view', 'wdm_instructor_role')}
                                        </Button>
                                        <Button className="ir-btn-subtle" onClick={() => openIframeInTab('475px', ir_fcb_loc.course_preview_link)} size="sm" variant="subtle" leftIcon={<DeviceMobile size={22} color={'#000000'} />} >
                                            {__('Mobile view', 'wdm_instructor_role')}
                                        </Button>
                                    </Popover.Dropdown>
                                </Popover> */}
								<Popover position="bottom" withArrow shadow="lg" radius={6} width={320}>
									<Popover.Target>
										<Button
											size="sm"
											fw={600}
											className="ir-btn-subtle"
											radius="md"
											variant="subtle"
										>
											{__('Status and Visibility', 'wdm_instructor_role')}
										</Button>
									</Popover.Target>
									<Popover.Dropdown className="ir-status-and-visibility">
										<Accordion width={320}>
											<Accordion.Item value="visibility" variant="filled">
												<Accordion.Control>
													<Group align="center" spacing="xs" position="apart">
														<Text size="sm" fw={600}>{__('Visibility', 'wdm_instructor_role')}</Text>
														<Text size="sm" fw={600} color="#2067FA;">{getTopicVisibility()}</Text>
													</Group>
												</Accordion.Control>
												<Accordion.Panel>
													<Radio.Group
														defaultValue='public'
														orientation='vertical'
														value={topicform.values.visibility}
														onChange={handleTopicVisibilityChange}
													>
														<Radio value="public" label={__('Public', 'wdm_instructor_role')} description={__('Visible to everyone', 'wdm_instructor_role')} />
														<Radio value="private" label={__('Private', 'wdm_instructor_role')} description={__('Only visible to site admins and editors', 'wdm_instructor_role')} />
														<Radio value="password" label={__('Password Protected', 'wdm_instructor_role')} description={__('Protected with a password you choose. Only those with the password can view this post.', 'wdm_instructor_role')} />
													</Radio.Group>
													{'password' === topicform.values.visibility && (
														<TextInput
															mt={5}
															placeholder={__('Use a secure Password', 'wdm_instructor_role')}
															size="sm"
															{...topicform.getInputProps('password')}
														/>
													)}
												</Accordion.Panel>
											</Accordion.Item>

											<Accordion.Item value="publish">
												<Accordion.Control>
													<Group align="center" spacing="xs" position="apart">
														<Text size="sm" fw={600}>{__('Publish', 'wdm_instructor_role')}</Text>
														<Text size="sm" fw={600} color="#2067FA;">{getTopicPublishedDate()}</Text>
													</Group>
												</Accordion.Control>
												<Accordion.Panel>
													<TimeInput
														className='ir-noborder'
														format="12"
														label={__('Time', 'wdm_instructor_role')}
														defaultValue={new Date()}
														icon={<ClockHour4 size={16} />}
														{...topicform.getInputProps('time')}
													/>
													<DatePicker
														label={__('Date', 'wdm_instructor_role')}
														defaultValue={new Date()}
														icon={<CalendarEvent size={16} />}
														{...topicform.getInputProps('date')}
													/>
												</Accordion.Panel>
											</Accordion.Item>
										</Accordion>
									</Popover.Dropdown>
								</Popover>
								<Button className="ir-btn-outline" variant="outline" onClick={() => { topicform.setFieldValue('status', 'draft'); }} type="submit"
									loading={loading}
								>
									{__('Save to draft', 'wdm_instructor_role')}
								</Button>
								<Button className="ir-btn-filled" variant="filled" type="submit" loading={loading} onClick={() => {
									topicform.setFieldValue('status', 'publish');
								}}>
									{__('Publish', 'wdm_instructor_role')}
								</Button>
							</Group>
						</Group>
					</form>
				</Modal>
				<Button disabled={((null === topicTags) || (null === topicCategories)) ? true : false}
					onClick={() => setTopicOpened(true)} className="ir-fed-add-topic ir-btn-outline" radius="md" variant="outline" leftIcon={<Plus size={18} />}>
					{sprintf(__('Add New %s', 'wdm_instructor_role'), ir_fcb_loc.topic_label)}
				</Button>
				<AddQuiz setCourseData={setCourseData} courseData={courseData} setCourseList={setCourseList} courseList={courseList} course_id={course_id} lesson_id={props.lesson_id} class="ir-btn-outline" />
			</>
		);
	}

	const LessonModal = (props) => {
		const [lesson_image, setLessonImage] = useState(null);
		const [loading, setLoading] = useState(false);
		const [new_quiz_loader, setNewQuizLoader] = useState(false);
		const [showCatFields, setShowCatFields] = useState(false);
		const [showTagLoader, setShowTagLoader] = useState(false);
		const { tags, setTags } = props;
		const [expand, setExpand] = useState('');
		const [new_category, setNewCategory] = useState('');
		const [parent_category, setParentCategory] = useState('');
		const [category_loading, setCategoryLoader] = useState(false);
		const [lessonTags, setLessonTags] = useState([]);
		const [lessonCategories, setLessonCategories] = useState([]);

		const lessonform = useForm({
			initialValues: {
				title: '',
				content: '<p></p>',
				status: 'draft',
				featured_image: null,
				featured_image_url: null,
				tag: [],
				ld_lesson_category: [],
				lesson_author: 0,
				visibility: 'public',
				password: '',
				date: '',
				time: ''
			},
			validate: {
				title: (value) => (value.length > 0 ? null : __('Please enter a valid title.', 'wdm_instructor_role'))
			}
		});
		const [checkedList, setCheckedList] = useState([false, false, false]);
		const handleChange = (index) => {
			const _checkedList = [...checkedList];
			_checkedList[index] = !_checkedList[index];
			setCheckedList(_checkedList);
		}

		const createNewLesson = (values) => {
			setLoading(true);
			const timer_amount = (parseInt(values.timer_hh) * 60 * 60) + (parseInt(values.timer_mm) * 60) + (parseInt(values.timer_ss));

			let lesson_category = null;
			if (typeof values.ld_lesson_category !== 'undefined' && values.ld_lesson_category.length > 0) {
				lesson_category = values.ld_lesson_category.filter(function (item) {
					return parseInt(item);
				})
			}

			let status = values.status;

			// Check if future published date.
			if (new Date().getTime() < getLessonCombinedDate(values.date, values.time).getTime()) {
				status = 'future';
			}

			// Check if private status.
			if ('draft' !== status && 'private' === values.visibility) {
				status = 'private';
			}
			// Check if course review enabled.
			// if ( ir_fcb_loc.instructor_settings?.review_course ) {
			// 	status = 'draft';
			// }

			apiFetch(
				{
					path: "/ldlms/v2/sfwd-lessons",
					method: 'POST',
					data: {
						title: values.title,
						content: tinymce.get("ir-lesson-content-editor").getContent(),
						status: status,
						course: course_id,
						lesson: props.lesson_id,
						slug: values.slug,
						materials: tinymce.get("ir-lesson-material-editor").getContent(),
						materials_enabled: values.materials_enabled,
						video_enabled: values.video_enabled,
						video_url: values.video_url,
						video_shown: values.video_shown,
						video_auto_complete: values.video_auto_complete,
						video_auto_complete_delay: values.video_auto_complete_delay,
						video_show_complete_button: values.video_show_complete_button,
						video_auto_start: values.video_auto_start,
						video_show_controls: values.video_show_controls,
						video_focus_pause: values.video_focus_pause,
						video_resume: values.video_resume,
						assignment_upload_enabled: values.assignment_upload_enabled,
						assignment_points_enabled: values.assignment_points_enabled,
						assignment_points_amount: values.assignment_points_amount,
						assignment_auto_approve: values.assignment_auto_approve,
						assignment_deletion_enabled: values.assignment_deletion_enabled,
						forced_timer_enabled: values.forced_timer_enabled,
						forced_timer_amount: timer_amount,
						assignment_upload_limit_extensions: values.assignment_upload_limit_extensions,
						assignment_upload_limit_size: values.assignment_upload_limit_size,
						assignment_upload_limit_count: values.assignment_upload_limit_count,
						featured_media: values.featured_image,
						visible_type: values.visible_type,
						visible_after: values.visible_after,
						ld_lesson_category: lesson_category,
						ld_lesson_tag: values.tag,
						is_sample: values.is_sample,
						author: values.lesson_author.toString(),
						password: values.password,
						date: getLessonCombinedDate(values.date, values.time)
					}
				}
			).then((response) => {
				if (false != response && response.hasOwnProperty('id')) {
					addNewLesson(response);
					if ('visible_after_specific_date' === values.visible_type) {
						const date_str = values.enrollment_date_yy + "-" + values.enrollment_date_mm + "-" + values.enrollment_date_dd + " " + values.enrollment_date_hh + ":" + values.enrollment_date_min;
						// dataset.visible_after_specific_date = date_str;
						const enrollment_date = new Date(date_str);
						const visible_after_specific_date = Math.trunc(enrollment_date.getTime() / 1000).toString();

						apiFetch(
							{
								path: "/ir/v1/sfwd-courses/" + course_id + "/remaining",
								method: 'POST',
								data: JSON.stringify({
									id: course_id,
									lesson_id: response.id,
									type: 'release_schedule',
									post_type: 'lesson',
									specific_date: visible_after_specific_date
								})
							}
						);
					}
				}
				setLoading(false)
				setLessonOpened(false);
				wp.editor.remove('ir-lesson-content-editor');
				wp.editor.remove('ir-lesson-material-editor');
			});
		}

		const addNewLesson = (lesson) => {
			let lessons_list = [...courseList.t["sfwd-lessons"]];
			lessons_list.push(lesson.id);

			let lesson_tree = { ...courseList.h["sfwd-lessons"] };
			lesson_tree[lesson.id] = {
				"sfwd-topic": [],
				"sfwd-quiz": []
			}

			// Update course data store if not present.
			if (!courseData.hasOwnProperty(lesson.id)) {
				let _courseData = { ...courseData };
				_courseData[lesson.id] = {
					post_title: lesson.title.rendered,
					post_modified: lesson.modified,
					post_status: lesson.status,
					is_sample: lesson.is_sample
				};
				setCourseData(_courseData);
			}

			// Add to Builder
			setCourseList({
				...courseList,
				t: {
					...courseList.t,
					"sfwd-lessons": lessons_list
				},
				h: {
					...courseList.h,
					"sfwd-lessons": lesson_tree
				}
			});

			if ('draft' === lesson.status) {
				// Check if course review enabled.
				// if ( ir_fcb_loc.instructor_settings?.review_course && 'draft' !== original_status) {
				// 	showNotification({
				// 		message: sprintf(__('Your %1$s has been submitted for review to the administrator. Please wait for approval before it can be published. In the meantime, your %1$s is saved as draft.', 'wdm_instructor_role'), ir_fcb_loc.lower_course_label),
				// 		autoClose: 10000
				// 	})
				// } else {
				showNotification({
					title: sprintf(__('Saved as Draft', 'wdm_instructor_role'), ir_fcb_loc.lesson_label),
					message: sprintf(__('Your %s has been successfully saved as a draft', 'wdm_instructor_role'), ir_fcb_loc.lower_lesson_label),
				});
				// }
			} else {
				showNotification({
					title: sprintf(__('%s Published!', 'wdm_instructor_role'), ir_fcb_loc.lesson_label),
					message: sprintf(__('Your %s is successfully published and live now', 'wdm_instructor_role'), ir_fcb_loc.lower_lesson_label),
				});
			}
		}

		const handleErrors = (errors) => {
			if (errors.title) {
				showNotification({ message: 'Please fill lesson title field', color: 'red' });
			}
		};

		/* const openIframeInTab = (device) => {
				var win = window.open();
				win.document.write('<iframe style="width:' + device + '; height:100vh; display:block; margin: auto;" src="http://localhost/elumine/courses/university-physics/" frameborder="0" allowfullscreen></iframe>')
		} */

		const createNewLessonTag = (query) => {
			setShowTagLoader(true);
			// Create New Lesson Tag.
			apiFetch(
				{
					path: "/wp/v2/ld_lesson_tag",
					method: 'POST',
					data: {
						name: query
					}
				}
			).then((response) => {
				if (false != response && response.hasOwnProperty('id')) {
					setShowTagLoader(false);
					const item = { value: response.id.toString(), label: response.name };
					setLessonTags((current) => [...current, item]);
					lessonform.setFieldValue('tag', [...lessonform.values.tag, response.id.toString()])
				}
			});
		}

		const createLessonCategory = () => {
			if (new_category.length === 0) {
				showNotification({
					title: __('Warning', 'wdm_instructor_role'),
					message: __('Category name cannot be empty', 'wdm_instructor_role'),
					icon: <ExclamationMark />,
					color: 'yellow'
				});
				return;
			}

			setCategoryLoader(true);
			let data = {
				name: new_category
			}
			if (null !== parent_category && parent_category.length > 0) {
				data.parent = parent_category;
			}

			// Create New Lesson Category.
			apiFetch(
				{
					path: "/wp/v2/ld_lesson_category",
					method: 'POST',
					data
				}
			).then((response) => {
				if (false != response && response.hasOwnProperty('id')) {
					const item = { value: response.id.toString(), label: response.name };
					setLessonCategories((current) => [...current, item]);
					lessonform.setFieldValue('ld_lesson_category', [...lessonform.values.ld_lesson_category, response.id.toString()])
				}
				setShowCatFields(false);
				setCategoryLoader(false);
				setParentCategory('');
				setNewCategory('');
			});
		}

		const getLessonVisibility = () => {
			let visibility = __('Public', 'wdm_instructor_role');
			if ('password' === lessonform.values.visibility) {
				visibility = __('Password Protected', 'wdm_instructor_role');
			} else if ('private' === lessonform.values.visibility) {
				visibility = __('Private', 'wdm_instructor_role')
			}
			return visibility;
		}

		const handleLessonVisibilityChange = (value) => {
			if ('private' === value) {
				lessonform.setFieldValue('password', '');
				lessonform.setFieldValue('visibility', value);
				lessonform.setFieldValue('status', value);
			} else if ('password' === value) {
				lessonform.setFieldValue('visibility', value);
			} else {
				lessonform.setFieldValue('password', '');
				lessonform.setFieldValue('visibility', value);
			}
		}

		const getLessonPublishedDate = () => {
			let date_string = '';
			const options = { month: 'long', day: 'numeric', hour12: true, hour: 'numeric', minute: 'numeric' };
			const publish_date = getLessonCombinedDate(lessonform.values.date, lessonform.values.time);

			return publish_date.toLocaleDateString(undefined, options);
		}

		const getLessonCombinedDate = (date_object, time_object) => {
			if (date_object instanceof Date && time_object instanceof Date) {
				let combined_date = '';
				combined_date = date_object.getFullYear() + '-' + (date_object.getMonth() + 1) + '-' + date_object.getDate() + ' ' + time_object.getHours() + ':' + time_object.getMinutes() + ':' + time_object.getSeconds();

				return new Date(combined_date);
			} else {
				return new Date();
			}
		}

		useEffect(() => {
			// Fetch lesson tags
			if (lessonOpened) {
				if ("yes" === ir_fcb_loc.ld_cat_tag?.lesson_tag) {
					apiFetch(
						{
							path: "/wp/v2/ld_lesson_tag?per_page=100",
							method: 'GET',
						}
					).then((data) => {
						let tags = [];
						data.map((tag, ind) => {
							tags.push({
								label: tag.name,
								value: tag.id.toString()
							})
						})
						setLessonTags(tags);
					});
				}
				if ("yes" === ir_fcb_loc.ld_cat_tag?.lesson) {
					// Fetch lesson categories
					apiFetch(
						{
							path: "/ir/v1/ld-lesson-category?per_page=100",
							method: 'GET',
						}
					).then((data) => {
						let categories = [];
						data.map((category, ind) => {
							if (category.hasOwnProperty('children')) {
								categories.push({
									label: category.name,
									value: category.id.toString(),
									children: category.children
								});
							} else {
								categories.push({
									label: category.name,
									value: category.id.toString()
								});
							}
						})
						setLessonCategories(categories);
					});
				}
				lessonform.setValues({
					date: new Date(),
					time: new Date(),
				})
			}

		}, [lessonOpened]);

		return (
			<>
				<Modal className="ir-topic-modal"
					opened={lessonOpened}
					onClose={() => {
						setLessonOpened(false);
						wp.editor.remove('ir-lesson-content-editor');
						wp.editor.remove('ir-lesson-material-editor');
					}}
					fullScreen={expand}
					title={ir_fcb_loc.lesson_label}
					size='830px'
				>
					<form onSubmit={lessonform.onSubmit(createNewLesson, handleErrors)}>
						{expand && (

							<Button className="ir-expand-modal ir-btn-subtle" onClick={() => setExpand('')}>
								<ArrowsMinimize
									size={25}
									strokeWidth={2}
									color={'#000000'}
								/>
							</Button>
						)}
						{!expand && (
							<Button className="ir-expand-modal ir-btn-subtle" onClick={() => setExpand('fullScreen')}>
								<ArrowsMaximize
									size={25}
									strokeWidth={2}
									color={'#000000'}
								/>
							</Button>
						)}
						<Tabs defaultValue="Content" styles={{
							tab: {
								color: '#A0A3BD',
								fontSize: "16px",
								'&[data-active]': {
									color: '#000000',
								},
							},

						}}>
							<Tabs.List>
								<Tabs.Tab value="Content">Content</Tabs.Tab>
								<Tabs.Tab value="Material">Material</Tabs.Tab>
								<Tabs.Tab value="settings">Settings</Tabs.Tab>
							</Tabs.List>
							<Tabs.Panel className="ir-panel-overflow" value="Content" pt="lg">
								<Group align="center" mt={10}>
									<TextInput className="ir-tt-text ir-parent-settings ir-flex-1"
										label={sprintf(__("%s Title", "wdm_instructor_role"), ir_fcb_loc.lesson_label)}
										placeholder={__("Write title here", "wdm_instructor_role")}
										size="lg"
										{...lessonform.getInputProps('title')}
									/>
								</Group>
								<Group mb={34} mt={20} p={8} className="ir-featured-image-section">
									{null === lesson_image && (
										<MediaUpload
											onSelect={(media) => {
												setLessonImage({
													id: media.id,
													url: media.url
												});
												lessonform.setFieldValue('featured_image', media.id);
											}}
											allowedTypes={['image']}
											render={({ open }) => (
												<Group>
													<img src={require('/images/featured-placeholder.png')} />
													<Button fw={400} className="ir-btn-subtle" onClick={open} leftIcon={<PhotoUp size={23} strokeWidth={1} />}  >{__('Set Featured Image', 'wdm_instructor_role')}</Button>
												</Group>
											)}
										/>
									)}
									{null !== lesson_image && (
										<Group>
											<div style={{ width: 88, marginLeft: 'auto', marginRight: 'auto' }}>
												<Image
													width={88}
													height={61}
													radius="md"
													src={lesson_image.url}
												/>
											</div>
											<div>
												<MediaUpload
													onSelect={(media) => {
														setLessonImage({
															id: media.id,
															url: media.url
														});
														lessonform.setFieldValue('featured_image', media.id);
													}}
													allowedTypes={['image']}
													value={lesson_image.id}
													render={({ open }) => (
														<>
															<Button fw={400} className="ir-btn-subtle" onClick={open} leftIcon={<Replace size={23} strokeWidth={1} />} >{__('Replace', 'wdm_instructor_role')}</Button>
															<Button fw={400} className="ir-btn-subtle ir-red-txt" onClick={() => {
																setLessonImage(null);
																lessonform.setFieldValue('featured_image', null);
															}} leftIcon={<Trash size={23} strokeWidth={1} />} >{__('Remove', 'wdm_instructor_role')}</Button>
														</>
													)}
												/>
											</div>
										</Group>
									)}
								</Group>
								{/* <Group spacing="xs" mb={34}> */}
								<Text className="ir-parent-settings" size="lg">{__('LESSON CONTENT', 'wdm_instructor_role')}</Text>
								<Textarea
									className="ir-tiny-mce-editors"
									id="ir-lesson-content-editor"
									minRows={15}
									{...lessonform.getInputProps('content')}
								/>
								{/* </Group> */}
								<Group spacing="xs" mb={34} mt={34} className="ir-fc-100">
									<Group position="apart" align="center" mb={10}>
										<Text className="ir-parent-settings" size="lg">{__('ADVANCE CONTENT OPTION', 'wdm_instructor_role')}</Text>
										<Text className="ir-section-name" size="md">{__("You can use only one option at a time", "wdm_instructor_role")}</Text>
									</Group>
									<Accordion width="100%" variant="contained" radius="md" defaultValue="customization"
										onChange={(option) => (lessonform.setValues({
											video_enabled: (option == 'video_enabled') ? true : false,
											assignment_upload_enabled: (option == 'assignment_upload_enabled') ? true : false,
											forced_timer_enabled: (option == 'forced_timer_enabled') ? true : false,
										}))}
									>
										<Accordion.Item value="video_enabled">
											<Group className="ir-acc-left ir-no-chevron" position="apart" align="center">
												<Text size="md" className="ir-d-flex">
													{__('Video Progression', 'wdm_instructor_role')}
													<div className="ir-textinfo">
														<Tooltip label={sprintf(__("Require users to watch the full video as part of the %s progression. Use shortcode [ld_video] to move within the post content.", "wdm_instructor_role"), ir_fcb_loc.lower_course_label)} withArrow>
															<Button><InfoCircle size={20} strokeWidth={2} color={'#444444'} /></Button>
														</Tooltip>
													</div>
												</Text>
												<Accordion.Control><img className="ir-disable" src={require('/images/switch.png')} /><img className="ir-enable" src={require('/images/switchon.png')} /></Accordion.Control>
											</Group>
											<Accordion.Panel className="ir-pl-35">
												<TextInput className="ir-input-settings"
													placeholder={__("Enter your video link/URL here", 'wdm_instructor_role')}
													label={__("Video URL", "wdm_instructor_role")}
													labelPosition="left"
													{...lessonform.getInputProps('video_url')}
												/>

												<Radio.Group mt={35}
													name="Display Timing"
													orientation="vertical"
													label={__("Display Timing", "wdm_instructor_role")}
													defaultValue="BEFORE"
													{...lessonform.getInputProps('video_shown')}
												>
													<Radio value="BEFORE" label={__("Before completed sub-steps", "wdm_instructor_role")} description={sprintf(__("The video will be shown and must be fully watched before the user can access the %s’s associated steps.", "wdm_instructor_role"), ir_fcb_loc.lower_lesson_label)} />
													<Radio value="AFTER" label={__("After completing sub-steps", "wdm_instructor_role")} description={sprintf(__("The video will be visible after the user has completed the %s’s associated steps. The full video must be watched in order to complete the %s.", "wdm_instructor_role"), ir_fcb_loc.lower_lesson_label, ir_fcb_loc.lower_lesson_label)} />
													<div className="ir-conditional-settings pl-0">
														<Group className="ir-acss" spacing={0} align="center" pl={32}>
															<Text className="mantine-InputWrapper-label ir-d-flex">
																{sprintf(__("%s auto-completion", "wdm_instructor_role"), ir_fcb_loc.lesson_label)}
																<div className="ir-textinfo">
																	<Tooltip label={sprintf(__("Automatically mark the %s as completed once the user has watched the full video.", "wdm_instructor_role"), ir_fcb_loc.lower_lesson_label)} withArrow>
																		<Button><InfoCircle size={20} strokeWidth={2} color={'#444444'} /></Button>
																	</Tooltip>
																</div>
															</Text>
															<Switch className="ir-vp-settings ir-acss" labelPosition="left" {...lessonform.getInputProps('video_auto_complete', { type: 'checkbox' })} />
														</Group>
														<Group className="ir-acss" spacing={0} align="center" pl={32} mt={15}>
															<Text className="mantine-InputWrapper-label ir-d-flex">
																{__('Completion delay', 'wdm_instructor_role')}
																<div className="ir-textinfo">
																	<Tooltip label={sprintf(__("Specify a delay between video completion and %s completion.", "wdm_instructor_role"), ir_fcb_loc.lower_lesson_label)} withArrow>
																		<Button><InfoCircle size={20} strokeWidth={2} color={'#444444'} /></Button>
																	</Tooltip>
																</div>
															</Text>
															<TextInput className="ir-input-settings ir-acss"
																placeholder="0"
																labelPosition="left"
																styles={{ input: { width: 85, borderRadius: '6px' } }}
																{...lessonform.getInputProps('video_auto_complete_delay')}
															/>
														</Group>
														<Group className="ir-acss" spacing={0} align="center" pl={32} mt={15} mb={15}>
															<Text className="mantine-InputWrapper-label ir-d-flex">
																{__("Mark Complete Button", 'wdm_instructor_role')}
																<div className="ir-textinfo">
																	<Tooltip label={sprintf(__("Display the Mark Complete button on a %s even if not yet clickable.", "wdm_instructor_role"), ir_fcb_loc.lower_lesson_label)} withArrow>
																		<Button><InfoCircle size={20} strokeWidth={2} color={'#444444'} /></Button>
																	</Tooltip>
																</div>
															</Text>
															<Switch className="ir-vp-settings ir-acss" labelPosition="left" value="Mark Complete Button" {...lessonform.getInputProps('video_show_complete_button', { type: 'checkbox' })} />
														</Group>
													</div>
												</Radio.Group>
												<Switch className="ir-vp-settings" mt={20} labelPosition="left" label={__("Autostart", "wdm_instructor_role")} description={__("The video now starts automatically on page load", "wdm_instructor_role")} {...lessonform.getInputProps('video_auto_start', { type: 'checkbox' })} />
												<Switch className="ir-vp-settings" mt={20} labelPosition="left" label={__("Video Controls Display", "wdm_instructor_role")} description={__("Users can pause, move backward and forward within the video", "wdm_instructor_role")} {...lessonform.getInputProps('video_show_controls', { type: 'checkbox' })} />
												<Switch className="ir-vp-settings" mt={20} labelPosition="left" label={__("Video Pause on Window Unfocused", "wdm_instructor_role")} {...lessonform.getInputProps('video_focus_pause', { type: 'checkbox' })} />
												<Switch className="ir-vp-settings" mt={20} labelPosition="left" label={__("Video resume", "wdm_instructor_role")} {...lessonform.getInputProps('video_resume', { type: 'checkbox' })} />
											</Accordion.Panel>
										</Accordion.Item>

										<Accordion.Item value="assignment_upload_enabled">
											<Group className="ir-acc-left ir-no-chevron" position="apart" align="center">
												<Text size="md" className="ir-d-flex">
													{__('Assignment Uploads', 'wdm_instructor_role')}
												</Text>
												<Accordion.Control><img className="ir-disable" src={require('/images/switch.png')} /><img className="ir-enable" src={require('/images/switchon.png')} /></Accordion.Control>
											</Group>
											<Accordion.Panel className="ir-pl-35">
												<TextInput className="ir-input-settings ir-vp-settings ir-au-settings" mb={24}
													placeholder={__("Enter extension here pdf, xls or zip", "wdm_instructor_role")}
													label={__("File Extensions", "wdm_instructor_role")}
													labelPosition="left"
													{...lessonform.getInputProps('assignment_upload_limit_extensions')}
												/>
												<TextInput className="ir-input-settings ir-vp-settings ir-au-settings" mb={24}
													placeholder={__("512 MB", "wdm_instructor_role")}
													label={__("File Size Limit", "wdm_instructor_role")}
													labelPosition="left"
													{...lessonform.getInputProps('assignment_upload_limit_size')}
												/>
												<Switch className="ir-vp-settings" mt={20} labelPosition="left" label="Points" description={__("Award points for submitting assignments", "wdm_instructor_role")} {...lessonform.getInputProps('assignment_points_enabled', { type: 'checkbox' })} />
												<TextInput className="ir-input-settings ir-vcd-text" pl={32}
													placeholder=""
													label={__("available point(s)", "wdm_instructor_role")}
													labelPosition="left"
													type="number"
													styles={{ input: { width: 85, borderRadius: '6px' } }}
													{...lessonform.getInputProps('assignment_points_amount')}
												/>
												<Radio.Group mt={35}
													name="Grading Type"
													orientation="vertical"
													label={__("Grading Type", "wdm_instructor_role")}
													defaultValue="auto_approve"
													onChange={(value) => (lessonform.setValues({
														assignment_auto_approve: ('auto_approve' === value) ? true : false
													}))}
												>
													<Radio value="auto_approve" label={__("Auto-approve", "wdm_instructor_role")} description={__("No grading or approval needed. The assignment will be automatically approved and full points will be awarded.", "wdm_instructor_role")} />
													<Radio value="manual_grade" label={__("Manually grade", "wdm_instructor_role")} description={sprintf(__("Admin or Group leader approval and grading required. The %s cannot be completed until the assignment is approved", "wdm_instructor_role"), ir_fcb_loc.lower_lesson_label)} />
													<div className="ir-conditional-settings pl-0">
														<TextInput className="ir-input-settings ir-acss" pl={32}
															placeholder=""
															label={__("Limit number of uploaded files", "wdm_instructor_role")}
															labelPosition="left"
															type="number"
															{...lessonform.getInputProps('assignment_upload_limit_count')}
															styles={{ input: { width: 85, borderRadius: '6px' } }}
														/>
														<Switch className="ir-vp-settings ir-acss" pl={32} mt={15} mb={15} labelPosition="left" label="Allow file deletion" {...lessonform.getInputProps('assignment_deletion_enabled', { type: 'checkbox' })} />
													</div>
												</Radio.Group>
											</Accordion.Panel>
										</Accordion.Item>
										<Accordion.Item value="forced_timer_enabled">
											<Group className="ir-acc-left ir-no-chevron" position="apart" align="center">
												<Text size="md" className="ir-d-flex">
													{sprintf(__('%s Timer', 'wdm_instructor_role'), ir_fcb_loc.lesson_label)}
													<div className="ir-textinfo">
														<Tooltip label={__("The module cannot be marked as completed until the set time has elapsed.", "wdm_instructor_role")} withArrow>
															<Button><InfoCircle size={20} strokeWidth={2} color={'#444444'} /></Button>
														</Tooltip>
													</div>
												</Text>
												<Accordion.Control><img className="ir-disable" src={require('/images/switch.png')} /><img className="ir-enable" src={require('/images/switchon.png')} /></Accordion.Control>
											</Group>
											<Accordion.Panel>
												<Group position="center" align="center">
													<NumberInput className="ir-input-settings ir-acss"
														placeholder="HH"
														labelPosition="left"
														type="number"
														min={0}
														styles={{ input: { width: 85, borderRadius: '6px' } }}
														{...lessonform.getInputProps('timer_hh')}
													/>
													<NumberInput className="ir-input-settings ir-acss"
														placeholder="MM"
														labelPosition="left"
														type="number"
														min={0}
														styles={{ input: { width: 85, borderRadius: '6px' } }}
														{...lessonform.getInputProps('timer_mm')}
													/>
													<NumberInput className="ir-input-settings ir-acss"
														placeholder="SS"
														labelPosition="left"
														type="number"
														min={0}
														styles={{ input: { width: 85, borderRadius: '6px' } }}
														{...lessonform.getInputProps('timer_ss')}
													/>
												</Group>
											</Accordion.Panel>
										</Accordion.Item>
									</Accordion>
								</Group>
								<Group spacing="xs" mb={34} className="ir-fc-100">
									<Text className="ir-parent-settings ir-capitalize-letter" size="lg" width="100%" >
										{__('Tags and Categories', 'wdm_instructor_role')}
									</Text>

									<Accordion width="100%" variant="contained" radius="md" defaultValue="customization">

										<Accordion.Item value="ld_lesson_tag">
											<Accordion.Control>
												{sprintf(__('%s Tags', 'wdm_instructor_role'), ir_fcb_loc.lesson_label)}
											</Accordion.Control>
											<Accordion.Panel>
												{"yes" === ir_fcb_loc.ld_cat_tag?.lesson_tag && (
													<MultiSelect className="ir-multiselect-loader"
														icon={showTagLoader && (<Loader size="xs" />)}
														label={sprintf(__("Add %s tags", 'wdm_instructor_role'), ir_fcb_loc.lesson_label)}
														data={lessonTags}
														placeholder={__("Select items", "wdm_instructor_role")}
														searchable
														creatable
														getCreateLabel={(query) => `+ Create ${query}`}
														onCreate={createNewLessonTag}
														{...lessonform.getInputProps('tag')}
													/>
												)}

												{"yes" !== ir_fcb_loc.ld_cat_tag?.lesson_tag && (
													<Text color="dimmed" weight={400}>{sprintf(__("%s tags are disabled from Learndash settings.", 'wdm_instructor_role'), ir_fcb_loc.lesson_label)} </Text>
												)}
											</Accordion.Panel>
										</Accordion.Item>

										<Accordion.Item value="ld_lesson_category">
											<Accordion.Control>
												{sprintf(__('%s Categories', 'wdm_instructor_role'), ir_fcb_loc.lessons_label)}
												<Text size="sm" color="dimmed" weight={400}>
													{__("Child categories cannot be created from here.", "wdm_instructor_role")}
												</Text>
											</Accordion.Control>
											<Accordion.Panel>
												{((ir_fcb_loc.is_admin && ir_fcb_loc.ld_cat_tag.lesson) || (false !== ir_fcb_loc.instructor_settings && ('off' !== ir_fcb_loc.instructor_settings?.enable_ld_category) && ir_fcb_loc.ld_cat_tag.lesson)) && (
													<>
														<Checkbox.Group
															{...lessonform.getInputProps('ld_lesson_category')}
															orientation="vertical"
														>
															{null !== lessonCategories && lessonCategories.map((category, index) => (
																<div>
																	<Checkbox value={category.value} label={category.label} />
																	{category.hasOwnProperty('children') && category.children.map((child) => (
																		<Checkbox ml={20} mt={20} size="md" value={child.id.toString()} label={child.name} />
																	))}
																</div>
															))}
															{null === lessonCategories && (
																<Text>{__('No categories available', 'wdm_instructor_role')}</Text>
															)}
														</Checkbox.Group>
														<Group align="center" spacing="xs" mt={20}>
															<Text className="ir-add-cat" size={14} color="#2067FA" fw={600} onClick={() => setShowCatFields(true)}>{sprintf(__('Add new %s category', 'wdm_instructor_role'), ir_fcb_loc.lower_lesson_label)}</Text>
															{!showCatFields && (
																<Plus onClick={() => setShowCatFields(true)}
																	size={18}
																	strokeWidth={2}
																	color={'#2067FA'}

																/>
															)}
															{showCatFields && (
																<X onClick={() => setShowCatFields(false)}
																	size={18}
																	strokeWidth={2}
																	color={'#2067FA'}

																/>
															)}
														</Group>
														{showCatFields && (
															<div>
																<TextInput
																	placeholder="Enter new category name"
																	value={new_category}
																	onChange={(event) => setNewCategory(event.currentTarget.value)}
																/>
																{/* <Select mt={6}
																																	placeholder="Select Parent Course Category "
																																	data={lessonCategories}
																																	value={parent_category} onChange={setParentCategory}
																															/> */}
																<Button
																	mt={6}
																	className="ir-btn-filled"
																	onClick={createLessonCategory}
																	loading={category_loading}
																>Add</Button>
															</div>
														)}
													</>
												)}
												{((!ir_fcb_loc.is_admin && (false === ir_fcb_loc.instructor_settings || 'off' === ir_fcb_loc.instructor_settings?.enable_ld_category)) || "yes" !== ir_fcb_loc.ld_cat_tag.lesson) && (
													<Text color="dimmed" weight={400}>{sprintf(__("%s categories are disabled from Learndash settings.", 'wdm_instructor_role'), ir_fcb_loc.lesson_label)} </Text>
												)}
											</Accordion.Panel>
										</Accordion.Item>
									</Accordion>
								</Group>

								{ir_fcb_loc.is_admin && (
									<Group spacing="xs" mb={34} className="ir-fc-100">
										<Text className="ir-parent-settings ir-capitalize-letter" size="lg" width="100%" >{__('Author', 'wdm_instructor_role')}</Text>
										<Accordion width="100%" variant="contained" radius="md">
											<Accordion.Item value="lesson-author">
												<Accordion.Control>
													{__('Author', 'wdm_instructor_role')}
												</Accordion.Control>
												<Accordion.Panel>
													<Select mt={6}
														placeholder={sprintf(__("Select %s Author", "wdm_instructor_role"), ir_fcb_loc.lesson_label)}
														data={authors}
														{...lessonform.getInputProps('lesson_author')}
													/>
												</Accordion.Panel>
											</Accordion.Item>
										</Accordion>
									</Group>
								)}

								{(ir_fcb_loc.is_admin || (false !== ir_fcb_loc.instructor_settings && ('off' !== ir_fcb_loc.instructor_settings?.enable_permalinks))) && (
									<Group spacing="xs" mb={34} className="ir-fc-100">
										<Text className="ir-parent-settings" size="lg" width="100%">{sprintf(__('%s URL', 'wdm_instructor_role'), ir_fcb_loc.lesson_label)}</Text>
										<Accordion width="100%" variant="contained" radius="md" defaultValue="customization">

											<Accordion.Item value="permalink">
												<Accordion.Control>{sprintf(__('%s URL', 'wdm_instructor_role'), ir_fcb_loc.lesson_label)}</Accordion.Control>
												<Accordion.Panel>
													<TextInput className="ir-input-settings ir-url-slug"
														placeholder={__("Enter here", 'wdm_instructor_role')}
														label={__("URL Slug", 'wdm_instructor_role')}
														labelPosition="left"
														{...lessonform.getInputProps('slug')}
													/>
												</Accordion.Panel>
											</Accordion.Item>
										</Accordion>
									</Group>
								)}
							</Tabs.Panel>
							<Tabs.Panel className="ir-panel-overflow" value="Material" pt="lg">
								<Switch
									mb={10}
									labelPosition="left"
									label={sprintf(__('%s Material', 'wdm_instructor_role'), ir_fcb_loc.lesson_label)}
									size="lg"
									checked={lessonform.values.materials_enabled}
									onChange={(event) => lessonform.setFieldValue('materials_enabled', event.currentTarget.checked)}
									className={lessonform.values.materials_enabled ? 'ir-checked ir-course-access-p-settings ir-switch ir-parent-settings ir-material' : 'ir-unchecked ir-course-access-p-settings ir-switch ir-parent-settings ir-material'}
									description={__('Any content added below is displayed on a separate tab, alongside your main content.', 'wdm_instructor_role')}
								/>
								<div className="ir-conditional-disabled">
									<Textarea
										className="ir-tiny-mce-editors"
										id="ir-lesson-material-editor"
										minRows={15}
										{...lessonform.getInputProps('materials')}
									/>
								</div>
							</Tabs.Panel>
							<Tabs.Panel className="ir-panel-overflow" value="settings" pt="lg">
								<Group position="apart" align="center" mb={10}>
									<Text size="lg" className="ir-parent-settings">{sprintf(__('%s Access Option', 'wdm_instructor_role'), ir_fcb_loc.lesson_label)}</Text>
									<Text className="ir-section-name" size="md">{sprintf(__("Controls the timing and way %s can be accessed", "wdm_instructor_role"), ir_fcb_loc.lessons_label)}</Text>
								</Group>
								<div className="ir-module-settings">
									<Switch mb={20} className="ir-trs"
										labelPosition="left"
										label={sprintf(__('Sample %s', 'wdm_instructor_role'), ir_fcb_loc.lesson_label)}
										size="lg"
										{...lessonform.getInputProps('is_sample', { type: 'checkbox' })}
									/>
									<Radio.Group className="ir-trs"
										name="LessonRelease"
										orientation="vertical"
										label={sprintf(__("%s Release Schedule", "wdm_instructor_role"), ir_fcb_loc.lesson_label)}
										defaultValue=""
										{...lessonform.getInputProps('visible_type')}
									>
										<Radio value="" label={__('Immediately', 'wdm_instructor_role')} description={sprintf(__("The %s is made available on course enrollment", "wdm_instructor_role"), ir_fcb_loc.lesson_label)} />
										<Radio value="visible_after" label={__("Enrollment-based", "wdm_instructor_role")} description={sprintf(__("The %s will be available X days after course enrollment", "wdm_instructor_role"), ir_fcb_loc.lesson_label)} />
										<TextInput className="ir-input-settings ir-vcd-text" pl={32}
											placeholder=""
											label={__("days(s)", "wdm_instructor_role")}
											labelPosition="right"
											type="number"
											styles={{ input: { width: 85, borderRadius: '10px' } }}
											{...lessonform.getInputProps('visible_after')}
										/>
										<Radio value="visible_after_specific_date" label={__("Specific date", "wdm_instructor_role")} description={sprintf(__("The %s will be available on a specific date.", "wdm_instructor_role"), ir_fcb_loc.lesson_label)} />
										<Group className="ir-sp-date" pl={32}>
											<Select
												placeholder="MM"
												labelPosition="right"
												styles={{ input: { width: 62, borderRadius: '10px' } }}
												defaultValue="0"
												data={[
													{ value: "0", label: __('None', 'wdm_instructor_role') },
													{ value: "1", label: 'Jan' },
													{ value: "2", label: 'Feb' },
													{ value: "3", label: 'Mar' },
													{ value: "4", label: 'Apr' },
													{ value: "5", label: 'May' },
													{ value: "6", label: 'Jun' },
													{ value: "7", label: 'Jul' },
													{ value: "8", label: 'Aug' },
													{ value: "9", label: 'Sep' },
													{ value: "10", label: 'Oct' },
													{ value: "11", label: 'Nov' },
													{ value: "12", label: 'Dec' },
												]}
												{...lessonform.getInputProps('enrollment_date_mm')}
											/>
											<TextInput
												placeholder="DD"
												label=""
												labelPosition="right"
												type="number"
												styles={{ input: { width: 62, borderRadius: '10px' } }}
												defaultValue={0}
												{...lessonform.getInputProps('enrollment_date_dd')}
											/>,
											<TextInput
												placeholder="YYYY"
												label=""
												labelPosition="right"
												type="number"
												styles={{ input: { width: 72, borderRadius: '10px' } }}
												defaultValue={0}
												{...lessonform.getInputProps('enrollment_date_yy')}
											/>@
											<TextInput
												placeholder="HH"
												label=""
												labelPosition="right"
												type="number"
												styles={{ input: { width: 62, borderRadius: '10px' } }}
												defaultValue={0}
												{...lessonform.getInputProps('enrollment_date_hh')}
											/>:
											<TextInput
												placeholder="MN"
												label=""
												labelPosition="right"
												type="number"
												styles={{ input: { width: 62, borderRadius: '10px' } }}
												defaultValue={0}
												{...lessonform.getInputProps('enrollment_date_min')}
											/>
										</Group>
									</Radio.Group>
								</div>
							</Tabs.Panel>
						</Tabs>
						<Group spacing="xs" mt={34} className="ir-tm-bottom-actions" position="apart" align="center">
							<Group>
								{/*loading && (<Loader size="md" variant="dots"/>)*/}
							</Group>
							<Group spacing="xs">
								{/* <Popover width={165} position="bottom" withArrow shadow="lg" radius={6}>
                                    <Popover.Target>
                                        <Button size="md" fw={400} className="ir-btn-subtle" radius="md" variant="subtle" leftIcon={<Eye size={22} />} >
                                            Preview
                                        </Button>
                                    </Popover.Target>
                                    <Popover.Dropdown className="ir-preview-module">
                                        <Button className="ir-btn-subtle" onClick={() => openIframeInTab('100%')} size="sm" variant="subtle" leftIcon={<DeviceDesktop size={22} color={'#000000'} />} >
                                            Desktop view
                                        </Button>
                                        <Button className="ir-btn-subtle" onClick={() => openIframeInTab('768px')} size="sm" variant="subtle" leftIcon={<DeviceTablet size={22} color={'#000000'} />} >
                                            Tablet view
                                        </Button>
                                        <Button className="ir-btn-subtle" onClick={() => openIframeInTab('475px')} size="sm" variant="subtle" leftIcon={<DeviceMobile size={22} color={'#000000'} />} >
                                            Mobile view
                                        </Button>
                                    </Popover.Dropdown>
                                </Popover> */}
								<Popover position="bottom" withArrow shadow="lg" radius={6} width={320}>
									<Popover.Target>
										<Button
											size="sm"
											fw={600}
											className="ir-btn-subtle"
											radius="md"
											variant="subtle"
										>
											{__('Status and Visibility', 'wdm_instructor_role')}
										</Button>
									</Popover.Target>
									<Popover.Dropdown className="ir-status-and-visibility">
										<Accordion width={320}>
											<Accordion.Item value="visibility" variant="filled">
												<Accordion.Control>
													<Group align="center" spacing="xs" position="apart">
														<Text size="sm" fw={600}>{__('Visibility', 'wdm_instructor_role')}</Text>
														<Text size="sm" fw={600} color="#2067FA;">{getLessonVisibility()}</Text>
													</Group>
												</Accordion.Control>
												<Accordion.Panel>
													<Radio.Group
														defaultValue='public'
														orientation='vertical'
														value={lessonform.values.visibility}
														onChange={handleLessonVisibilityChange}
													>
														<Radio value="public" label={__('Public', 'wdm_instructor_role')} description={__('Visible to everyone', 'wdm_instructor_role')} />
														<Radio value="private" label={__('Private', 'wdm_instructor_role')} description={__('Only visible to site admins and editors', 'wdm_instructor_role')} />
														<Radio value="password" label={__('Password Protected', 'wdm_instructor_role')} description={__('Protected with a password you choose. Only those with the password can view this post.', 'wdm_instructor_role')} />
													</Radio.Group>
													{'password' === lessonform.values.visibility && (
														<TextInput
															mt={5}
															placeholder={__('Use a secure Password', 'wdm_instructor_role')}
															size="sm"
															{...lessonform.getInputProps('password')}
														/>
													)}
												</Accordion.Panel>
											</Accordion.Item>

											<Accordion.Item value="publish">
												<Accordion.Control>
													<Group align="center" spacing="xs" position="apart">
														<Text size="sm" fw={600}>{__('Publish', 'wdm_instructor_role')}</Text>
														<Text size="sm" fw={600} color="#2067FA;">{getLessonPublishedDate()}</Text>
													</Group>
												</Accordion.Control>
												<Accordion.Panel>
													<TimeInput
														className='ir-noborder'
														format="12"
														label={__('Time', 'wdm_instructor_role')}
														defaultValue={new Date()}
														icon={<ClockHour4 size={16} />}
														{...lessonform.getInputProps('time')}
													/>
													<DatePicker
														label={__('Date', 'wdm_instructor_role')}
														defaultValue={new Date()}
														icon={<CalendarEvent size={16} />}
														{...lessonform.getInputProps('date')}
													/>
												</Accordion.Panel>
											</Accordion.Item>
										</Accordion>
									</Popover.Dropdown>
								</Popover>

								<Button className="ir-btn-outline" variant="outline" loading={loading} onClick={() => {
									lessonform.setFieldValue('status', 'draft');
								}} type="submit">
									{__('Save to draft', 'wdm_instructor_role')}
								</Button>
								<Button className="ir-btn-filled" variant="filled" loading={loading} type="submit" onClick={() => {
									lessonform.setFieldValue('status', 'publish');
								}}>
									{__('Publish', 'wdm_instructor_role')}
								</Button>
							</Group>
						</Group>
					</form>
				</Modal>

				<Button size="md"
					onClick={() => {
						setLessonOpened(true);
					}}
					className="ir-fed-add-lesson ir-btn-filled"
					radius="md"
					variant="outline"
					leftIcon={<Plus size={22} />}
				// disabled={((null === lessonTags) || (null === lessonCategories)) ? true : false}
				>
					{sprintf(__('Add New %s', 'wdm_instructor_role'), ir_fcb_loc.lesson_label)}
				</Button>
			</>
		);
	}

	const removeLesson = (id, trash = false) => {
		// if ( trash ) {
		let _quizList = { ...quizList };
		let _topicList = { ...topicList };
		// Add topics and quizzes back to library.
		Object.keys(courseList.h["sfwd-lessons"][id]["sfwd-topic"]).map((topic_id) => {
			Object.keys(courseList.h["sfwd-lessons"][id]["sfwd-topic"][topic_id]["sfwd-quiz"]).map((quiz_id) => {
				_quizList[quiz_id] = {
					title: courseData[quiz_id].post_title,
					timestamp: convertToTimestamp(courseData[quiz_id].post_modified),
					status: courseData[quiz_id].post_status
				};
			});
			_topicList[topic_id] = {
				title: courseData[topic_id].post_title,
				timestamp: convertToTimestamp(courseData[topic_id].post_modified),
				status: courseData[topic_id].post_status
			};
		});

		Object.keys(courseList.h["sfwd-lessons"][id]["sfwd-quiz"]).map((quiz_id) => {
			_quizList[quiz_id] = {
				title: courseData[quiz_id].post_title,
				timestamp: convertToTimestamp(courseData[quiz_id].post_modified),
				status: courseData[quiz_id].post_status
			};
		});
		setQuizList(_quizList);
		setTopicList(_topicList);
		// }
		const updated_lessons = courseList.t["sfwd-lessons"].filter(item => item !== id);
		let updated_tree = { ...courseList.h["sfwd-lessons"] };
		delete updated_tree[id];

		let next_lesson = parseInt(jQuery('.ir-acc-' + id).nextAll('.mantine-Accordion-item').find('.ir-builder-lesson').val());

		let updated_sections = [...courseList.sections];

		updated_sections.forEach((section, ind) => {
			if (section.steps[0] === id) {
				if (isNaN(next_lesson)) {
					updated_sections[ind].steps = [];
				} else {
					updated_sections[ind].steps = [next_lesson];
				}
			}
		});

		setCourseList({
			...courseList,
			t: {
				...courseList.t,
				"sfwd-lessons": updated_lessons
			},
			h: {
				...courseList.h,
				"sfwd-lessons": updated_tree
			},
			sections: updated_sections
		});

		// Whether to trash lesson or add back to lesson library.
		if (!trash) {
			let _lessonList = { ...lessonList };
			_lessonList[id] = {
				title: courseData[id].post_title,
				timestamp: convertToTimestamp(courseData[id].post_modified),
				status: courseData[id].post_status,
				is_sample: courseData[id].is_sample
			};
			setLessonList(_lessonList);
		}
	};

	const removeTopic = (id, lesson, trash = false) => {
		// if ( trash ) {
		// Add quizzes back to library.
		let _quizList = { ...quizList };
		Object.keys(courseList.h["sfwd-lessons"][lesson]["sfwd-topic"][id]["sfwd-quiz"]).map((quiz_id) => {
			_quizList[quiz_id] = {
				title: courseData[quiz_id].post_title,
				timestamp: convertToTimestamp(courseData[quiz_id].post_modified),
				status: courseData[quiz_id].post_status
			};
		});
		setQuizList(_quizList);
		// }
		let updated_tree = JSON.parse(JSON.stringify(courseList.h["sfwd-lessons"][lesson]["sfwd-topic"]));
		delete updated_tree[id];
		setCourseList({
			...courseList,
			h: {
				...courseList.h,
				"sfwd-lessons": {
					...courseList.h["sfwd-lessons"],
					[lesson]: {
						...courseList.h["sfwd-lessons"][lesson],
						"sfwd-topic": updated_tree
					}
				}
			}
		});

		// Whether to trash topic or add back to topic library.
		if (!trash) {
			let _topicList = { ...topicList };
			_topicList[id] = {
				title: courseData[id].post_title,
				timestamp: convertToTimestamp(courseData[id].post_modified),
				status: courseData[id].post_status
			};
			setTopicList(_topicList);
		}
	};

	const removeQuiz = (id, trash = false) => {
		/*let parent_type = '';
		let parent1 = 0;
		let parent2 = 0;

		// Check if final quiz removed.
		Object.keys(courseList.h["sfwd-quiz"]).map((key) => {
				if (key === id) {
						parent_type = 'final';
				}
		})

		// 
		if (parent_type.length === 0) {
				Object.keys(courseList.h["sfwd-lessons"]).map((lesson_key) => {
						Object.keys(courseList.h["sfwd-lessons"][lesson_key]["sfwd-quiz"]).map((quiz_key) => {
								if (quiz_key === id) {
										parent_type = 'lesson';
										parent2 = 0;
										parent1 = lesson_key;
								}
						})
				})
		}

		if (parent_type.length === 0) {
				Object.keys(courseList.h["sfwd-lessons"]).map((lesson_key) => {
						Object.keys(courseList.h["sfwd-lessons"][lesson_key]["sfwd-topic"]).map((topic_key) => {
								Object.keys(courseList.h["sfwd-lessons"][lesson_key]["sfwd-topic"][topic_key]["sfwd-quiz"]).map((quiz_key) => {
										if (quiz_key === id) {
												parent_type = 'topic';
												parent2 = topic_key;
												parent1 = lesson_key;
										}
								})
						})
				})
		}*/
		let parent_type = 'final';
		let parent1 = 0;
		let parent2 = 0;
		if (jQuery('#ir-builder-element-' + id).parents('.ir-final-quizzes').length) {
			parent_type = 'final';
			parent1 = 0;
			parent2 = 0;
		} else if (jQuery('#ir-builder-element-' + id).parents('.ir-topics-panel').length) {
			parent_type = 'topic';
			parent2 = parseInt(jQuery('#ir-builder-element-' + id).parents('.ir-topics-panel').siblings().find('.ir-builder-lesson-topic').val());
			parent1 = parseInt(jQuery('#ir-builder-element-' + id).parents('.ir-lessons-panel').siblings().find('.ir-builder-lesson').val());
		} else {
			parent_type = 'lesson';
			parent1 = parseInt(jQuery('#ir-builder-element-' + id).parents('.ir-lessons-panel').siblings().find('.ir-builder-lesson').val());
			parent2 = 0;
		}

		if ('final' === parent_type) {
			let updated_tree = JSON.parse(JSON.stringify(courseList.h["sfwd-quiz"]));
			delete updated_tree[id];
			setCourseList({
				...courseList,
				h: {
					...courseList.h,
					"sfwd-quiz": updated_tree
				}
			});
		} else if ('lesson' === parent_type) {
			let updated_tree = JSON.parse(JSON.stringify(courseList.h["sfwd-lessons"][parent1]["sfwd-quiz"]));
			delete updated_tree[id];
			setCourseList({
				...courseList,
				h: {
					...courseList.h,
					"sfwd-lessons": {
						...courseList.h["sfwd-lessons"],
						[parent1]: {
							...courseList.h["sfwd-lessons"][parent1],
							"sfwd-quiz": updated_tree
						}
					}
				}
			});
		} else if ('topic' === parent_type) {
			let updated_tree = JSON.parse(JSON.stringify(courseList.h["sfwd-lessons"][parent1]["sfwd-topic"][parent2]['sfwd-quiz']));
			delete updated_tree[id];
			setCourseList({
				...courseList,
				h: {
					...courseList.h,
					"sfwd-lessons": {
						...courseList.h["sfwd-lessons"],
						[parent1]: {
							...courseList.h["sfwd-lessons"][parent1],
							"sfwd-topic": {
								...courseList.h["sfwd-lessons"][parent1]["sfwd-topic"],
								[parent2]: {
									...courseList.h["sfwd-lessons"][parent1]["sfwd-topic"][parent2],
									"sfwd-quiz": updated_tree
								}
							}
						}
					}
				}
			});
		}

		// Whether to trash quiz or add back to quiz library.
		if (!trash) {
			let _quizList = { ...quizList };
			_quizList[id] = {
				title: courseData[id].post_title,
				timestamp: convertToTimestamp(courseData[id].post_modified),
				status: courseData[id].post_status
			};
			setQuizList(_quizList);
		}
	};

	const removePost = (id, parent, type) => {
		switch (type) {
			case 'lesson':
				removeLesson(id);
				break;
			case 'topic':
				removeTopic(id, parent);
				break;
			case 'quiz':
				removeQuiz(id);
				break;
		}
	}

	const deletePost = (id, parent, type) => {
		let path_name = '';
		let label = '';
		switch (type) {
			case 'lesson':
				path_name = "sfwd-lessons";
				label = ir_fcb_loc.lesson_label;
				break;
			case 'topic':
				path_name = "sfwd-topic";
				label = ir_fcb_loc.topic_label;
				break;
			case 'quiz':
				path_name = "sfwd-quiz";
				label = ir_fcb_loc.quiz_label;
				break;
		}

		if (0 === path_name.length) {
			return;
		}

		showNotification({
			loading: true,
			id: 'ir-fcc-deleting-data',
			title: sprintf(__('Trashing %s', 'wdm_instructor_role'), label),
			autoClose: false,
			disallowClose: true,
			color: 'red'
		});

		apiFetch(
			{
				path: "/ldlms/v2/" + path_name + "/" + id,
				method: 'DELETE'
			}
		).then((response) => {
			if (false != response && response.hasOwnProperty('id')) {
				updateNotification({
					id: 'ir-fcc-deleting-data',
					title: sprintf(__('%s Trashed', 'wdm_instructor_role'), label),
					message: sprintf(__('%s moved to trash successfully!!', 'wdm_instructor_role'), label)
				})

				switch (type) {
					case 'lesson':
						removeLesson(id, true);
						break;
					case 'topic':
						removeTopic(id, parent, true);
						break;
					case 'quiz':
						removeQuiz(id, true);
						break;
				}
			}
		})
	}

	const addLessonToBuilder = (lesson) => {
		let lessons_list = [...courseList.t["sfwd-lessons"]];
		lessons_list.push(lesson);

		let lesson_tree = { ...courseList.h["sfwd-lessons"] };
		lesson_tree[lesson] = {
			"sfwd-topic": [],
			"sfwd-quiz": []
		}

		// Update course data store if not present.
		if (!courseData.hasOwnProperty(lesson)) {
			let _courseData = { ...courseData };
			_courseData[lesson] = {
				post_title: lessonList[lesson].title,
				post_modified: convertToDate(lessonList[lesson].timestamp),
				post_status: lessonList[lesson].status,
				is_sample: lessonList[lesson].is_sample
			};
			setCourseData(_courseData);
		}

		// Remove from library.
		let lessons_library = { ...lessonList };
		delete lessons_library[lesson];

		setLessonList(lessons_library);

		// Add to Builder
		setCourseList({
			...courseList,
			t: {
				...courseList.t,
				"sfwd-lessons": lessons_list
			},
			h: {
				...courseList.h,
				"sfwd-lessons": lesson_tree
			}
		});

		showNotification({
			title: sprintf(__('%s Added', 'wdm_instructor_role'), ir_fcb_loc.lesson_label),
			message: sprintf(__('%s added to the builder successfully!!', 'wdm_instructor_role'), ir_fcb_loc.lesson_label),
		});
	}

	const addSelectedLessonsToBuilder = () => {
		if (0 === selectedLessons.length) {
			return;
		}

		let lessons_list = [...courseList.t["sfwd-lessons"]];
		let lesson_tree = { ...courseList.h["sfwd-lessons"] };
		let lessons_library = { ...lessonList };
		let _courseData = { ...courseData };
		selectedLessons.map((lesson) => {
			lessons_list.push(parseInt(lesson));

			lesson_tree[lesson] = {
				"sfwd-topic": [],
				"sfwd-quiz": []
			}

			// Update course data store if not present.
			if (!courseData.hasOwnProperty(lesson)) {
				_courseData[lesson] = {
					post_title: lessonList[lesson].title,
					post_modified: convertToDate(lessonList[lesson].timestamp),
					post_status: lessonList[lesson].status,
					is_sample: lessonList[lesson].is_sample
				};
				setCourseData(_courseData);
			}

			// Remove from library.
			delete lessons_library[lesson];
		});

		setLessonList(lessons_library);

		// Add to Builder
		setCourseList({
			...courseList,
			t: {
				...courseList.t,
				"sfwd-lessons": lessons_list
			},
			h: {
				...courseList.h,
				"sfwd-lessons": lesson_tree
			}
		});

		showNotification({
			title: sprintf(__('%s Added', 'wdm_instructor_role'), ir_fcb_loc.lessons_label),
			message: sprintf(__('Selected %s added to the builder successfully!!', 'wdm_instructor_role'), ir_fcb_loc.lower_lesson_label),
		});
		setSelectedLessons([]);
		setCheckedLessons(false);
	}

	const addTopicToBuilder = (topic) => {
		let selected_lesson = parseInt(jQuery('.ir-builder-lesson').first().val());
		if (jQuery('.ir-active .ir-builder-lesson').length > 0) {
			selected_lesson = parseInt(jQuery('.ir-active .ir-builder-lesson').val());
		}

		if (isNaN(selected_lesson)) {
			showNotification({
				title: sprintf(__('No %s found', 'wdm_instructor_role'), ir_fcb_loc.lesson_label),
				message: sprintf(__('Start by adding a new %s first', 'wdm_instructor_role'), ir_fcb_loc.lesson_label),
				icon: <ExclamationMark />,
				color: 'yellow'
			})
		} else {

			let topic_tree = { ...courseList.h["sfwd-lessons"][selected_lesson]["sfwd-topic"] };
			topic_tree[topic] = {
				"sfwd-quiz": []
			}

			// Update course data store if not present.
			let _courseData = { ...courseData };
			if (!_courseData.hasOwnProperty(topic)) {
				_courseData[topic] = {
					post_title: topicList[topic].title,
					post_modified: convertToDate(topicList[topic].timestamp),
					post_status: topicList[topic].status
				};
				setCourseData(_courseData);
			}

			// Remove from library.
			let topic_library = { ...topicList };
			delete topic_library[topic];
			setTopicList(topic_library);

			// Add to Builder
			setCourseList({
				...courseList,
				h: {
					...courseList.h,
					"sfwd-lessons": {
						...courseList.h["sfwd-lessons"],
						[selected_lesson]: {
							...courseList.h["sfwd-lessons"][selected_lesson],
							"sfwd-topic": topic_tree
						}
					}
				}
			});

			showNotification({
				title: sprintf(__('%s Added', 'wdm_instructor_role'), ir_fcb_loc.topic_label),
				message: sprintf(__('%s added to the builder successfully!!', 'wdm_instructor_role'), ir_fcb_loc.topic_label),
			});
		}
	}

	const addSelectedTopicsToBuilder = () => {
		if (0 === selectedTopics.length) {
			return;
		}
		let selected_lesson = parseInt(jQuery('.ir-builder-lesson').first().val());
		if (jQuery('.ir-active .ir-builder-lesson').length > 0) {
			selected_lesson = parseInt(jQuery('.ir-active .ir-builder-lesson').val());
		}

		let topic_tree = { ...courseList.h["sfwd-lessons"][selected_lesson]["sfwd-topic"] };
		let topic_library = { ...topicList };
		let _courseData = { ...courseData };

		selectedTopics.map((topic) => {
			topic_tree[topic] = {
				"sfwd-quiz": []
			}
			// Update course data store if not present.
			if (!courseData.hasOwnProperty(topic)) {
				_courseData[topic] = {
					post_title: topicList[topic].title,
					post_modified: convertToDate(topicList[topic].timestamp),
					post_status: topicList[topic].status
				};
				setCourseData(_courseData);
			}

			// Remove from library.
			delete topic_library[topic];
		});

		setTopicList(topic_library);

		// Add to Builder
		setCourseList({
			...courseList,
			h: {
				...courseList.h,
				"sfwd-lessons": {
					...courseList.h["sfwd-lessons"],
					[selected_lesson]: {
						...courseList.h["sfwd-lessons"][selected_lesson],
						"sfwd-topic": topic_tree
					}
				}
			}
		});

		showNotification({
			title: sprintf(__('%s Added', 'wdm_instructor_role'), ir_fcb_loc.topics_label),
			message: sprintf(__('Selected %s added to the builder successfully!!', 'wdm_instructor_role'), ir_fcb_loc.topics_label),
		});
		setSelectedTopics([]);
		setCheckedTopics(false);
	}

	const addQuizToBuilder = (quiz, is_final = false) => {
		let selected_lesson = parseInt(jQuery('.ir-builder-lesson').first().val());
		if (jQuery('.ir-active .ir-builder-lesson').length > 0) {
			selected_lesson = parseInt(jQuery('.ir-active .ir-builder-lesson').val());
		}
		let quiz_tree = {};
		if (isNaN(selected_lesson) || is_final) {
			quiz_tree = { ...courseList.h["sfwd-quiz"] };
		} else {
			quiz_tree = { ...courseList.h["sfwd-lessons"][selected_lesson]["sfwd-quiz"] };
		}
		quiz_tree[quiz] = {};


		// Update course data store if not present.
		if (!courseData.hasOwnProperty(quiz)) {
			let _courseData = { ...courseData };
			_courseData[quiz] = {
				post_title: quizList[quiz].title,
				post_modified: convertToDate(quizList[quiz].timestamp),
				post_status: quizList[quiz].status
			};
			setCourseData(_courseData);
		}

		// Remove from library.
		let quiz_library = { ...quizList };
		delete quiz_library[quiz];
		setQuizList(quiz_library);

		// Add to Builder
		if (isNaN(selected_lesson) || is_final) {
			setCourseList({
				...courseList,
				h: {
					...courseList.h,
					"sfwd-quiz": quiz_tree
				}
			});
		} else {
			setCourseList({
				...courseList,
				h: {
					...courseList.h,
					"sfwd-lessons": {
						...courseList.h["sfwd-lessons"],
						[selected_lesson]: {
							...courseList.h["sfwd-lessons"][selected_lesson],
							"sfwd-quiz": quiz_tree
						}
					}
				}
			});
		}

		showNotification({
			title: sprintf(__('%s Added', 'wdm_instructor_role'), ir_fcb_loc.quiz_label),
			message: sprintf(__('%s added to the builder successfully!!', 'wdm_instructor_role'), ir_fcb_loc.quiz_label),
		});
	}

	const addQuizToTopicBuilder = (quiz, topic_id, lesson_id) => {
		let quiz_tree = {};
		if (isNaN(quiz) || isNaN(topic_id) || isNaN(lesson_id)) {
			showNotification({
				title: sprintf(__('Some Error Occurred!', 'wdm_instructor_role'), ir_fcb_loc.quiz_label),
				message: sprintf(__('Some error occurred. Please refresh the page and try again', 'wdm_instructor_role'), ir_fcb_loc.quiz_label),
				color: 'red'
			});
			return;
		}

		quiz_tree = { ...courseList.h["sfwd-lessons"][lesson_id]["sfwd-topic"][topic_id]["sfwd-quiz"] };
		quiz_tree[quiz] = {};


		// Update course data store if not present.
		if (!courseData.hasOwnProperty(quiz)) {
			let _courseData = { ...courseData };
			_courseData[quiz] = {
				post_title: quizList[quiz].title,
				post_modified: convertToDate(quizList[quiz].timestamp),
				post_status: quizList[quiz].status
			};
			setCourseData(_courseData);
		}

		// Remove from library.
		let quiz_library = { ...quizList };
		delete quiz_library[quiz];
		setQuizList(quiz_library);

		// Add to Builder 
		setCourseList({
			...courseList,
			h: {
				...courseList.h,
				"sfwd-lessons": {
					...courseList.h["sfwd-lessons"],
					[lesson_id]: {
						...courseList.h["sfwd-lessons"][lesson_id],
						"sfwd-topic": {
							...courseList.h["sfwd-lessons"][lesson_id]["sfwd-topic"],
							[topic_id]: {
								...courseList.h["sfwd-lessons"][lesson_id]["sfwd-topic"][topic_id],
								"sfwd-quiz": quiz_tree
							}
						},
					}
				}
			}
		});

		showNotification({
			title: sprintf(__('%s Added', 'wdm_instructor_role'), ir_fcb_loc.quiz_label),
			message: sprintf(__('%s added to the builder successfully!!', 'wdm_instructor_role'), ir_fcb_loc.quiz_label),
		});
	}

	const addSelectedQuizzesToBuilder = () => {
		if (0 === selectedQuizzes.length) {
			return;
		}
		let selected_lesson = parseInt(jQuery('.ir-builder-lesson').first().val());
		if (jQuery('.ir-active .ir-builder-lesson').length > 0) {
			selected_lesson = parseInt(jQuery('.ir-active .ir-builder-lesson').val());
		}

		let quiz_tree = { ...courseList.h["sfwd-lessons"][selected_lesson]["sfwd-quiz"] };
		let quiz_library = { ...quizList };
		let _courseData = { ...courseData };

		selectedQuizzes.map((quiz) => {
			quiz_tree[quiz] = {}
			// Update course data store if not present.
			if (!courseData.hasOwnProperty(quiz)) {
				_courseData[quiz] = {
					post_title: quizList[quiz].title,
					post_modified: convertToDate(quizList[quiz].timestamp),
					post_status: quizList[quiz].status
				};
				setCourseData(_courseData);
			}

			// Remove from library.
			delete quiz_library[quiz];
		});

		setQuizList(quiz_library);

		// Add to Builder
		setCourseList({
			...courseList,
			h: {
				...courseList.h,
				"sfwd-lessons": {
					...courseList.h["sfwd-lessons"],
					[selected_lesson]: {
						...courseList.h["sfwd-lessons"][selected_lesson],
						"sfwd-quiz": quiz_tree
					}
				}
			}
		});

		showNotification({
			title: sprintf(__('%s Added', 'wdm_instructor_role'), ir_fcb_loc.quiz_label),
			message: sprintf(__('Selected %s added to the builder successfully!!', 'wdm_instructor_role'), ir_fcb_loc.quizzes_label),
		});
		setSelectedQuizzes([]);
		setCheckedQuizzes(false);
	}

	const convertToTimestamp = (date_string = '') => {
		let timestamp = date_string;
		if (timestamp.toString().length > 0) {
			timestamp = new Date(timestamp);
		} else {
			timestamp = new Date();
		}

		return (timestamp.getTime() / 1000).toString();
	}

	const convertToDate = (timestamp = '') => {
		let date_string = timestamp;
		if (date_string.toString().length > 0) {
			date_string = new Date(date_string * 1000);
		} else {
			date_string = new Date();
		}

		return date_string.getFullYear() + '-' + (date_string.getMonth() + 1) + '-' + date_string.getDate() + ' ' + date_string.getHours() + ':' + date_string.getMinutes() + ':' + date_string.getSeconds();
	}

	const getCombinedDate = (date_object, time_object) => {
		if (date_object instanceof Date && time_object instanceof Date) {
			let combined_date = '';
			combined_date = date_object.getFullYear() + '-' + (date_object.getMonth() + 1) + '-' + date_object.getDate() + ' ' + time_object.getHours() + ':' + time_object.getMinutes() + ':' + time_object.getSeconds();

			return new Date(combined_date);
		} else {
			return new Date();
		}
	}

	useEffect(() => {
		fetchCourseData();
		setCourseTitle(document.getElementsByClassName('ir-course-builder-container')[0].dataset.label);
	}, []);

	const openQuizPage = (quiz_id) => {
		window.open(ir_fcb_loc.quiz_builder_url + '/' + quiz_id, '_blank');
	}

	const sortModules = () => {
		jQuery(".ir-sortable-topics").sortable();
		jQuery(".ir-sortable-quizzes").sortable();
		jQuery(".sortable").sortable();
		jQuery('.ir-lessons-checkboxes .ir-draggable > .mantine-Stack-root').sortable();
		jQuery('.ir-topics-checkboxes .ir-draggable > .mantine-Stack-root').sortable();
		jQuery('.ir-quizzes-checkboxes .ir-draggable > .mantine-Stack-root').sortable();
		jQuery(".ir-sortable-topics").sortable('destroy');
		jQuery(".ir-sortable-quizzes").sortable('destroy');
		jQuery(".sortable").sortable('destroy');
		jQuery('.ir-lessons-checkboxes .ir-draggable > .mantine-Stack-root').sortable('destroy');
		jQuery('.ir-topics-checkboxes .ir-draggable > .mantine-Stack-root').sortable('destroy');
		jQuery('.ir-quizzes-checkboxes .ir-draggable > .mantine-Stack-root').sortable('destroy');

		setTimeout(function () {

			jQuery(".ir-sortable-topics").sortable({
				connectWith: [".ir-sortable-topics"],
				dropOnEmpty: true,
				scroll: true,
				scrollSensitivity: 100,
				helper: 'clone',
				stop: function (event, ui) {
					var dropped_id = parseInt(jQuery(ui.item).parents('.ir-lessons-panel').prev().find('.ir-builder-lesson').val());
					jQuery(".ir-sortable-topics").sortable('cancel');
					var dragged_id = parseInt(jQuery(ui.item).parents('.ir-lessons-panel').prev().find('.ir-builder-lesson').val());
					var item_id = parseInt(jQuery(ui.item).find('.ir-builder-lesson-topic').val());
					let dragged_updated_tree = { ...courseList.h["sfwd-lessons"][dragged_id]["sfwd-topic"] };
					let topic_tree = dragged_updated_tree[item_id];
					delete dragged_updated_tree[item_id];
					let dropped_updated_tree = { ...courseList.h["sfwd-lessons"][dropped_id]["sfwd-topic"] };
					dropped_updated_tree[item_id] = topic_tree;
					setCourseList({
						...courseList,
						h: {
							...courseList.h,
							"sfwd-lessons": {
								...courseList.h["sfwd-lessons"],
								[dragged_id]: {
									...courseList.h["sfwd-lessons"][dragged_id],
									"sfwd-topic": dragged_updated_tree
								},
								[dropped_id]: {
									...courseList.h["sfwd-lessons"][dropped_id],
									"sfwd-topic": dropped_updated_tree
								}
							}
						}
					});
				}
			});

			jQuery(".ir-sortable-quizzes").sortable({
				connectWith: [".ir-sortable-quizzes"],
				dropOnEmpty: true,
				scroll: true,
				stop: function (event, ui) {
					// Find Current Item ID.
					var item_id = jQuery(ui.item).find('.ir-builder-quiz').val();
					let destination_type = '';
					let source_type = '';
					let destination_topic = 0;
					let destination_lesson = 0;
					let source_topic = 0;
					let source_lesson = 0;
					let drag_tree = {};
					let drop_tree = {};

					// Find Destination ( Drop ) Details. 
					if (jQuery(ui.item).parents('.ir-topics-panel').length) {
						destination_type = 'topic';
						destination_topic = parseInt(jQuery('#ir-builder-element-' + item_id).parents('.ir-topics-panel').siblings().find('.ir-builder-lesson-topic').val());
						destination_lesson = parseInt(jQuery('#ir-builder-element-' + item_id).parents('.ir-lessons-panel').siblings().find('.ir-builder-lesson').val());
					}
					else if (jQuery(ui.item).parents('.ir-lessons-panel').length) {
						destination_type = 'lesson';
						destination_lesson = parseInt(jQuery('#ir-builder-element-' + item_id).parents('.ir-lessons-panel').siblings().find('.ir-builder-lesson').val());
						destination_topic = 0;
					}
					else if (jQuery(ui.item).parents('.ir-fed-final-quizzes').length) {
						destination_type = 'final';
					}

					// Cancel Sorting.
					jQuery(".ir-sortable-quizzes").sortable('cancel');

					// Find Source ( Drag ) Details. 
					if (jQuery(ui.item).parents('.ir-topics-panel').length) {
						source_type = 'topic';
						source_topic = parseInt(jQuery('#ir-builder-element-' + item_id).parents('.ir-topics-panel').siblings().find('.ir-builder-lesson-topic').val());
						source_lesson = parseInt(jQuery('#ir-builder-element-' + item_id).parents('.ir-lessons-panel').siblings().find('.ir-builder-lesson').val());
					}
					else if (jQuery(ui.item).parents('.ir-lessons-panel').length) {
						source_type = 'lesson';
						source_lesson = parseInt(jQuery('#ir-builder-element-' + item_id).parents('.ir-lessons-panel').siblings().find('.ir-builder-lesson').val());
						source_topic = 0;
					}
					else if (jQuery(ui.item).parents('.ir-fed-final-quizzes').length) {
						source_type = 'final';
					}

					// Handle source to destination.
					if ('final' === source_type) {
						drag_tree = { ...courseList.h["sfwd-quiz"] };
						delete drag_tree[item_id];

						switch (destination_type) {
							case 'final':
								drop_tree = { ...courseList.h['sfwd-quiz'] };
								drop_tree[item_id] = {};
								setCourseList({
									...courseList,
									h: {
										...courseList.h,
										"sfwd-quiz": drop_tree
									}
								});
								break;

							case 'lesson':
								drop_tree = { ...courseList.h["sfwd-lessons"][destination_lesson]['sfwd-quiz'] };
								drop_tree[item_id] = {};
								setCourseList({
									...courseList,
									h: {
										...courseList.h,
										"sfwd-lessons": {
											...courseList.h["sfwd-lessons"],
											[destination_lesson]: {
												...courseList.h["sfwd-lessons"][destination_lesson],
												"sfwd-quiz": drop_tree
											}
										},
										"sfwd-quiz": drag_tree
									}
								});
								break;

							case 'topic':
								drop_tree = { ...courseList.h["sfwd-lessons"][destination_lesson]["sfwd-topic"][destination_topic]['sfwd-quiz'] };
								drop_tree[item_id] = {};
								setCourseList({
									...courseList,
									h: {
										...courseList.h,
										"sfwd-lessons": {
											...courseList.h["sfwd-lessons"],
											[destination_lesson]: {
												...courseList.h["sfwd-lessons"][destination_lesson],
												"sfwd-topic": {
													...courseList.h["sfwd-lessons"][destination_lesson]["sfwd-topic"],
													[destination_topic]: {
														...courseList.h["sfwd-lessons"][destination_lesson]["sfwd-topic"][destination_topic],
														"sfwd-quiz": drop_tree
													}
												}
											}
										},
										"sfwd-quiz": drag_tree
									}
								});
								break;
						}
					} else if ('lesson' === source_type) {
						drag_tree = { ...courseList.h["sfwd-lessons"][source_lesson]["sfwd-quiz"] };
						delete drag_tree[item_id];

						switch (destination_type) {
							case 'final':
								drop_tree = { ...courseList.h['sfwd-quiz'] };
								drop_tree[item_id] = {};
								setCourseList({
									...courseList,
									h: {
										...courseList.h,
										"sfwd-lessons": {
											...courseList.h["sfwd-lessons"],
											[source_lesson]: {
												...courseList.h["sfwd-lessons"][source_lesson],
												"sfwd-quiz": drag_tree
											}
										},

										"sfwd-quiz": drop_tree

									}
								});
								break;

							case 'lesson':
								drop_tree = { ...courseList.h["sfwd-lessons"][destination_lesson]['sfwd-quiz'] };
								drop_tree[item_id] = {};
								setCourseList({
									...courseList,
									h: {
										...courseList.h,
										"sfwd-lessons": {
											...courseList.h["sfwd-lessons"],
											[source_lesson]: {
												...courseList.h["sfwd-lessons"][source_lesson],
												"sfwd-quiz": drag_tree
											},
											[destination_lesson]: {
												...courseList.h["sfwd-lessons"][destination_lesson],
												"sfwd-quiz": drop_tree
											}
										}
									}
								});
								break;

							case 'topic':
								drop_tree = { ...courseList.h["sfwd-lessons"][destination_lesson]["sfwd-topic"][destination_topic]['sfwd-quiz'] };
								drop_tree[item_id] = {};
								if (source_lesson === destination_lesson) {
									setCourseList({
										...courseList,
										h: {
											...courseList.h,
											"sfwd-lessons": {
												...courseList.h["sfwd-lessons"],
												[destination_lesson]: {
													...courseList.h["sfwd-lessons"][destination_lesson],
													"sfwd-topic": {
														...courseList.h["sfwd-lessons"][destination_lesson]["sfwd-topic"],
														[destination_topic]: {
															...courseList.h["sfwd-lessons"][destination_lesson]["sfwd-topic"][destination_topic],
															"sfwd-quiz": drop_tree
														}
													},
													"sfwd-quiz": drag_tree
												}
											}
										}
									});
								}
								else {
									setCourseList({
										...courseList,
										h: {
											...courseList.h,
											"sfwd-lessons": {
												...courseList.h["sfwd-lessons"],
												[source_lesson]: {
													...courseList.h["sfwd-lessons"][source_lesson],
													"sfwd-quiz": drag_tree
												},
												[destination_lesson]: {
													...courseList.h["sfwd-lessons"][destination_lesson],
													"sfwd-topic": {
														...courseList.h["sfwd-lessons"][destination_lesson]["sfwd-topic"],
														[destination_topic]: {
															...courseList.h["sfwd-lessons"][destination_lesson]["sfwd-topic"][destination_topic],
															"sfwd-quiz": drop_tree
														}
													}
												}
											}
										}
									});
								}
								break;
						}
					} else if ('topic' === source_type) {
						drag_tree = { ...courseList.h["sfwd-lessons"][source_lesson]["sfwd-topic"][source_topic]['sfwd-quiz'] };
						delete drag_tree[item_id];

						switch (destination_type) {
							case 'lesson':
								drop_tree = { ...courseList.h["sfwd-lessons"][destination_lesson]['sfwd-quiz'] };
								drop_tree[item_id] = {};
								if (source_lesson === destination_lesson) {
									setCourseList({
										...courseList,
										h: {
											...courseList.h,
											"sfwd-lessons": {
												...courseList.h["sfwd-lessons"],
												[source_lesson]: {
													...courseList.h["sfwd-lessons"][source_lesson],
													"sfwd-topic": {
														...courseList.h["sfwd-lessons"][source_lesson]["sfwd-topic"],
														[source_topic]: {
															...courseList.h["sfwd-lessons"][source_lesson]["sfwd-topic"][source_topic],
															"sfwd-quiz": drag_tree
														}
													},
													"sfwd-quiz": drop_tree
												}
											}
										}
									});
								}
								else {
									setCourseList({
										...courseList,
										h: {
											...courseList.h,
											"sfwd-lessons": {
												...courseList.h["sfwd-lessons"],
												[source_lesson]: {
													...courseList.h["sfwd-lessons"][source_lesson],
													"sfwd-topic": {
														...courseList.h["sfwd-lessons"][source_lesson]["sfwd-topic"],
														[source_topic]: {
															...courseList.h["sfwd-lessons"][source_lesson]["sfwd-topic"][source_topic],
															"sfwd-quiz": drag_tree
														}
													}
												},
												[destination_lesson]: {
													...courseList.h["sfwd-lessons"][destination_lesson],
													"sfwd-quiz": drop_tree
												}
											}
										}
									});
								}
								break;

							case 'topic':
								drop_tree = { ...courseList.h["sfwd-lessons"][destination_lesson]["sfwd-topic"][destination_topic]['sfwd-quiz'] };
								drop_tree[item_id] = {};
								setCourseList({
									...courseList,
									h: {
										...courseList.h,
										"sfwd-lessons": {
											...courseList.h["sfwd-lessons"],
											[source_lesson]: {
												...courseList.h["sfwd-lessons"][source_lesson],
												"sfwd-topic": {
													...courseList.h["sfwd-lessons"][source_lesson]["sfwd-topic"],
													[source_topic]: {
														...courseList.h["sfwd-lessons"][source_lesson]["sfwd-topic"][source_topic],
														"sfwd-quiz": drag_tree
													}
												}
											},
											[destination_lesson]: {
												...courseList.h["sfwd-lessons"][destination_lesson],
												"sfwd-topic": {
													...courseList.h["sfwd-lessons"][destination_lesson]["sfwd-topic"],
													[destination_topic]: {
														...courseList.h["sfwd-lessons"][destination_lesson]["sfwd-topic"][destination_topic],
														"sfwd-quiz": drop_tree
													}
												}
											}
										}
									}
								});
								break;

							case 'final':
								drop_tree = { ...courseList.h['sfwd-quiz'] };
								drop_tree[item_id] = {};
								setCourseList({
									...courseList,
									h: {
										...courseList.h,
										"sfwd-lessons": {
											...courseList.h["sfwd-lessons"],
											[source_lesson]: {
												...courseList.h["sfwd-lessons"][source_lesson],
												"sfwd-topic": {
													...courseList.h["sfwd-lessons"][source_lesson]["sfwd-topic"],
													[source_topic]: {
														...courseList.h["sfwd-lessons"][source_lesson]["sfwd-topic"][source_topic],
														"sfwd-quiz": drag_tree
													}
												}
											},
										},
										"sfwd-quiz": drop_tree

									}
								});
								break;
						}
					}
				}
			});

			jQuery('.ir-lessons-checkboxes .ir-draggable > .mantine-Stack-root').sortable({
				connectWith: [".sortable"],
				dropOnEmpty: true,
				scroll: true,
				start: function (e, ui) {
					jQuery(ui.item).addClass('ir-picked')
				},
				stop: function (e, ui) {
					if (jQuery(ui.item).parent().hasClass('sortable')) {
						jQuery(ui.item).removeClass('ir-picked');
						const lessonid = parseInt(jQuery(ui.item).attr('dataId'));
						jQuery('.ir-lessons-checkboxes .ir-draggable > .mantine-Stack-root').sortable('cancel');
						addLessonToBuilder(lessonid);
						// jQuery(ui.item).hide();
					} else {
						jQuery(ui.item).removeClass('ir-picked');
						// jQuery(ui.item).hide();
					}
				}
			});

			jQuery('.ir-topics-checkboxes .ir-draggable > .mantine-Stack-root').sortable({
				connectWith: [".ir-sortable-topics"],
				dropOnEmpty: true,
				scroll: true,
				start: function (e, ui) {
					jQuery(ui.item).addClass('ir-picked')
				},
				stop: function (e, ui) {
					if (jQuery(ui.item).parent().hasClass('ir-sortable-topics')) {
						jQuery(ui.item).removeClass('ir-picked');
						const topicId = parseInt(jQuery(ui.item).attr('dataId'));
						jQuery('.ir-topics-checkboxes .ir-draggable > .mantine-Stack-root').sortable('cancel');
						addTopicToBuilder(topicId);
						// jQuery(ui.item).hide();
					} else {
						jQuery(ui.item).removeClass('ir-picked');
						// jQuery(ui.item).hide();
					}
				}
			});

			jQuery('.ir-quizzes-checkboxes .ir-draggable > .mantine-Stack-root').sortable({
				connectWith: [".ir-sortable-quizzes"],
				dropOnEmpty: true,
				scroll: true,
				start: function (e, ui) {
					jQuery(ui.item).addClass('ir-picked')
				},
				stop: function (e, ui) {
					if (jQuery(ui.item).parent().hasClass('ir-sortable-quizzes')) {
						const is_final_quiz = jQuery(ui.item).parent().hasClass('ir-final-quizzes');
						const is_topic_quiz = jQuery(ui.item).parent().hasClass('ir-topic-quizzes');

						let topic_id = 0;
						let lesson_id = 0;
						if (is_topic_quiz) {
							topic_id = parseInt(jQuery(ui.item).parents('.ir-topics-panel').siblings().find('.ir-builder-lesson-topic').val());
							lesson_id = parseInt(jQuery(ui.item).parents('.ir-lessons-panel').siblings().find('.ir-builder-lesson').val());
						}

						jQuery(ui.item).removeClass('ir-picked');
						const quizId = parseInt(jQuery(ui.item).attr('dataId'));
						jQuery('.ir-quizzes-checkboxes .ir-draggable > .mantine-Stack-root').sortable('cancel');

						if (is_final_quiz) {
							addQuizToBuilder(quizId, true);
						} else if (is_topic_quiz) {
							addQuizToTopicBuilder(quizId, topic_id, lesson_id);
						} else {
							addQuizToBuilder(quizId);
						}
						// jQuery(ui.item).hide();
					} else {
						jQuery(ui.item).removeClass('ir-picked');
						// jQuery(ui.item).hide();
					}
				}
			});

			jQuery(".sortable").sortable({
				stop: function (event, ui) {
					// const section_id = jQuery(ui.item).find('.ir-section').attr('data-id');
					// const lesson_id = parseInt(jQuery(ui.item).next().find('.ir-builder-lesson').val());

					let course_structure_before = {
						"sfwd-lessons": {},
						"sfwd-quiz": {},
						"section-heading": {}
					};
					course_structure_before = extractBuilderStructure(course_structure_before);
					// console.log(course_structure_before);

					// let prev_item = true;
					// if ( jQuery(ui.item).hasClass('ir-fed-sections') ) {
					//     if( jQuery(ui.item).prev().hasClass('ir-fed-sections') || jQuery(ui.item).next().hasClass('ir-fed-sections') ) {
					//         prev_item = false;
					//     }
					// }

					jQuery(".sortable").sortable('cancel');


					let updated_sections = [];
					Object.keys(course_structure_before['section-heading']).map((item) => {
						updated_sections.push(course_structure_before['section-heading'][item]);
					});

					// if ( prev_item ) {
					setCourseList({
						...courseList,
						sections: updated_sections,
						t: {
							...courseList.t,
							"sfwd-lessons": course_structure_before.lessons_list
						}
					});
					// }
				}
			});

		});
	}

	const searchModule = (event, selector) => {
		const keyword = event.target.value.toLowerCase();
		jQuery(selector + ' .ir-chk-wrap').filter(function () {
			jQuery(this).toggle(jQuery(this).text().toLowerCase().indexOf(keyword) > -1)
		});
	}
	const sortAlphabetically = (type) => {
		// const newList = lessonList.sort((a, b) => a.firstname.localeCompare(b.firstname));
		// const sortable = Object.entries(maxSpeed)
		//     .sort(([,a],[,b]) => a-b)
		//     .reduce((r, [k, v]) => ({ ...r, [k]: v }), {});
		// const newList = Object.values(lessonList).sort((a,b)=>a-b);
		// const newList = Object.entries(lessonList).sort((a,b)=>{
		//       if(b[1] > a[1]) return 1;
		//         else if(b[1] < a[1]) return -1;
		//         else {
		//              if(a[0] > b[0]) return 1;
		//              else if(a[0] < b[0]) return -1;
		//              else return 0
		//       }
		//   });
		// const newList = Object.values(lessonList).sort().reduce(
		//   (obj, key) => { 
		//     obj = lessonList[obj]; 
		//     return obj;
		//   }, 
		//   {}
		// );
		//  const getKeyByValue = (object, value) => {
		//    return Object.keys(object).find(key => object[key] === value);
		// };
		// const sorted = Object.values(lessonList).sort();
		// let newList = {};
		// Object.keys(lessonList).forEach(function (value, i) {
		//      newList[getKeyByValue(lessonList,sorted[i])] = sorted[i];
		// });

		jQuery('.ir-recent-' + type).removeClass('ir-btn-grey');
		jQuery('.ir-view-all-' + type).addClass('ir-btn-grey');
		var mylist = jQuery('.ir-' + type + '-checkboxes .mantine-CheckboxGroup-root > .mantine-Stack-root');
		var listitems = mylist.children('.ir-chk-wrap').get();
		listitems.sort(function (a, b) {
			return jQuery(a).text().toUpperCase().localeCompare(jQuery(b).text().toUpperCase());
		})
		jQuery.each(listitems, function (idx, itm) { mylist.append(itm); });
	}

	const showRecentContent = (e, type) => {
		jQuery('.ir-view-all-' + type).removeClass('ir-btn-grey');
		jQuery('.ir-recent-' + type).addClass('ir-btn-grey');
		jQuery(".ir-" + type + "-checkboxes .mantine-Stack-root .ir-chk-wrap").sort(sort_li).appendTo('.ir-' + type + '-checkboxes .mantine-Stack-root');
		function sort_li(a, b) {
			return (jQuery(a).data('timestamp')) < (jQuery(b).data('timestamp')) ? 1 : -1;
		}
	}

	const html_entity_decode = (message) => {
		var domParser = new DOMParser();
		var decodedString = domParser.parseFromString(message, 'text/html').body.textContent;
		return decodedString;
	}

	useEffect(() => {
		sortModules();
	}, [courseList, lessonList, topicList, quizList]);

	useEffect(() => {
		setTimeout(function () {
			wp.editor.initialize(
				'ir-lesson-content-editor',
				{
					tinymce: {
						wpautop: true,
						height: '300px',
						plugins: 'charmap colorpicker compat3x directionality hr image lists media paste tabfocus textcolor wordpress wpautoresize wpdialogs wpeditimage link',
						toolbar1: 'formatselect bold italic underline strikethrough | bullist numlist | blockquote hr wp_more | alignleft aligncenter alignright | link unlink | media wp_add_media'
					},
					quicktags: false,
					mediaButtons: true,
				}
			);
			wp.editor.initialize(
				'ir-lesson-material-editor',
				{
					tinymce: {
						wpautop: true,
						height: '300px',
						plugins: 'charmap colorpicker compat3x directionality hr image lists media paste tabfocus textcolor wordpress wpautoresize wpdialogs wpeditimage link',
						toolbar1: 'formatselect bold italic underline strikethrough | bullist numlist | blockquote hr wp_more | alignleft aligncenter alignright | link unlink | media wp_add_media'
					},
					quicktags: false,
					mediaButtons: true,
				}
			);
		});
	}, [lessonOpened]);

	const openIframeInTab = (device, url) => {
		var win = window.open();
		win.document.write('<iframe style="width:' + device + '; height:100vh; display:block; margin: auto;" src="' + url + '" frameborder="0" allowfullscreen></iframe>')
	}

	const checkIfchecked = (type) => {
		setTimeout(function () {
			switch (type) {
				case 'lesson':
					const checkedLessonItems = jQuery('.ir-lessons-checkboxes .mantine-Checkbox-root[data-checked=true]');
					if (checkedLessonItems.length > 0) {
						setCheckedLessons(true);
					}
					else {
						setCheckedLessons(false);
					}
					break;

				case 'topic':
					const checkedTopicItems = jQuery('.ir-topics-checkboxes .mantine-Checkbox-root[data-checked=true]');
					if (checkedTopicItems.length > 0) {
						setCheckedTopics(true);
					}
					else {
						setCheckedTopics(false);
					}
					break;

				case 'quiz':
					const checkedQuizItems = jQuery('.ir-quizzes-checkboxes .mantine-Checkbox-root[data-checked=true]');
					if (checkedQuizItems.length > 0) {
						setCheckedQuizzes(true);
					}
					else {
						setCheckedQuizzes(false);
					}
					break;
			}
		})
	}

	const confirmDraft = () => openConfirmModal({
		modal: 'switch-draft',
		title: __('Switch to draft', 'wdm_instructor_role'),
		children: (
			<Text size="sm">
				{sprintf(__('Are you sure you want to Unpublish this %s?', 'wdm_instructor_role'), ir_fcb_loc.course_label)}
				{__('This will be saved as draft.', 'wdm_instructor_role')}
			</Text>
		),
		labels: { confirm: 'Yes', cancel: 'Cancel' },
		confirmProps: { color: 'red' },
		modalId: "bgdvbsj",
		onCancel: () => console.log('Cancel'),
		onConfirm: () => updateCourseData('draft'),
	});

	const coursePublished = (link) => openConfirmModal({
		modal: 'course-published',
		centered: true,
		children: (
			<>
				<CircleCheck
					size={95}
					strokeWidth={1}
					color={'#4DCE11'}
				/>
				<Text size={18} fw={600}>
					{sprintf(__('Congratulations your %s is published!', 'wdm_instructor_role'), ir_fcb_loc.course_label)}
				</Text>
			</>
		),
		labels: { confirm: sprintf(__('View %s', 'wdm_instructor_role'), ir_fcb_loc.lower_course_label), cancel: 'Back' },
		confirmProps: { color: '#2067FA' },
		modalId: "bgdvbsj",
		onCancel: () => console.log('Cancel'),
		onConfirm: () => window.open(link, '_blank'),
	});

	const confirmDelete = (content_id, content_type, parent_id = false) => openConfirmModal({
		modal: 'delete-module',
		centered: true,
		children: (
			<>
				<Group align="top" noWrap>
					<XboxX
						size={28}
						strokeWidth={1}
						color={'#A0A3BD'}
					/>
					<Text size={18} fw={400}>
						<strong>{__('Remove :', 'wdm_instructor_role')}</strong> {sprintf(__('Removes the content from the %s builder and can be found in the %s library', 'wdm_instructor_role'), ir_fcb_loc.lower_course_label, ir_fcb_loc.lower_course_label)}
					</Text>
				</Group>
				<Group align="top" noWrap>
					<Trash
						size={28}
						strokeWidth={1}
						color={'#A0A3BD'}
					/>
					<Text size={18} fw={400}>
						<strong>{__('Trash :', 'wdm_instructor_role')}</strong> {__('It moves the content to trash', 'wdm_instructor_role')}
					</Text>
				</Group>

			</>
		),
		labels: { confirm: __('Remove', 'wdm_instructor_role'), cancel: __('Trash', 'wdm_instructor_role') },
		confirmProps: { color: '#2067FA' },
		modalId: "bgdvbsj",
		onCancel: () => deletePost(content_id, parent_id, content_type),
		onConfirm: () => removePost(content_id, parent_id, content_type),
	});

	const trashCourse = () => openConfirmModal({
		modal: 'switch-draft',
		title: __('Move to Trash', 'wdm_instructor_role'),
		children: (
			<>
				<Text size="sm">
					{sprintf(__('Are you sure you want to trash this %s?', 'wdm_instructor_role'), ir_fcb_loc.course_label)}
				</Text>
				<Text size="sm">
					{sprintf(__('The %s will be moved to trash and you will be redirected to the all %s listing page', 'wdm_instructor_role'), ir_fcb_loc.lower_course_label, ir_fcb_loc.courses_label)}
				</Text>
			</>
		),
		labels: { confirm: 'Yes', cancel: 'Cancel' },
		confirmProps: { color: '#2067FA' },
		modalId: "bgdvbsj",
		closeOnConfirm: false,
		onCancel: () => console.log('Cancel'),
		onConfirm: () => {
			openConfirmModal({
				modal: 'switch-draft',
				title: sprintf(__('%s is being moved to trash', 'wdm_instructor_role'), ir_fcb_loc.course_label),
				labels: { confirm: 'Close modal', cancel: 'Back' },
				closeOnConfirm: false,
				confirmProps: { id: 'ir-hidden' },
				cancelProps: { id: 'ir-hidden' },
				children: (
					<>
						<Group align="center">
							<Text size="md">
								{sprintf(__('Please wait!!', 'wdm_instructor_role'))}
							</Text>
							<Loader size="sm" />
						</Group>
					</>
				)

			}),
				apiFetch(
					{
						path: "/ldlms/v2/sfwd-courses/" + course_id,
						method: 'DELETE'
					}
				).then((response) => {
					if (false != response && response.hasOwnProperty('id')) {
						openConfirmModal({
							modal: 'switch-draft',
							title: sprintf(__('%s moved to trash successfully', 'wdm_instructor_role'), ir_fcb_loc.course_label),
							labels: { confirm: 'Close modal', cancel: 'Back' },
							closeOnConfirm: false,
							confirmProps: { id: 'ir-hidden' },
							cancelProps: { id: 'ir-hidden' },
							children: (
								<Text size="sm">
									{sprintf(__('You will now be redirected to the all %s listing page!!', 'wdm_instructor_role'), ir_fcb_loc.course_label, ir_fcb_loc.lower_course_label)}
								</Text>
							),
							onConfirm: closeAllModals,
						}),

							window.location.replace(all_courses_link);
					}
				}).catch((error) => {
					if (error.hasOwnProperty('code') && 'rest_cannot_delete' === error.code) {
						openConfirmModal({
							modal: 'switch-draft',
							title: sprintf(__('%s not Trashed', 'wdm_instructor_role'), ir_fcb_loc.course_label),
							labels: { confirm: 'Close modal', cancel: 'Cancel' },
							closeOnConfirm: false,
							confirmProps: { id: 'ir-hidden' },
							cancelProps: { id: 'ir-hidden' },
							children: (
								<Text size="sm">
									{sprintf(__('You are not allowed to trash this %s.', 'wdm_instructor_role'), ir_fcb_loc.course_label)}
								</Text>
							),
							onConfirm: closeAllModals,
							onCancel: closeAllModals
						})
					}
				})
		},
	});

	const getPublishedDate = () => {
		let date_string = '';
		const options = { month: 'long', day: 'numeric', hour12: true, hour: 'numeric', minute: 'numeric' };
		const publish_date = getCombinedDate(courseform.values.date, courseform.values.time);


		return publish_date.toLocaleDateString(undefined, options);
	}

	const getVisibility = () => {
		let visibility = __('Public', 'wdm_instructor_role');
		if ('password' === courseform.values.visibility) {
			visibility = __('Password Protected', 'wdm_instructor_role');
		} else if ('private' === courseform.values.visibility) {
			visibility = __('Private', 'wdm_instructor_role')
		}
		return visibility;
	}

	const setVisibility = (data) => {
		// Set course visibility
		let _visibility = data.status;
		if (data.content.protected) {
			_visibility = 'password';
		} else if ('private' === data.status) {
			_visibility = 'private';
			// Remove keyword Private from title.
			// if ( -1 !== data.title.rendered.indexOf('Private:')) {
			// 	const post_title = data.title.rendered;
			// 	courseform.setFieldValue('title', post_title.replace('Private:', ''));
			// }
		} else {
			_visibility = 'public';
		}

		courseform.setFieldValue('visibility', _visibility);
	}

	const handleVisibilityChange = (value) => {
		if ('private' === value) {
			courseform.setFieldValue('password', '');
			openConfirmModal({
				modal: 'switch-draft',
				title: sprintf(__('Publish %s Privately?', 'wdm_instructor_role'), ir_fcb_loc.course_label),
				children: (
					<Text size="sm">
						{sprintf(__('Would you like to privately publish this %s now?', 'wdm_instructor_role'), ir_fcb_loc.lower_course_label)}
					</Text>
				),
				labels: { confirm: 'Confirm', cancel: 'Cancel' },
				onCancel: () => console.log('Cancel'),
				onConfirm: () => {
					setLoading(true);
					// Update course settings.
					apiFetch(
						{
							path: "/ldlms/v2/sfwd-courses/" + course_id,
							method: 'POST',
							data: {
								status: value,
							}
						}
					).then((resp) => {
						setLoading(false);
						courseform.setFieldValue('visibility', value);
					})
				},
			});
		} else if ('password' === value) {
			courseform.setFieldValue('visibility', value);
		} else {
			courseform.setFieldValue('password', '');
			courseform.setFieldValue('visibility', value);
		}
	}

	return (
		<MantineProvider theme={{ colorScheme: 'light' }}>
			<NotificationsProvider>
				<ModalsProvider>
					<div className="ir-fed-header" height={150} span={4} offset={8} mb={40}>
						<Group className="ir-header-top" height='80px' position='apart' align="center" pl={25} pr={35}>
							<Group align="center">
								<Button component="a" href={all_courses_link} variant="subtle" leftIcon={<ChevronLeft size={20} />}>
									{sprintf(__('All %s', 'wdm_instructor_role'), ir_fcb_loc.courses_label)}
								</Button>
								{0 < course_title.length && (
									<Title className="ir-course-title" order={2} color="#000000">
										{html_entity_decode(course_title)}
										{'publish' === courseform.values.status &&
											<Badge className="ir-published" color="#00B61D" variant="light" ml={15}>
												{__('Published', 'wdm_instructor_role')}
											</Badge>
										}
										{'draft' === courseform.values.status &&
											<Badge className="ir-drafted" color="#00B61D" ml={15}>
												{__('Draft', 'wdm_instructor_role')}
											</Badge>
										}
									</Title>

								)}
								{0 === course_title.length && (
									<Title className="ir-course-title" order={2} color="#A0A3BD">
										{sprintf(__('%s Title', 'wdm_instructor_role'), ir_fcb_loc.course_label)}
									</Title>
								)}
							</Group>
							<Group>
								<Button
									p={0}
									size="sm"
									fw={600}
									className="ir-btn-subtle ir-red-txt"
									radius="md"
									variant="subtle"
									leftIcon={<Trash size={22} />}
									onClick={trashCourse}
									disabled={(null === courseList) ? true : false}
								>
									{__('Move to trash', 'wdm_instructor_role')}
								</Button>
								<Popover width={165} position="bottom" withArrow shadow="lg" radius={6}>
									<Popover.Target>
										<Button
											size="sm"
											fw={600}
											className="ir-btn-subtle"
											radius="md"
											variant="subtle"
											leftIcon={<Eye size={22} />}
											disabled={(null === courseList) ? true : false}
										>
											{sprintf(__('View %s', 'wdm_instructor_role'), ir_fcb_loc.course_label)}
										</Button>
									</Popover.Target>
									<Popover.Dropdown className="ir-preview-module">
										<Button className="ir-btn-subtle" onClick={() => openIframeInTab('100%', ir_fcb_loc.course_preview_link)} size="sm" variant="subtle" leftIcon={<DeviceDesktop size={22} color={'#000000'} />} >
											{__('Desktop view', 'wdm_instructor_role')}
										</Button>
										<Button className="ir-btn-subtle" onClick={() => openIframeInTab('768px', ir_fcb_loc.course_preview_link)} size="sm" variant="subtle" leftIcon={<DeviceTablet size={22} color={'#000000'} />} >
											{__('Tablet view', 'wdm_instructor_role')}
										</Button>
										<Button className="ir-btn-subtle" onClick={() => openIframeInTab('475px', ir_fcb_loc.course_preview_link)} size="sm" variant="subtle" leftIcon={<DeviceMobile size={22} color={'#000000'} />} >
											{__('Mobile view', 'wdm_instructor_role')}
										</Button>
									</Popover.Dropdown>
								</Popover>
								<Popover position="bottom" withArrow shadow="lg" radius={6} width={320}>
									<Popover.Target>
										<Button
											size="sm"
											fw={600}
											className="ir-btn-subtle"
											radius="md"
											variant="subtle"
											disabled={(null === courseList) ? true : false}
										>
											{__('Status and Visibility', 'wdm_instructor_role')}
										</Button>
									</Popover.Target>
									<Popover.Dropdown className="ir-status-and-visibility">
										<Accordion width={320}>
											<Accordion.Item value="visibility" variant="filled">
												<Accordion.Control>
													<Group align="center" spacing="xs" position="apart">
														<Text size="sm" fw={600}>{__('Visibility', 'wdm_instructor_role')}</Text>
														<Text size="sm" fw={600} color="#2067FA;">{getVisibility()}</Text>
													</Group>
												</Accordion.Control>
												<Accordion.Panel>
													<Radio.Group
														defaultValue='public'
														orientation='vertical'
														value={courseform.values.visibility}
														onChange={handleVisibilityChange}
													>
														<Radio value="public" label={__('Public', 'wdm_instructor_role')} description={__('Visible to everyone', 'wdm_instructor_role')} />
														<Radio value="private" label={__('Private', 'wdm_instructor_role')} description={__('Only visible to site admins and editors', 'wdm_instructor_role')} />
														<Radio value="password" label={__('Password Protected', 'wdm_instructor_role')} description={__('Protected with a password you choose. Only those with the password can view this post.', 'wdm_instructor_role')} />
													</Radio.Group>
													{'password' === courseform.values.visibility && (
														<TextInput
															mt={5}
															placeholder={__('Use a secure Password', 'wdm_instructor_role')}
															size="sm"
															{...courseform.getInputProps('password')}
														/>
													)}
												</Accordion.Panel>
											</Accordion.Item>

											<Accordion.Item value="publish">
												<Accordion.Control>
													<Group align="center" spacing="xs" position="apart">
														<Text size="sm" fw={600}>{__('Publish', 'wdm_instructor_role')}</Text>
														<Text size="sm" fw={600} color="#2067FA;">{getPublishedDate()}</Text>
													</Group>
												</Accordion.Control>
												<Accordion.Panel>
													<TimeInput
														className='ir-noborder'
														format="12"
														label={__('Time', 'wdm_instructor_role')}
														defaultValue={new Date()}
														icon={<ClockHour4 size={16} />}
														{...courseform.getInputProps('time')}
													/>
													<DatePicker
														label={__('Date', 'wdm_instructor_role')}
														defaultValue={new Date()}
														icon={<CalendarEvent size={16} />}
														{...courseform.getInputProps('date')}
													/>
												</Accordion.Panel>
											</Accordion.Item>
										</Accordion>
									</Popover.Dropdown>
								</Popover>
								<Button
									className="ir-btn-outline"
									variant="outline"
									radius="md"
									size="sm"
									onClick={() => {
										{ ('publish' === courseform.values.status || 'private' === courseform.values.status) && confirmDraft() }
										{ 'draft' === courseform.values.status && updateCourseData('draft') }
									}}
									loading={drafting}
									disabled={(null === courseList) ? true : false}
								>
									{('publish' === courseform.values.status || 'private' === courseform.values.status || 'future' === courseform.values.status) && __('Switch to Draft', 'wdm_instructor_role')}
									{'draft' === courseform.values.status && __('Save to Draft', 'wdm_instructor_role')}
								</Button>
								<Button
									className="ir-publish ir-btn-filled"
									variant="filled" radius="md"
									size="sm"
									color="#2067FA"
									onClick={() => { updateCourseData('publish'); }}
									loading={loading}
									disabled={(null === courseList) ? true : false}
								>
									{'draft' === courseform.values.status && __('Publish', 'wdm_instructor_role')}
									{('publish' === courseform.values.status || 'private' === courseform.values.status) && __('Update', 'wdm_instructor_role')}
									{'future' === courseform.values.status && __('Schedule', 'wdm_instructor_role')}
								</Button>
							</Group>
						</Group>
						<Group className="ir-header-bottom" position='left' align="center">
							<Tabs defaultValue="ir-course-page" className="ir-fed-tabs">
								<Tabs.List>
									<Tabs.Tab value="ir-course-page">{sprintf(__('%s Page', 'wdm_instructor_role'), ir_fcb_loc.course_label)}</Tabs.Tab>
									<Tabs.Tab value="ir-builder">
										{__('Builder', 'wdm_instructor_role')}
									</Tabs.Tab>
									<Tabs.Tab value="ir-settings">
										{__('Settings', 'wdm_instructor_role')}
									</Tabs.Tab>
									<Tabs.Tab value="ir-group-settings">
										{sprintf(__('%s', 'wdm_instructor_role'), ir_fcb_loc.groups_label)}
									</Tabs.Tab>
								</Tabs.List>
								<Tabs.Panel className="ir-main-panel" value="ir-course-page">
									<CoursePage course_id={course_id} form={courseform} course_title={course_title} setCourseTitle={setCourseTitle} instructorSettings={instructorSettings} setInstructorSettings={setInstructorSettings} review_data={review_data} setReviewData={setReviewData} />
								</Tabs.Panel>
								<Tabs.Panel className="ir-main-panel" value="ir-builder">
									<Grid>
										<Grid.Col span={8}>
											<div id="sortable" className='ir-builder'>
												<Paper radius="xl" p="xl" className="ir-fed-builder ir-fed-course-content">
													<Group position="apart" align="center">
														<Title size={32} order={2}>{sprintf(__('%s Content', 'wdm_instructor_role'), ir_fcb_loc.course_label)}</Title>
														{null !== courseList && Object.keys(courseList.h["sfwd-lessons"]).length > 0 && expanded &&
															<Button onClick={() => setExpanded(false)} className="ir-btn-subtle ir-collapse-all" variant="subtle" leftIcon={<ChevronUp size={18} />}>Collapse All</Button>
														}
														{null !== courseList && Object.keys(courseList.h["sfwd-lessons"]).length > 0 && !expanded &&
															<Button onClick={() => setExpanded(true)} className="ir-btn-subtle ir-expand-all" variant="subtle" leftIcon={<ChevronDown size={18} />}>Expand All</Button>
														}
													</Group>

													<Accordion className="sortable ir-fed-container" chevron={<ChevronDown size={28} color="#2067FA" />} styles={{
														item: {
															// styles added to all items
															backgroundColor: '#EFF0F7',
															borderRadius: '15px',
															marginBottom: "16px",
															padding: "0px",
															// styles added to expanded item
															// '&[data-active]': {
															//   backgroundColor: '#EFF0F7',
															// },
														},

														chevron: {
															width: "28px",
															marginLeft: "10px",
															// styles added to chevron when it should rotate
															'&[data-rotate]': {
																transform: 'rotate(-90deg)',
															},
														},
													}}

														multiple>
														{null === courseList && (
															<>
																<Paper className="ir-fed-topics ir-skeleton" mb={16} radius="lg" p="md" withBorder>
																	<Group align="center" position="apart">
																		<Group align="center" spacing="xs">
																			<GridDots
																				size={20}
																				strokeWidth={2}
																				color={'#2067FA'}
																			/>
																			<Center style={{ width: 20, height: 20, backgroundColor: '#19A2D0', borderRadius: 6 }}>
																				<Skeleton height={20} width={20} radius="md" />
																			</Center>
																			<div>
																				<Skeleton height={16} width={120} radius="md" />
																			</div>
																		</Group>
																		<Group align="center" spacing="sm" pr={15}>
																			<PencilMinus
																				size={24}
																				strokeWidth={2}
																				color={'#2067FA'}
																			/>
																			<X
																				size={24}
																				strokeWidth={2}
																				color={'#2067FA'}
																			/>
																			<ChevronDown
																				size={24}
																				strokeWidth={2}
																				color={'#2067FA'}
																			/>

																		</Group>
																	</Group>
																</Paper>
																<Paper className="ir-fed-topics ir-skeleton" mb={16} radius="lg" p="md" withBorder>
																	<Group align="center" position="apart">
																		<Group align="center" spacing="xs">
																			<GridDots
																				size={20}
																				strokeWidth={2}
																				color={'#2067FA'}
																			/>
																			<Center style={{ width: 20, height: 20, backgroundColor: '#19A2D0', borderRadius: 6 }}>
																				<Skeleton height={20} width={20} radius="md" />
																			</Center>
																			<div>
																				<Skeleton height={16} width={180} radius="md" />
																			</div>
																		</Group>
																		<Group align="center" spacing="sm" pr={15}>
																			<PencilMinus
																				size={24}
																				strokeWidth={2}
																				color={'#2067FA'}
																			/>
																			<X
																				size={24}
																				strokeWidth={2}
																				color={'#2067FA'}
																			/>
																			<ChevronDown
																				size={24}
																				strokeWidth={2}
																				color={'#2067FA'}
																			/>
																		</Group>
																	</Group>
																</Paper>
																<Paper className="ir-fed-topics ir-skeleton" mb={16} radius="lg" p="md" withBorder>
																	<Group align="center" position="apart">
																		<Group align="center" spacing="xs">
																			<GridDots
																				size={20}
																				strokeWidth={2}
																				color={'#2067FA'}
																			/>
																			<Center style={{ width: 20, height: 20, backgroundColor: '#19A2D0', borderRadius: 6 }}>
																				<Skeleton height={20} width={20} radius="md" />
																			</Center>
																			<div>
																				<Skeleton height={16} width={200} radius="md" />
																			</div>
																		</Group>
																		<Group align="center" spacing="sm"  >
																			<Group className="ir-br-2" pr={15}>
																				<PencilMinus
																					size={24}
																					strokeWidth={2}
																					color={'#2067FA'}
																				/>
																				<X
																					size={24}
																					strokeWidth={2}
																					color={'#2067FA'}
																				/>
																			</Group>
																			<Text className="ir-section-name" fz="sm" color="#555552" fw={400}>{__('Section', 'wdm_instructor_role')}</Text>
																		</Group>
																	</Group>
																</Paper>
																<Paper className="ir-fed-topics ir-skeleton" mb={16} radius="lg" p="md" withBorder>
																	<Group align="center" position="apart">
																		<Group align="center" spacing="xs">
																			<GridDots
																				size={20}
																				strokeWidth={2}
																				color={'#2067FA'}
																			/>
																			<Center style={{ width: 20, height: 20, backgroundColor: '#19A2D0', borderRadius: 6 }}>
																				<Skeleton height={20} width={20} radius="md" />
																			</Center>
																			<div>
																				<Skeleton height={16} width={160} radius="md" />
																			</div>
																		</Group>
																		<Group align="center" spacing="sm" pr={15}>
																			<PencilMinus
																				size={24}
																				strokeWidth={2}
																				color={'#2067FA'}
																			/>
																			<X
																				size={24}
																				strokeWidth={2}
																				color={'#2067FA'}
																			/>
																			<ChevronDown
																				size={24}
																				strokeWidth={2}
																				color={'#2067FA'}
																			/>

																		</Group>
																	</Group>
																</Paper>
															</>
														)}
														{null !== courseList && Object.keys(courseList.h["sfwd-lessons"]).length < 1 && courseList.sections.length < 1 && (
															<>
																<Title order={6} align="center">{sprintf(__('%s has no content', 'wdm_instructor_role'), ir_fcb_loc.course_label)}</Title>
															</>
														)}
														{null !== courseList && null !== courseData && courseList.t["sfwd-lessons"].map((lesson) => (
															<>
																<RenderCourseSection sections={courseList.sections} lesson={lesson} />
																<Accordion.Item value={"ir-acc-" + lesson} className={"ir-acc-" + lesson}>
																	<Group className="ir-fed-lessons" align="center" position="apart">
																		<Group className="ir-lessons-left" align="center" position="apart">
																			<Group align="center" spacing="xs">
																				<GridDots
																					size={20}
																					strokeWidth={2}
																					color={'#2067FA'}
																				/>
																				<Center style={{ width: 20, height: 20, backgroundColor: '#010E56', borderRadius: 6 }}>
																					<LetterL
																						size={12}
																						strokeWidth={4}
																						color={'#ffffff'}
																					/>
																				</Center>
																				<Text fz="md" fw={600} c="#222222">{`${courseData && html_entity_decode(courseData[lesson].post_title)}`}</Text>
																				{null !== courseData[lesson].post_status && 'draft' === courseData[lesson].post_status && (
																					<Text className="ir-drafted" fw={400} color="#6F717C">{__('- Draft', 'wdm_instructor_role')}</Text>
																				)}
																				<input type="hidden" id={"ir-builder-element-" + lesson} className='ir-builder-lesson ir-builder-top-element' value={lesson} />
																			</Group>
																			<Group align="center" spacing="sm" pr={15} className="ir-br-2">
																				{null !== courseData[lesson] && courseData[lesson].is_sample && (
																					<Badge color="#2067FA" variant="outline" >
																						{sprintf(__('Sample %s', 'wdm_instructor_role'), ir_fcb_loc.lesson_label)}
																					</Badge>
																				)}
																				<EditModule label={ir_fcb_loc.lesson_label} course={course_id} lesson={lesson} courseData={courseData} setCourseData={setCourseData} instructorSettings={instructorSettings} review_data={review_data} setReviewData={setReviewData} />
																				<X onClick={() => confirmDelete(lesson, 'lesson')}
																					size={24}
																					strokeWidth={2}
																					color={'#2067FA'}
																				/>

																			</Group>
																		</Group>
																		<Accordion.Control className="ir-fed-lessons">

																		</Accordion.Control>
																	</Group>
																	<Accordion.Panel className="connect ir-lessons-panel">
																		<div className="ir-sortable-topics">
																			{Object.keys(courseList.h["sfwd-lessons"][lesson]["sfwd-topic"]).map((topicKeyName, i) => (
																				<Accordion.Item value={"ir-acc-" + topicKeyName} className={"ir-fed-topics ir-acc-" + topicKeyName}>
																					<Group className="ir-topics-acc-top" align="center" position="apart">
																						<Group className="ir-topics-left" align="center" spacing="xs">
																							<GridDots
																								size={20}
																								strokeWidth={2}
																								color={'#2067FA'}
																							/>
																							<Center style={{ width: 20, height: 20, backgroundColor: '#0D5994', borderRadius: 6 }}>
																								<LetterT
																									size={12}
																									strokeWidth={4}
																									color={'#ffffff'}
																								/>
																							</Center>
																							<Text fz="md" fw={600} c="#222222">{`${courseData && html_entity_decode(courseData[topicKeyName].post_title)}`}</Text>
																							{null !== courseData[topicKeyName].post_status && 'draft' === courseData[topicKeyName].post_status && (
																								<Text className="ir-drafted" fw={400} color="#6F717C">{__('- Draft', 'wdm_instructor_role')}</Text>
																							)}
																							<input id={"ir-builder-element-" + topicKeyName} className='ir-builder-lesson-topic' type="hidden" value={topicKeyName} />
																						</Group>
																						<Group align="center" spacing="sm" pr={15} className="ir-br-2">
																							<EditModule label={ir_fcb_loc.topic_label} course={course_id} lesson={lesson} topic={topicKeyName} courseData={courseData} setCourseData={setCourseData} instructorSettings={instructorSettings} review_data={review_data} setReviewData={setReviewData} />
																							<X onClick={() => confirmDelete(topicKeyName, 'topic', lesson)}
																								size={24}
																								strokeWidth={2}
																								color={'#2067FA'}
																							/>

																						</Group>
																						<Accordion.Control>

																						</Accordion.Control>
																					</Group>
																					<Accordion.Panel pb={20} className="ir-topics-panel">
																						<div className="ir-sortable-quizzes ir-topic-quizzes">
																							{Object.keys(courseList.h["sfwd-lessons"][lesson]["sfwd-topic"][topicKeyName]['sfwd-quiz']).map((quizKeyName, i) => (
																								<Paper className="ir-fed-quizzes" mb={16} radius="lg" p="md" withBorder>
																									<Group align="center" spacing="xs" position="apart">
																										<Group className="ir-quizzes-acc-top" align="center">
																											<Group align="center" spacing="xs">
																												<GridDots
																													size={20}
																													strokeWidth={2}
																													color={'#2067FA'}
																												/>
																												<Center style={{ width: 20, height: 20, backgroundColor: '#19A2D0', borderRadius: 6 }}>
																													<LetterQ
																														size={12}
																														strokeWidth={4}
																														color={'#ffffff'}
																													/>
																												</Center>
																												<div>
																													<Text fz="md" fw={600} c="#222222">{`${courseData && html_entity_decode(courseData[quizKeyName].post_title)}`}</Text>
																													<input id={"ir-builder-element-" + quizKeyName} className='ir-builder-quiz' type="hidden" value={quizKeyName} />
																												</div>
																												{null !== courseData[quizKeyName].post_status && 'draft' === courseData[quizKeyName].post_status && (
																													<Text className="ir-drafted" fw={400} color="#6F717C">{__('- Draft', 'wdm_instructor_role')}</Text>
																												)}
																											</Group>
																										</Group>
																										<Group align="center" spacing="xs">
																											<PencilMinus onClick={() => openQuizPage(quizKeyName)}
																												size={24}
																												strokeWidth={2}
																												color={'#2067FA'}
																											/>
																											{/* <QuizPopoverNotification /> */}
																											<X onClick={() => confirmDelete(quizKeyName, 'quiz')}
																												size={24}
																												strokeWidth={2}
																												color={'#2067FA'}
																											/>
																										</Group>
																									</Group>
																								</Paper>
																							))}
																						</div>
																						<Paper className="ir-dropzone-area dotted-border" radius="lg" p="lg" mb={15} withBorder>
																							<span className="dropzone-text">{sprintf(__('Drag and drop or add existing %s from the %s content library', 'wdm_instructor_role'), ir_fcb_loc.quizzes_label, ir_fcb_loc.course_label)}</span>
																						</Paper>
																						<AddQuiz setCourseData={setCourseData} courseData={courseData} setCourseList={setCourseList} courseList={courseList} course_id={course_id} lesson_id={lesson} topic_id={topicKeyName} class="ir-btn-outline" />
																					</Accordion.Panel>
																				</Accordion.Item>
																			))}
																		</div>
																		<div className="ir-sortable-quizzes ir-lesson-quizzes">
																			{Object.keys(courseList.h["sfwd-lessons"][lesson]['sfwd-quiz']).map((quizKeyName, i) => (
																				<Paper className="ir-fed-quizzes" mb={16} radius="lg" p="md" withBorder>
																					<Group align="center" spacing="xs" position="apart">
																						<Group className="ir-quizzes-acc-top" align="center" spacing="xs">
																							<Group align="center" spacing="xs">
																								<GridDots
																									size={20}
																									strokeWidth={2}
																									color={'#2067FA'}
																								/>
																								<Center style={{ width: 20, height: 20, backgroundColor: '#19A2D0', borderRadius: 6 }}>
																									<LetterQ
																										size={12}
																										strokeWidth={4}
																										color={'#ffffff'}
																									/>
																								</Center>
																								<div>
																									<Text fz="md" fw={600} c="#222222">{`${courseData && html_entity_decode(courseData[quizKeyName].post_title)}`}</Text>
																									<input id={"ir-builder-element-" + quizKeyName} className='ir-builder-quiz' type="hidden" value={quizKeyName} />
																								</div>
																								{null !== courseData[quizKeyName].post_status && 'draft' === courseData[quizKeyName].post_status && (
																									<Text className="ir-drafted" fw={400} color="#6F717C">{__('- Draft', 'wdm_instructor_role')}</Text>
																								)}
																							</Group>
																						</Group>
																						<Group align="center" spacing="xs">
																							<PencilMinus onClick={() => openQuizPage(quizKeyName)}
																								size={24}
																								strokeWidth={2}
																								color={'#2067FA'}
																							/>
																							{/* <QuizPopoverNotification /> */}
																							<X onClick={() => confirmDelete(quizKeyName, 'quiz')}
																								size={24}
																								strokeWidth={2}
																								color={'#2067FA'}
																							/>
																						</Group>
																					</Group>
																				</Paper>
																			))}
																		</div>

																		<Paper className="ir-dropzone-area dotted-border" radius="lg" p="lg" withBorder>
																			<span className="dropzone-text">{sprintf(__('Drag and drop or add existing %s and %s from the %s content library', 'wdm_instructor_role'), ir_fcb_loc.topics_label, ir_fcb_loc.quizzes_label, ir_fcb_loc.lower_course_label)}</span>
																		</Paper>
																		<Group position='center' mt={16}>
																			<TopicModal lesson_id={lesson} />
																		</Group>
																	</Accordion.Panel>
																</Accordion.Item>
															</>
														))}
														{null !== courseList && courseList.sections.map((section) => (
															<RenderLastCourseSections section={section} />
														))}
													</Accordion>
													<Paper className="ir-dropzone-area dotted-border" radius="lg" p="lg" withBorder>
														<span className="dropzone-text">{sprintf(__('Drag and drop or add existing %s from %s content library.', 'wdm_instructor_role'), ir_fcb_loc.lessons_label, ir_fcb_loc.lower_course_label)}</span>
													</Paper>
													<Group position='center' mt={30} mb={23}>
														<LessonModal />
														<AddSection setCourseList={setCourseList} courseList={courseList} />
													</Group>


												</Paper>
												<Space h="xl" />
												<Paper radius="xl" p="xl" className="ir-fed-builder ir-fed-final-quizzes">
													<Title size={32} order={2} className="ir-final-quiz-text">
														{sprintf(__('Final %s', 'wdm_instructor_role'), ir_fcb_loc.quizzes_label)}
														{/* <QuizPopoverNotification /> */}
													</Title>
													<Accordion>
														{null === courseList && (
															<>
																<Paper className="ir-fed-topics ir-skeleton" mb={16} radius="lg" p="md" withBorder>
																	<Group align="center" position="apart">
																		<Group align="center" spacing="xs">
																			<GridDots
																				size={20}
																				strokeWidth={2}
																				color={'#2067FA'}
																			/>
																			<Center style={{ width: 20, height: 20, backgroundColor: '#19A2D0', borderRadius: 6 }}>
																				<Skeleton height={20} width={20} radius="md" />
																			</Center>
																			<div>
																				<Skeleton height={16} width={120} radius="md" />
																			</div>
																		</Group>
																		<Group align="center" spacing="sm" pr={15}>
																			<PencilMinus
																				size={24}
																				strokeWidth={2}
																				color={'#2067FA'}
																			/>
																			<X
																				size={24}
																				strokeWidth={2}
																				color={'#2067FA'}
																			/>
																			<ChevronDown
																				size={24}
																				strokeWidth={2}
																				color={'#2067FA'}
																			/>

																		</Group>
																	</Group>
																</Paper>
																<Paper className="ir-fed-topics ir-skeleton" mb={16} radius="lg" p="md" withBorder>
																	<Group align="center" position="apart">
																		<Group align="center" spacing="xs">
																			<GridDots
																				size={20}
																				strokeWidth={2}
																				color={'#2067FA'}
																			/>
																			<Center style={{ width: 20, height: 20, backgroundColor: '#19A2D0', borderRadius: 6 }}>
																				<Skeleton height={20} width={20} radius="md" />
																			</Center>
																			<div>
																				<Skeleton height={16} width={120} radius="md" />
																			</div>
																		</Group>
																		<Group align="center" spacing="sm" pr={15}>
																			<PencilMinus
																				size={24}
																				strokeWidth={2}
																				color={'#2067FA'}
																			/>
																			<X
																				size={24}
																				strokeWidth={2}
																				color={'#2067FA'}
																			/>
																			<ChevronDown
																				size={24}
																				strokeWidth={2}
																				color={'#2067FA'}
																			/>

																		</Group>
																	</Group>
																</Paper>
															</>
														)}

														<div className="ir-sortable-quizzes ir-final-quizzes">
															{null !== courseList && null !== courseData && Object.keys(courseList.h["sfwd-quiz"]).map((quiz, i) => (
																<Paper className="ir-fed-quizzes" mb={16} radius="lg" p="md" withBorder>
																	<Group align="center" spacing="xs" position="apart">
																		<Group align="center" spacing="xs">
																			<GridDots
																				size={20}
																				strokeWidth={2}
																				color={'#2067FA'}
																			/>
																			<Center style={{ width: 20, height: 20, backgroundColor: '#19A2D0', borderRadius: 6 }}>
																				<LetterQ
																					size={12}
																					strokeWidth={4}
																					color={'#ffffff'}
																				/>
																			</Center>
																			<div>
																				<Text fz="md" fw={600} c="#222222">{`${courseData && html_entity_decode(courseData[quiz].post_title)}`}</Text>
																				<input id={"ir-builder-element-" + quiz} className='ir-builder-quiz' type="hidden" value={quiz} />
																			</div>
																			{null !== courseData[quiz].post_status && 'draft' === courseData[quiz].post_status && (
																				<Text className="ir-drafted" fw={400} color="#6F717C">{__('- Draft', 'wdm_instructor_role')}</Text>
																			)}
																		</Group>
																		<Group align="center" spacing="xs">
																			<PencilMinus onClick={() => openQuizPage(quiz)}
																				size={24}
																				strokeWidth={2}
																				color={'#2067FA'}
																			/>
																			{/* <QuizPopoverNotification /> */}
																			<X onClick={() => confirmDelete(quiz, 'quiz')}
																				size={24}
																				strokeWidth={2}
																				color={'#2067FA'}
																			/>
																		</Group>
																	</Group>
																</Paper>
															))}
														</div>
														<Title className="ir-no-quizzes" order={6} mb={20} align="center">{sprintf(__('%s has no final %s', 'wdm_instructor_role'), ir_fcb_loc.course_label, ir_fcb_loc.quizzes_label)}</Title>
														<Paper className="ir-dropzone-area dotted-border" radius="lg" p="lg" withBorder mb={20}>
															<span className="dropzone-text">{sprintf(__('Drag and drop or add %s from %s library', 'wdm_instructor_role'), ir_fcb_loc.quizzes_label, ir_fcb_loc.lower_course_label)}</span>
														</Paper>
														<AddQuiz setCourseData={setCourseData} courseData={courseData} setCourseList={setCourseList} courseList={courseList} course_id={course_id} class="ir-btn-filled" />
													</Accordion>
												</Paper>
											</div>
										</Grid.Col>
										<Grid.Col span={4}>
											<Paper radius="xl" p="xl">
												<Title size={32} order={2}>{sprintf(__('%s Content Library', 'wdm_instructor_role'), ir_fcb_loc.course_label)}</Title>
												<Text size="md" color="#555552" mb={16}>{sprintf(__('Drag and drop or selectively add previously made %s, %s and %s.', 'wdm_instructor_role'), ir_fcb_loc.lessons_label, ir_fcb_loc.topics_label, ir_fcb_loc.quizzes_label)}</Text>
												<Accordion variant='separated' radius={'md'} defaultValue={''} styles={{
													item: {
														// styles added to all items
														backgroundColor: '#EFF0F7',
														borderRadius: '15px',
														marginBottom: "16px",
														// styles added to expanded item
														'&[data-active]': {
															backgroundColor: '#EFF0F7',
														},
													},

													chevron: {
														width: "28px",
														marginLeft: "10px",
														// styles added to chevron when it should rotate
														'&[data-rotate]': {
															transform: 'rotate(-90deg)',
														},
													},
												}}>
													<Accordion.Item value="ir_other_lessons">
														<Accordion.Control>
															<Group align="center" spacing="xs">
																<Center style={{ width: 25, height: 25, backgroundColor: '#010E56', borderRadius: 6 }}>
																	<LetterL
																		size={14}
																		strokeWidth={4}
																		color={'#ffffff'}
																	/>
																</Center>
																<Text>{sprintf(__('%s', 'wdm_instructor_role'), ir_fcb_loc.lessons_label)}</Text>
																{true === lessons_loading && (<Loader variant='dots' size='sm' />)}
															</Group>
														</Accordion.Control>
														<Accordion.Panel className="ir-lessons-checkboxes">
															<TextInput onKeyUp={(e) => searchModule(e, ".ir-lessons-checkboxes")} className="ir-serachbox" placeholder="Search Lessons" radius="md" icon={<Search size={18} />} />
															<Group align="center" spacing={0} mt={15} mb={15}>
																<Button size="xs" onClick={(e) => showRecentContent(e, 'lessons')} className="ir-btn-outline-black ir-btn-grey ir-recent-lessons">{__('Recent', 'wdm_instructor_role')}</Button>
																<Button onClick={() => sortAlphabetically('lessons')} size="xs" className="ir-btn-outline-black ir-view-all-lessons">{__('View All', 'wdm_instructor_role')}</Button>
															</Group>
															{0 === Object.keys(lessonList).length && (
																<Text>{sprintf(__('There are no existing %s', 'wdm_instructor_role'), ir_fcb_loc.lessons_label)}</Text>
															)}
															<Checkbox.Group className="ir-draggable" size="md" fw={600}
																orientation="vertical"
																value={selectedLessons}
																onChange={(value) => {
																	setSelectedLessons(value);
																	checkIfchecked('lesson');
																}}
																defaultValue={[]}
															>
																{lessonList && (
																	Object.keys(lessonList).map((lessonId, index) => (
																		<Group className="ir-chk-wrap" position="apart" align="top" noWrap='true' id={"ir-lib-lesson-" + lessonId} dataId={lessonId} data-timestamp={lessonList[lessonId].timestamp}>
																			<Checkbox fw={600} value={lessonId} label={html_entity_decode(lessonList[lessonId].title)} />
																			{lessonList[lessonId].status === 'draft' && (
																				<Text size={14} c="#6F717C" fw={400}>{__('- Draft', 'wdm_instructor_role')}</Text>
																			)}
																			<Button size="xs" className="ir-btn-add ir-btn-filled" onClick={() => addLessonToBuilder(lessonId)}>
																				{__('Add', 'wdm_instructor_role')}
																			</Button>
																		</Group>
																	))
																)}
															</Checkbox.Group>
															{0 !== Object.keys(lessonList).length && (
																<Group mt={20} position="right">
																	<Button className="ir-btn-filled" onClick={() => addSelectedLessonsToBuilder()} disabled={!checkedLessons}>{__('Add Selected', 'wdm_instructor_role')}</Button>
																</Group>
															)}
														</Accordion.Panel>
													</Accordion.Item>
													<Accordion.Item value="ir_other_topics">
														<Accordion.Control>
															<Group align="center" spacing="xs">
																<Center style={{ width: 25, height: 25, backgroundColor: '#0D5994', borderRadius: 6 }}>
																	<LetterT
																		size={14}
																		strokeWidth={4}
																		color={'#ffffff'}
																	/>
																</Center>
																<Text>{sprintf(__('%s', 'wdm_instructor_role'), ir_fcb_loc.topics_label)}</Text>
																{true === topics_loading && (<Loader variant='dots' size='sm' />)}
															</Group>
														</Accordion.Control>
														<Accordion.Panel className="ir-topics-checkboxes">
															<TextInput onKeyUp={(e) => searchModule(e, ".ir-topics-checkboxes")} className="ir-serachbox" placeholder="Search Topics" radius="md" icon={<Search size={18} />} />
															<Group align="center" spacing={0} mt={15} mb={15}>
																<Button onClick={(e) => showRecentContent(e, 'topics')} size="xs" className="ir-btn-outline-black ir-btn-grey ir-recent-topics">{__('Recent', 'wdm_instructor_role')}</Button>
																<Button onClick={() => sortAlphabetically('topics')} size="xs" className="ir-btn-outline-black ir-view-all-topics">{__('View All', 'wdm_instructor_role')}</Button>
															</Group>
															{0 === Object.keys(topicList).length && (
																<Text>{sprintf(__('There are no existing %s', 'wdm_instructor_role'), ir_fcb_loc.topics_label)}</Text>
															)}
															<Checkbox.Group
																className="ir-draggable"
																orientation="vertical"
																value={selectedTopics}
																onChange={(value) => {
																	setSelectedTopics(value);
																	checkIfchecked('topic');
																}}
																defaultValue={[]}
															>
																{topicList && (
																	Object.keys(topicList).map((topicId, i) => (
																		<Group className="ir-chk-wrap" position="apart" align="top" noWrap='true' id={"ir-lib-topic-" + topicId} dataId={topicId} data-timestamp={topicList[topicId].timestamp}>
																			<Checkbox value={topicId} label={html_entity_decode(topicList[topicId].title)} />
																			{topicList[topicId].status === 'draft' && (
																				<Text size={14} c="#6F717C" fw={400}>{__('- Draft', 'wdm_instructor_role')}</Text>
																			)}
																			<Button size="xs" className="ir-btn-add ir-btn-filled" onClick={() => addTopicToBuilder(topicId)}>
																				{__('Add', 'wdm_instructor_role')}
																			</Button>
																		</Group>
																	))
																)}
															</Checkbox.Group>
															{0 !== Object.keys(topicList).length && (
																<Group mt={20} position="right">
																	<Button className="ir-btn-filled" onClick={() => addSelectedTopicsToBuilder()} disabled={!checkedTopics}>{__('Add Selected', 'wdm_instructor_role')}</Button>
																</Group>
															)}
														</Accordion.Panel>
													</Accordion.Item>
													<Accordion.Item value="ir_other_quizzes">
														<Accordion.Control>
															<Group align="center" spacing="xs">
																<Center style={{ width: 25, height: 25, backgroundColor: '#19A2D0', borderRadius: 6 }}>
																	<LetterQ
																		size={14}
																		strokeWidth={4}
																		color={'#ffffff'}
																	/>
																</Center>
																<Text>{ir_fcb_loc.quizzes_label}</Text>
																{/* <QuizPopoverNotification /> */}
																{true === quiz_loading && (<Loader variant='dots' />)}
															</Group>
														</Accordion.Control>
														<Accordion.Panel className="ir-quizzes-checkboxes">
															<TextInput onKeyUp={(e) => searchModule(e, ".ir-quizzes-checkboxes")} className="ir-serachbox" placeholder="Search Quizzes" radius="md" icon={<Search size={18} />} />
															<Group align="center" spacing={0} mt={15} mb={15}>
																<Button onClick={(e) => showRecentContent(e, 'quizzes')} size="xs" className="ir-btn-outline-black ir-btn-grey ir-recent-quizzes">{__('Recent', 'wdm_instructor_role')}</Button>
																<Button onClick={() => sortAlphabetically('quizzes')} size="xs" className="ir-btn-outline-black ir-view-all-quizzes">{__('View All', 'wdm_instructor_role')}</Button>
															</Group>
															{0 === Object.keys(quizList).length && (
																<Text>{sprintf(__('There are no existing %s', 'wdm_instructor_role'), ir_fcb_loc.quizzes_label)}</Text>
															)}
															<Checkbox.Group
																className="ir-draggable"
																orientation="vertical"
																value={selectedQuizzes}
																onChange={(value) => {
																	setSelectedQuizzes(value);
																	checkIfchecked('quiz');
																}}
																defaultValue={[]}
															>
																{quizList && (
																	Object.keys(quizList).map((quizId, i) => (
																		<Group className="ir-chk-wrap" position="apart" align="top" noWrap='true' id={"ir-lib-quiz-" + quizId} dataId={quizId} data-timestamp={quizList[quizId].timestamp}>
																			<Checkbox value={quizId} label={html_entity_decode(quizList[quizId].title)} />
																			{quizList[quizId].status === 'draft' && (
																				<Text size={14} c="#6F717C" fw={400}>{__('- Draft', 'wdm_instructor_role')}</Text>
																			)}
																			<Button size="xs" className="ir-btn-add ir-btn-filled" onClick={() => addQuizToBuilder(quizId)}>
																				{__('Add', 'wdm_instructor_role')}
																			</Button>
																		</Group>

																	))
																)}
															</Checkbox.Group>
															{0 !== Object.keys(quizList).length && (
																<Group mt={20} position="right">
																	<Button className="ir-btn-filled" onClick={() => addSelectedQuizzesToBuilder()} disabled={!checkedQuizzes}>{__('Add Selected', 'wdm_instructor_role')}</Button>
																</Group>
															)}
														</Accordion.Panel>
													</Accordion.Item>
												</Accordion>
											</Paper>
										</Grid.Col>
									</Grid>
								</Tabs.Panel>
								<Tabs.Panel className="ir-main-panel" value="ir-settings">
									<CourseSettings form={courseform} course_id={course_id} instructorSettings={instructorSettings} />
								</Tabs.Panel>
								<Tabs.Panel className="ir-main-panel" value="ir-group-settings">
									<CourseGroups form={courseform} course_id={course_id} />
								</Tabs.Panel>
							</Tabs>
						</Group>
					</div>
				</ModalsProvider>
			</NotificationsProvider>
		</MantineProvider>
	);
}

document.addEventListener("DOMContentLoaded", function (event) {

	let elem = document.getElementsByClassName('wisdm-mui-poc');
	if (elem.length > 0) {
		ReactDOM.render(React.createElement(save), elem[0]);
	}

	setTimeout(function () {
		jQuery('.ir-fed-container').find('.mantine-Accordion-item').first().addClass('ir-active');
		jQuery(document).on('click', '.ir-fed-container > .mantine-Accordion-item', function () {
			jQuery('.ir-fed-container > .mantine-Accordion-item').removeClass('ir-active');
			jQuery(this).addClass('ir-active');
		})
		jQuery(document).on('click', '.ir-expand-all', function () {
			jQuery('.ir-fed-course-content [aria-expanded="false"]').trigger('click');
		})

		jQuery(document).on('click', '.ir-collapse-all', function () {
			jQuery('.ir-fed-course-content [aria-expanded="true"]').trigger('click');
		})

		var didScroll;
		var lastScrollTop = 0;
		var delta = 5;
		var navbarHeight = jQuery('.ir-header-top').outerHeight();

		jQuery(window).scroll(function (event) {
			didScroll = true;
		});

		setInterval(function () {
			if (didScroll) {
				hasScrolled();
				didScroll = false;
			}
		}, 250);

		function hasScrolled() {
			var st = jQuery(window).scrollTop();
			// Make sure they scroll more than delta
			if (Math.abs(lastScrollTop - st) <= delta)
				return;

			// If they scrolled down and are past the navbar, add class .nav-up.
			// This is necessary so you never see what is "behind" the navbar.
			if (st > lastScrollTop && st > navbarHeight) {
				// Scroll Down
				jQuery('.wisdm-mui-poc').removeClass('nav-down').addClass('nav-up');
			} else {
				// Scroll Up
				if (st + jQuery(window).height() < jQuery(document).height()) {
					jQuery('.wisdm-mui-poc').removeClass('nav-up').addClass('nav-down');
				}
			}
			lastScrollTop = st;
		}
	}, 5000)
});
